@@ -1,4 +1,23 @@
-
+/* WINGs.h
+ *
+ *  Copyright (c) 1998 scottc
+ *  Copyright (c) 1999-2004 Dan Pascu
+ *  Copyright (c) 1999-2001 Alfredo K. Kojima
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
 
 #ifndef _WINGS_H_
 #define _WINGS_H_
@@ -7,7 +26,7 @@
 #include <WINGs/WUtil.h>
 #include <X11/Xlib.h>
 
-#define WINGS_H_VERSION  20040406
+#define WINGS_H_VERSION  20041030
 
 
 #ifdef __cplusplus
@@ -590,7 +609,7 @@ typedef struct W_DragDestinationProcs {
 } WMDragDestinationProcs;
 
 
-/* ...................................................................... */
+/* ---[ WINGs/wmisc.c ]--------------------------------------------------- */
 
 
 WMPoint wmkpoint(int x, int y);
@@ -604,20 +623,21 @@ WMRect wmkrect(int x, int y, unsigned int width, unsigned int height);
 #define wmkpoint(x, y)         (WMPoint){(x), (y)}
 #endif
 
-/* ....................................................................... */
+/* ---[ WINGs/wapplication.c ]-------------------------------------------- */
 
 
 
-void WMInitializeApplication(char *applicationName, int *argc, char **argv);
+void WMInitializeApplication(const char *applicationName, int *argc, char **argv);
 
-void WMSetResourcePath(char *path);
+void WMSetResourcePath(const char *path);
 
 /* don't free the returned string */
-char* WMGetApplicationName();
+char* WMGetApplicationName(void);
 
 /* Try to locate resource file. ext may be NULL */
-char* WMPathForResourceOfType(char *resource, char *ext);
+char* WMPathForResourceOfType(const char *resource, const char *ext);
 
+/* ---[ WINGs/widgets.c ]------------------------------------------------- */
 
 WMScreen* WMOpenScreen(const char *display);
 
@@ -640,7 +660,9 @@ Display* WMScreenDisplay(WMScreen *scr);
 
 int WMScreenDepth(WMScreen *scr);
 
+void WMSetFocusToWidget(WMWidget *widget);
 
+/* ---[ WINGs/wappresource.c ]-------------------------------------------- */
 
 void WMSetApplicationIconImage(WMScreen *app, RImage *image);
 
@@ -651,11 +673,11 @@ void WMSetApplicationIconPixmap(WMScreen *app, WMPixmap *icon);
 WMPixmap* WMGetApplicationIconPixmap(WMScreen *app);
 
 /* If color==NULL it will use the default color for panels: ae/aa/ae */
-WMPixmap* WMCreateApplicationIconBlendedPixmap(WMScreen *scr, RColor *color);
+WMPixmap* WMCreateApplicationIconBlendedPixmap(WMScreen *scr, const RColor *color);
 
 void WMSetApplicationIconWindow(WMScreen *scr, Window window);
 
-void WMSetFocusToWidget(WMWidget *widget);
+/* ---[ WINGs/wevent.c ]-------------------------------------------------- */
 
 WMEventHook* WMHookEventHandler(WMEventHook *handler);
 
@@ -677,8 +699,12 @@ void WMNextEvent(Display *dpy, XEvent *event);
 
 void WMMaskEvent(Display *dpy, long mask, XEvent *event);
 
+void WMSetViewNextResponder(WMView *view, WMView *responder);
 
-/* ....................................................................... */
+void WMRelayToNextResponder(WMView *view, XEvent *event);
+
+
+/* ---[ WINGs/selection.c ]----------------------------------------------- */
 
 
 Bool WMCreateSelectionHandler(WMView *view, Atom selection, Time timestamp,
@@ -693,7 +719,7 @@ Bool WMRequestSelection(WMView *view, Atom selection, Atom target,
 
 extern char *WMSelectionOwnerDidChangeNotification;
 
-/* ....................................................................... */
+/* ---[ WINGs/dragcommon.c ]---------------------------------------------- */
 
 WMArray* WMCreateDragOperationArray(int initialSize);
 
@@ -704,6 +730,8 @@ WMDragOperationType WMGetDragOperationItemType(WMDragOperationItem* item);
 
 char* WMGetDragOperationItemText(WMDragOperationItem* item);
 
+/* ---[ WINGs/dragsource.c ]---------------------------------------------- */
+
 void WMSetViewDragImage(WMView* view, WMPixmap *dragImage);
 
 void WMReleaseViewDragImage(WMView* view);
@@ -724,19 +752,19 @@ void WMSetViewDraggable(WMView *view, WMDragSourceProcs *procs, WMPixmap *dragIm
 
 void WMUnsetViewDraggable(WMView *view);
 
+/* ---[ WINGs/dragdestination.c ]----------------------------------------- */
+
 void WMRegisterViewForDraggedTypes(WMView *view, WMArray *acceptedTypes);
 
 void WMUnregisterViewDraggedTypes(WMView *view);
 
 void WMSetViewDragDestinationProcs(WMView *view, WMDragDestinationProcs *procs);
 
-/* ....................................................................... */
+/* ---[ WINGs/wfont.c ]--------------------------------------------------- */
 
 Bool WMIsAntialiasingEnabled(WMScreen *scrPtr);
 
-/* ....................................................................... */
-
-WMFont* WMCreateFont(WMScreen *scrPtr, char *fontName);
+WMFont* WMCreateFont(WMScreen *scrPtr, const char *fontName);
 
 WMFont* WMCopyFontWithStyle(WMScreen *scrPtr, WMFont *font, WMFontStyle style);
 
@@ -760,7 +788,16 @@ WMFont* WMSystemFontOfSize(WMScreen *scrPtr, int size);
 
 WMFont* WMBoldSystemFontOfSize(WMScreen *scrPtr, int size);
 
-/* ....................................................................... */
+void WMDrawString(WMScreen *scr, Drawable d, WMColor *color, WMFont *font,
+                  int x, int y, const char *text, int length);
+
+void WMDrawImageString(WMScreen *scr, Drawable d, WMColor *color,
+                       WMColor *background, WMFont *font, int x, int y,
+                       const char *text, int length);
+
+int WMWidthOfString(WMFont *font, const char *text, int length);
+
+/* ---[ WINGs/wpixmap.c ]------------------------------------------------- */
 
 WMPixmap* WMRetainPixmap(WMPixmap *pixmap);
 
@@ -780,13 +817,13 @@ WMPixmap* WMCreatePixmapFromXPMData(WMScreen *scrPtr, char **data);
 
 WMSize WMGetPixmapSize(WMPixmap *pixmap);
 
-WMPixmap* WMCreatePixmapFromFile(WMScreen *scrPtr, char *fileName);
+WMPixmap* WMCreatePixmapFromFile(WMScreen *scrPtr, const char *fileName);
 
 WMPixmap* WMCreateBlendedPixmapFromRImage(WMScreen *scrPtr, RImage *image,
-                                          RColor *color);
+                                          const RColor *color);
 
-WMPixmap* WMCreateBlendedPixmapFromFile(WMScreen *scrPtr, char *fileName,
-                                        RColor *color);
+WMPixmap* WMCreateBlendedPixmapFromFile(WMScreen *scrPtr, const char *fileName,
+                                        const RColor *color);
 
 void WMDrawPixmap(WMPixmap *pixmap, Drawable d, int x, int y);
 
@@ -796,7 +833,7 @@ Pixmap WMGetPixmapMaskXID(WMPixmap *pixmap);
 
 WMPixmap* WMGetSystemPixmap(WMScreen *scr, int image);
 
-/* ....................................................................... */
+/* ---[ WINGs/wcolor.c ]-------------------------------------------------- */
 
 
 WMColor* WMDarkGrayColor(WMScreen *scr);
@@ -828,7 +865,7 @@ WMColor* WMCreateRGBAColor(WMScreen *scr, unsigned short red,
                            unsigned short green, unsigned short blue,
                            unsigned short alpha, Bool exact);
 
-WMColor* WMCreateNamedColor(WMScreen *scr, char *name, Bool exact);
+WMColor* WMCreateNamedColor(WMScreen *scr, const char *name, Bool exact);
 
 RColor WMGetRColorFromColor(WMColor *color);
 
@@ -844,21 +881,7 @@ unsigned short WMGetColorAlpha(WMColor *color);
 
 char* WMGetColorRGBDescription(WMColor *color);
 
-/* ....................................................................... */
-
-
-void WMDrawString(WMScreen *scr, Drawable d, WMColor *color, WMFont *font,
-                  int x, int y, char *text, int length);
-
-void WMDrawImageString(WMScreen *scr, Drawable d, WMColor *color,
-                       WMColor *background, WMFont *font, int x, int y,
-                       char *text, int length);
-
-int WMWidthOfString(WMFont *font, char *text, int length);
-
-
-
-/* ....................................................................... */
+/* ---[ WINGs/widgets.c ]------------------------------------------------- */
 
 WMScreen* WMWidgetScreen(WMWidget *w);
 
@@ -904,10 +927,12 @@ unsigned int WMWidgetHeight(WMWidget *w);
 
 Window WMWidgetXID(WMWidget *w);
 
-Window WMViewXID(WMView *view);
-
 void WMRedisplayWidget(WMWidget *w);
 
+/* ---[ WINGs/wview.c ]--------------------------------------------------- */
+
+Window WMViewXID(WMView *view);
+
 void WMSetViewNotifySizeChanges(WMView *view, Bool flag);
 
 void WMSetViewExpandsToParent(WMView *view, int topOffs, int leftOffs,
@@ -921,10 +946,6 @@ WMPoint WMGetViewScreenPosition(WMView *view);
 
 WMWidget* WMWidgetOfView(WMView *view);
 
-void WMSetViewNextResponder(WMView *view, WMView *responder);
-
-void WMRelayToNextResponder(WMView *view, XEvent *event);
-
 /* notifications */
 extern char *WMViewSizeDidChangeNotification;
 
@@ -932,10 +953,9 @@ extern char *WMViewFocusDidChangeNotification;
 
 extern char *WMViewRealizedNotification;
 
+/* ---[ WINGs/wballoon.c ]------------------------------------------------ */
 
-/* ....................................................................... */
-
-void WMSetBalloonTextForView(char *text, WMView *view);
+void WMSetBalloonTextForView(const char *text, WMView *view);
 
 void WMSetBalloonTextAlignment(WMScreen *scr, WMAlignment alignment);
 
@@ -948,22 +968,22 @@ void WMSetBalloonDelay(WMScreen *scr, int delay);
 void WMSetBalloonEnabled(WMScreen *scr, Bool flag);
 
 
-/* ....................................................................... */
+/* ---[ WINGs/wwindow.c ]------------------------------------------------- */
 
-WMWindow* WMCreateWindow(WMScreen *screen, char *name);
+WMWindow* WMCreateWindow(WMScreen *screen, const char *name);
 
-WMWindow* WMCreateWindowWithStyle(WMScreen *screen, char *name, int style);
+WMWindow* WMCreateWindowWithStyle(WMScreen *screen, const char *name, int style);
 
-WMWindow* WMCreatePanelWithStyleForWindow(WMWindow *owner, char *name,
+WMWindow* WMCreatePanelWithStyleForWindow(WMWindow *owner, const char *name,
                                           int style);
 
-WMWindow* WMCreatePanelForWindow(WMWindow *owner, char *name);
+WMWindow* WMCreatePanelForWindow(WMWindow *owner, const char *name);
 
 void WMChangePanelOwner(WMWindow *win, WMWindow *newOwner);
 
-void WMSetWindowTitle(WMWindow *wPtr, char *title);
+void WMSetWindowTitle(WMWindow *wPtr, const char *title);
 
-void WMSetWindowMiniwindowTitle(WMWindow *win, char *title);
+void WMSetWindowMiniwindowTitle(WMWindow *win, const char *title);
 
 void WMSetWindowMiniwindowImage(WMWindow *win, RImage *image);
 
@@ -992,7 +1012,7 @@ void WMSetWindowDocumentEdited(WMWindow *win, Bool flag);
 
 void WMCloseWindow(WMWindow *win);
 
-/* ....................................................................... */
+/* ---[ WINGs/wbutton.c ]------------------------------------------------- */
 
 void WMSetButtonAction(WMButton *bPtr, WMAction *action, void *clientData);
 
@@ -1024,9 +1044,9 @@ void WMSetButtonFont(WMButton *bPtr, WMFont *font);
 
 void WMSetButtonTextAlignment(WMButton *bPtr, WMAlignment alignment);
 
-void WMSetButtonText(WMButton *bPtr, char *text);
+void WMSetButtonText(WMButton *bPtr, const char *text);
 
-void WMSetButtonAltText(WMButton *bPtr, char *text);
+void WMSetButtonAltText(WMButton *bPtr, const char *text);
 
 void WMSetButtonTextColor(WMButton *bPtr, WMColor *color);
 
@@ -1056,7 +1076,7 @@ void WMSetButtonContinuous(WMButton *bPtr, Bool flag);
 
 void WMSetButtonPeriodicDelay(WMButton *bPtr, float delay, float interval);
 
-/* ....................................................................... */
+/* ---[ WINGs/wlabel.c ]-------------------------------------------------- */
 
 WMLabel* WMCreateLabel(WMWidget *parent);
 
@@ -1074,7 +1094,7 @@ void WMSetLabelTextAlignment(WMLabel *lPtr, WMAlignment alignment);
 
 void WMSetLabelRelief(WMLabel *lPtr, WMReliefType relief);
 
-void WMSetLabelText(WMLabel *lPtr, char *text);
+void WMSetLabelText(WMLabel *lPtr, const char *text);
 
 WMFont* WMGetLabelFont(WMLabel *lPtr);
 
@@ -1082,7 +1102,7 @@ void WMSetLabelFont(WMLabel *lPtr, WMFont *font);
 
 void WMSetLabelTextColor(WMLabel *lPtr, WMColor *color);
 
-/* ....................................................................... */
+/* ---[ WINGs/wframe.c ]-------------------------------------------------- */
 
 WMFrame* WMCreateFrame(WMWidget *parent);
 
@@ -1090,20 +1110,20 @@ void WMSetFrameTitlePosition(WMFrame *fPtr, WMTitlePosition position);
 
 void WMSetFrameRelief(WMFrame *fPtr, WMReliefType relief);
 
-void WMSetFrameTitle(WMFrame *fPtr, char *title);
+void WMSetFrameTitle(WMFrame *fPtr, const char *title);
 
-/* ....................................................................... */
+/* ---[ WINGs/wtextfield.c ]---------------------------------------------- */
 
 WMTextField* WMCreateTextField(WMWidget *parent);
 
-void WMInsertTextFieldText(WMTextField *tPtr, char *text, int position);
+void WMInsertTextFieldText(WMTextField *tPtr, const char *text, int position);
 
 void WMDeleteTextFieldRange(WMTextField *tPtr, WMRange range);
 
 /* you can free the returned string */
 char* WMGetTextFieldText(WMTextField *tPtr);
 
-void WMSetTextFieldText(WMTextField *tPtr, char *text);
+void WMSetTextFieldText(WMTextField *tPtr, const char *text);
 
 void WMSetTextFieldAlignment(WMTextField *tPtr, WMAlignment alignment);
 
@@ -1125,6 +1145,8 @@ void WMSelectTextFieldRange(WMTextField *tPtr, WMRange range);
 
 void WMSetTextFieldCursorPosition(WMTextField *tPtr, unsigned int position);
 
+unsigned WMGetTextFieldCursorPosition(WMTextField *tPtr);
+
 void WMSetTextFieldNextTextField(WMTextField *tPtr, WMTextField *next);
 
 void WMSetTextFieldPrevTextField(WMTextField *tPtr, WMTextField *prev);
@@ -1137,7 +1159,7 @@ extern char *WMTextDidChangeNotification;
 extern char *WMTextDidBeginEditingNotification;
 extern char *WMTextDidEndEditingNotification;
 
-/* ....................................................................... */
+/* ---[ WINGs/wscroller.c ]----------------------------------------------- */
 
 WMScroller* WMCreateScroller(WMWidget *parent);
 
@@ -1157,7 +1179,7 @@ void WMSetScrollerArrowsPosition(WMScroller *sPtr,
 
 extern char *WMScrollerDidScrollNotification;
 
-/* ....................................................................... */
+/* ---[ WINGs/wlist.c ]--------------------------------------------------- */
 
 WMList* WMCreateList(WMWidget *parent);
 
@@ -1167,13 +1189,13 @@ void WMSetListAllowEmptySelection(WMList *lPtr, Bool flag);
 
 #define WMAddListItem(lPtr, text) WMInsertListItem((lPtr), -1, (text))
 
-WMListItem* WMInsertListItem(WMList *lPtr, int row, char *text);
+WMListItem* WMInsertListItem(WMList *lPtr, int row, const char *text);
 
 void WMSortListItems(WMList *lPtr);
 
 void WMSortListItemsWithComparer(WMList *lPtr, WMCompareDataProc *func);
 
-int WMFindRowOfListItemWithTitle(WMList *lPtr, char *title);
+int WMFindRowOfListItemWithTitle(WMList *lPtr, const char *title);
 
 WMListItem* WMGetListItem(WMList *lPtr, int row);
 
@@ -1237,7 +1259,7 @@ Bool WMListAllowsEmptySelection(WMList *lPtr);
 extern char *WMListDidScrollNotification;
 extern char *WMListSelectionDidChangeNotification;
 
-/* ....................................................................... */
+/* ---[ WINGs/wbrowser.c ]------------------------------------------------ */
 
 WMBrowser* WMCreateBrowser(WMWidget *parent);
 
@@ -1245,7 +1267,7 @@ void WMSetBrowserAllowMultipleSelection(WMBrowser *bPtr, Bool flag);
 
 void WMSetBrowserAllowEmptySelection(WMBrowser *bPtr, Bool flag);
 
-void WMSetBrowserPathSeparator(WMBrowser *bPtr, char *separator);
+void WMSetBrowserPathSeparator(WMBrowser *bPtr, const char *separator);
 
 void WMSetBrowserTitled(WMBrowser *bPtr, Bool flag);
 
@@ -1257,9 +1279,9 @@ void WMRemoveBrowserItem(WMBrowser *bPtr, int column, int row);
 
 void WMSetBrowserMaxVisibleColumns(WMBrowser *bPtr, int columns);
 
-void WMSetBrowserColumnTitle(WMBrowser *bPtr, int column, char *title);
+void WMSetBrowserColumnTitle(WMBrowser *bPtr, int column, const char *title);
 
-WMListItem* WMInsertBrowserItem(WMBrowser *bPtr, int column, int row, char *text, Bool isBranch);
+WMListItem* WMInsertBrowserItem(WMBrowser *bPtr, int column, int row, const char *text, Bool isBranch);
 
 void WMSortBrowserColumn(WMBrowser *bPtr, int column);
 
@@ -1305,7 +1327,7 @@ Bool WMBrowserAllowsEmptySelection(WMBrowser *bPtr);
 
 void WMSetBrowserHasScroller(WMBrowser *bPtr, int hasScroller);
 
-/* ....................................................................... */
+/* ---[ WINGs/wmenuitem.c ]----------------------------------------------- */
 
 
 Bool WMMenuItemIsSeparator(WMMenuItem *item);
@@ -1322,7 +1344,7 @@ char* WMGetMenuItemShortcut(WMMenuItem *item);
 
 unsigned WMGetMenuItemShortcutModifierMask(WMMenuItem *item);
 
-void WMSetMenuItemShortcut(WMMenuItem *item, char *shortcut);
+void WMSetMenuItemShortcut(WMMenuItem *item, const char *shortcut);
 
 void WMSetMenuItemShortcutModifierMask(WMMenuItem *item, unsigned mask);
 
@@ -1336,7 +1358,7 @@ WMAction* WMGetMenuItemAction(WMMenuItem *item);
 
 void* WMGetMenuItemData(WMMenuItem *item);
 
-void WMSetMenuItemTitle(WMMenuItem *item, char *title);
+void WMSetMenuItemTitle(WMMenuItem *item, const char *title);
 
 char* WMGetMenuItemTitle(WMMenuItem *item);
 
@@ -1368,7 +1390,7 @@ WMMenu* WMGetMenuItemSubmenu(WMMenuItem *item);
 Bool WMGetMenuItemHasSubmenu(WMMenuItem *item);
 */
 
-/* ....................................................................... */
+/* ---[ WINGs/wpopupbutton.c ]-------------------------------------------- */
 
 WMPopUpButton* WMCreatePopUpButton(WMWidget *parent);
 
@@ -1377,10 +1399,10 @@ void WMSetPopUpButtonAction(WMPopUpButton *sPtr, WMAction *action,
 
 void WMSetPopUpButtonPullsDown(WMPopUpButton *bPtr, Bool flag);
 
-WMMenuItem* WMAddPopUpButtonItem(WMPopUpButton *bPtr, char *title);
+WMMenuItem* WMAddPopUpButtonItem(WMPopUpButton *bPtr, const char *title);
 
 WMMenuItem* WMInsertPopUpButtonItem(WMPopUpButton *bPtr, int index,
-                                    char *title);
+                                    const char *title);
 
 void WMRemovePopUpButtonItem(WMPopUpButton *bPtr, int index);
 
@@ -1392,7 +1414,7 @@ void WMSetPopUpButtonSelectedItem(WMPopUpButton *bPtr, int index);
 
 int WMGetPopUpButtonSelectedItem(WMPopUpButton *bPtr);
 
-void WMSetPopUpButtonText(WMPopUpButton *bPtr, char *text);
+void WMSetPopUpButtonText(WMPopUpButton *bPtr, const char *text);
 
 /* don't free the returned data */
 char* WMGetPopUpButtonItem(WMPopUpButton *bPtr, int index);
@@ -1405,24 +1427,7 @@ void WMSetPopUpButtonEnabled(WMPopUpButton *bPtr, Bool flag);
 
 Bool WMGetPopUpButtonEnabled(WMPopUpButton *bPtr);
 
-/* ....................................................................... */
-
-WMProgressIndicator* WMCreateProgressIndicator(WMWidget *parent);
-
-void WMSetProgressIndicatorMinValue(WMProgressIndicator *progressindicator, int value);
-
-void WMSetProgressIndicatorMaxValue(WMProgressIndicator *progressindicator, int value);
-
-void WMSetProgressIndicatorValue(WMProgressIndicator *progressindicator, int value);
-
-int WMGetProgressIndicatorMinValue(WMProgressIndicator *progressindicator);
-
-int WMGetProgressIndicatorMaxValue(WMProgressIndicator *progressindicator);
-
-int WMGetProgressIndicatorValue(WMProgressIndicator *progressindicator);
-
-
-/* ....................................................................... */
+/* ---[ WINGs/wcolorpanel.c ]--------------------------------------------- */
 
 WMColorPanel* WMGetColorPanel(WMScreen *scrPtr);
 
@@ -1442,7 +1447,7 @@ void WMSetColorPanelAction(WMColorPanel *panel, WMAction2 *action, void *data);
 
 extern char *WMColorPanelColorChangedNotification;
 
-/* ....................................................................... */
+/* ---[ WINGs/wcolorwell.c ]---------------------------------------------- */
 
 WMColorWell* WMCreateColorWell(WMWidget *parent);
 
@@ -1456,7 +1461,7 @@ void WSetColorWellBordered(WMColorWell *cPtr, Bool flag);
 extern char *WMColorWellDidChangeNotification;
 
 
-/* ...................................................................... */
+/* ---[ WINGs/wscrollview.c ]--------------------------------------------- */
 
 WMScrollView* WMCreateScrollView(WMWidget *parent);
 
@@ -1481,7 +1486,7 @@ void WMSetScrollViewLineScroll(WMScrollView *sPtr, int amount);
 
 void WMSetScrollViewPageScroll(WMScrollView *sPtr, int amount);
 
-/* ....................................................................... */
+/* ---[ WINGs/wslider.c ]------------------------------------------------- */
 
 WMSlider* WMCreateSlider(WMWidget *parent);
 
@@ -1505,7 +1510,7 @@ void WMSetSliderKnobThickness(WMSlider *sPtr, int thickness);
 
 void WMSetSliderImage(WMSlider *sPtr, WMPixmap *pixmap);
 
-/* ....................................................................... */
+/* ---[ WINGs/wsplitview.c ]---------------------------------------------- */
 
 
 WMSplitView* WMCreateSplitView(WMWidget *parent);
@@ -1621,9 +1626,9 @@ void WMSetTextBackgroundColor(WMText *tPtr, WMColor *color);
 
 void WMSetTextBackgroundPixmap(WMText *tPtr, WMPixmap *pixmap);
 
-void WMPrependTextStream(WMText *tPtr, char *text);
+void WMPrependTextStream(WMText *tPtr, const char *text);
 
-void WMAppendTextStream(WMText *tPtr, char *text);
+void WMAppendTextStream(WMText *tPtr, const char *text);
 
 #define WMClearText(tPtr) WMAppendTextStream \
     ((tPtr), (NULL))
@@ -1654,23 +1659,24 @@ int WMGetTextSelectionUnderlined(WMText *tPtr);
 
 void WMSetTextAlignment(WMText *tPtr, WMAlignment alignment);
 
-Bool WMFindInTextStream(WMText *tPtr, char *needle, Bool direction,
+Bool WMFindInTextStream(WMText *tPtr, const char *needle, Bool direction,
                         Bool caseSensitive);
 
+/* Warning: replacement can be modified by the function */
 Bool WMReplaceTextSelection(WMText *tPtr, char *replacement);
 
 
 /* parser related stuff... use only if implementing a new parser */
 
-void* WMCreateTextBlockWithObject(WMText *tPtr, WMWidget *w, char *description,
+void* WMCreateTextBlockWithObject(WMText *tPtr, WMWidget *w, const char *description,
                                   WMColor *color, unsigned short first,
                                   unsigned short extraInfo);
 
-void* WMCreateTextBlockWithPixmap(WMText *tPtr, WMPixmap *p, char *description,
+void* WMCreateTextBlockWithPixmap(WMText *tPtr, WMPixmap *p, const char *description,
                                   WMColor *color, unsigned short first,
                                   unsigned short extraInfo);
 
-void* WMCreateTextBlockWithText(WMText *tPtr, char *text, WMFont *font,
+void* WMCreateTextBlockWithText(WMText *tPtr, const char *text, WMFont *font,
                                 WMColor *color, unsigned short first,
                                 unsigned short length);
 
@@ -1699,8 +1705,7 @@ void* WMRemoveTextBlock(WMText *tPtr);
 
 void WMDestroyTextBlock(WMText *tPtr, void *vtb);
 
-/* ....................................................................... */
-
+/* ---[ WINGs/wtabview.c ]------------------------------------------------ */
 
 WMTabView* WMCreateTabView(WMWidget *parent);
 
@@ -1717,7 +1722,7 @@ void WMInsertItemInTabView(WMTabView *tPtr, int index, WMTabViewItem *item);
 void WMRemoveTabViewItem(WMTabView *tPtr, WMTabViewItem *item);
 
 WMTabViewItem* WMAddTabViewItemWithView(WMTabView *tPtr, WMView *view,
-                                        int identifier, char *label);
+                                        int identifier, const char *label);
 
 WMTabViewItem* WMTabViewItemAtPoint(WMTabView *tPtr, int x, int y);
 
@@ -1744,7 +1749,7 @@ void WMSetTabViewItemEnabled(WMTabViewItem *tPtr, Bool flag);
 
 int WMGetTabViewItemIdentifier(WMTabViewItem *item);
 
-void WMSetTabViewItemLabel(WMTabViewItem *item, char *label);
+void WMSetTabViewItemLabel(WMTabViewItem *item, const char *label);
 
 char* WMGetTabViewItemLabel(WMTabViewItem *item);
 
@@ -1755,7 +1760,7 @@ WMView* WMGetTabViewItemView(WMTabViewItem *item);
 void WMDestroyTabViewItem(WMTabViewItem *item);
 
 
-/* ....................................................................... */
+/* ---[ WINGs/wbox.c ]---------------------------------------------------- */
 
 WMBox* WMCreateBox(WMWidget *parent);
 
@@ -1771,28 +1776,28 @@ void WMRemoveBoxSubview(WMBox *bPtr, WMView *view);
 
 void WMSetBoxHorizontal(WMBox *box, Bool flag);
 
-/* ....................................................................... */
+/* ---[ WINGs/wpanel.c ]-------------------------------------------------- */
 
-int WMRunAlertPanel(WMScreen *app, WMWindow *owner, char *title, char *msg,
-                    char *defaultButton, char *alternateButton,
-                    char *otherButton);
+int WMRunAlertPanel(WMScreen *app, WMWindow *owner, const char *title, const char *msg,
+                    const char *defaultButton, const char *alternateButton,
+                    const char *otherButton);
 
 /* you can free the returned string */
-char* WMRunInputPanel(WMScreen *app, WMWindow *owner, char *title, char *msg,
-                      char *defaultText, char *okButton, char *cancelButton);
+char* WMRunInputPanel(WMScreen *app, WMWindow *owner, const char *title, const char *msg,
+                      const char *defaultText, const char *okButton, const char *cancelButton);
 
-WMAlertPanel* WMCreateAlertPanel(WMScreen *app, WMWindow *owner, char *title,
-                                 char *msg, char *defaultButton,
-                                 char *alternateButton, char *otherButton);
+WMAlertPanel* WMCreateAlertPanel(WMScreen *app, WMWindow *owner, const char *title,
+                                 const char *msg, const char *defaultButton,
+                                 const char *alternateButton, const char *otherButton);
 
-WMInputPanel* WMCreateInputPanel(WMScreen *app, WMWindow *owner, char *title,
-                                 char *msg, char *defaultText, char *okButton,
-                                 char *cancelButton);
+WMInputPanel* WMCreateInputPanel(WMScreen *app, WMWindow *owner, const char *title,
+                                 const char *msg, const char *defaultText, const char *okButton,
+                                 const char *cancelButton);
 
 
 WMGenericPanel* WMCreateGenericPanel(WMScreen *scrPtr, WMWindow *owner,
-                                     char *title, char *defaultButton,
-                                     char *alternateButton);
+                                     const char *title, const char *defaultButton,
+                                     const char *alternateButton);
 
 void WMDestroyAlertPanel(WMAlertPanel *panel);
 
@@ -1800,7 +1805,7 @@ void WMDestroyInputPanel(WMInputPanel *panel);
 
 void WMDestroyGenericPanel(WMGenericPanel *panel);
 
-/* ....................................................................... */
+/* ---[ WINGs/wfilepanel.c ]---------------------------------------------- */
 
 /* only 1 instance per WMScreen */
 WMOpenPanel* WMGetOpenPanel(WMScreen *scrPtr);
@@ -1821,14 +1826,14 @@ char* WMGetFilePanelFileName(WMFilePanel *panel);
 void WMFreeFilePanel(WMFilePanel *panel);
 
 int WMRunModalFilePanelForDirectory(WMFilePanel *panel, WMWindow *owner,
-                                    char *path, char *name, char **fileTypes);
+                                    char *path, const char *name, char **fileTypes);
 
 void WMSetFilePanelAccessoryView(WMFilePanel *panel, WMView *view);
 
 WMView* WMGetFilePanelAccessoryView(WMFilePanel *panel);
 
 
-/* ...................................................................... */
+/* ---[ WINGs/wfontpanel.c ]---------------------------------------------- */
 
 /* only 1 instance per WMScreen */
 WMFontPanel* WMGetFontPanel(WMScreen *scr);
@@ -1841,10 +1846,15 @@ void WMFreeFontPanel(WMFontPanel *panel);
 
 void WMSetFontPanelAction(WMFontPanel *panel, WMAction2 *action, void *data);
 
-void WMSetFontPanelFont(WMFontPanel *panel, char *fontName);
+void WMSetFontPanelFont(WMFontPanel *panel, const char *fontName);
 
 WMFont* WMGetFontPanelFont(WMFontPanel *panel);
 
+/* ---[ WINGs/configuration.c ]------------------------------------------- */
+unsigned W_getconf_mouseWheelUp(void);
+unsigned W_getconf_mouseWheelDown(void);
+void W_setconf_doubleClickDelay(int value);
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
