@@ -1,3 +1,24 @@
+/* WUtil.h
+ *
+ *  Copyright (c) 1998 scottc
+ *  Copyright (c) 1999-2004 Dan Pascu
+ *  Copyright (c) 1999-2000 Alfredo K. Kojima
+ *
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU General Public License as published by
+ *  the Free Software Foundation; either version 2 of the License, or
+ *  (at your option) any later version.
+ *
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU General Public License for more details.
+ *
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+ */
+
 #ifndef _WUTIL_H_
 #define _WUTIL_H_
 
@@ -193,15 +214,17 @@ void __wmessage(const char *func, const char *file, int line, int type, const ch
 
 /* ---[ WINGs/findfile.c ]------------------------------------------------ */
 
-char* wfindfile(char *paths, char *file);
+/* For the 4 function below, you have to free the returned string when you no longer need it */
 
-char* wfindfileinlist(char **path_list, char *file);
+char* wfindfile(const char *paths, const char *file);
 
-char* wfindfileinarray(WMPropList* array, char *file);
+char* wfindfileinlist(char *const *path_list, const char *file);
 
-char* wexpandpath(char *path);
+char* wfindfileinarray(WMPropList* array, const char *file);
 
-int wcopy_file(char *toPath, char *srcFile, char *destFile);
+char* wexpandpath(const char *path);
+
+int wcopy_file(const char *toPath, const char *srcFile, const char *destFile);
 
 /* don't free the returned string */
 char* wgethomedir(void);
@@ -220,7 +243,7 @@ char* wstrndup(const char *str, size_t len);
  * str1 and str2 can be any strings including static and constant strings.
  * str1 and str2 will not be modified.
  * Free the returned string when you're done with it. */
-char* wstrconcat(char *str1, char *str2);
+char* wstrconcat(const char *str1, const char *str2);
 
 /* This will append src to dst, and return dst. dst MUST be either NULL
  * or a string that was a result of a dynamic allocation (malloc, realloc
@@ -229,7 +252,7 @@ char* wstrconcat(char *str1, char *str2);
  * it is equivalent to calling wstrdup(src) ).
  * The returned address may be different from the original address of dst,
  * so always assign the returned address to avoid dangling pointers. */
-char* wstrappend(char *dst, char *src);
+char* wstrappend(char *dst, const char *src);
 
 size_t wstrlcpy(char *, const char *, size_t);
 size_t wstrlcat(char *, const char *, size_t);
@@ -553,7 +576,7 @@ WMData* WMCreateDataWithCapacity(unsigned capacity);
 
 WMData* WMCreateDataWithLength(unsigned length);
 
-WMData* WMCreateDataWithBytes(void *bytes, unsigned length);
+WMData* WMCreateDataWithBytes(const void *bytes, unsigned length);
 
 /* destructor is a function called to free the data when releasing the data
  * object, or NULL if no freeing of data is necesary. */
@@ -594,13 +617,13 @@ unsigned WMGetDataLength(WMData *aData);
 
 /* Adding data */
 
-void WMAppendDataBytes(WMData *aData, void *bytes, unsigned length);
+void WMAppendDataBytes(WMData *aData, const void *bytes, unsigned length);
 
 void WMAppendData(WMData *aData, WMData *anotherData);
 
 /* Modifying data */
 
-void WMReplaceDataBytesInRange(WMData *aData, WMRange aRange, void *bytes);
+void WMReplaceDataBytesInRange(WMData *aData, WMRange aRange, const void *bytes);
 
 void WMResetDataBytesInRange(WMData *aData, WMRange aRange);
 
@@ -713,11 +736,11 @@ void WMEnqueueCoalesceNotification(WMNotificationQueue *queue,
 
 void WMPLSetCaseSensitive(Bool caseSensitive);
 
-WMPropList* WMCreatePLString(char *str);
+WMPropList* WMCreatePLString(const char *str);
 
 WMPropList* WMCreatePLData(WMData *data);
 
-WMPropList* WMCreatePLDataWithBytes(unsigned char *bytes, unsigned int length);
+WMPropList* WMCreatePLDataWithBytes(const unsigned char *bytes, unsigned int length);
 
 WMPropList* WMCreatePLDataWithBytesNoCopy(unsigned char *bytes,
                                           unsigned int length,
@@ -800,26 +823,31 @@ WMPropList* WMShallowCopyPropList(WMPropList *plist);
 /* Makes a completely separate replica of the original proplist */
 WMPropList* WMDeepCopyPropList(WMPropList *plist);
 
-WMPropList* WMCreatePropListFromDescription(char *desc);
+WMPropList* WMCreatePropListFromDescription(const char *desc);
 
 /* Free the returned string when you no longer need it */
 char* WMGetPropListDescription(WMPropList *plist, Bool indented);
 
-WMPropList* WMReadPropListFromFile(char *file);
+WMPropList* WMReadPropListFromFile(const char *file);
 
-Bool WMWritePropListToFile(WMPropList *plist, char *path);
+WMPropList* WMReadPropListFromPipe(const char *command);
+
+Bool WMWritePropListToFile(WMPropList *plist, const char *path);
 
 /* ---[ WINGs/userdefaults.c ]-------------------------------------------- */
 
-char* wusergnusteppath(void);
+/* don't free the returned string */
+const char* wusergnusteppath(void);
 
-char* wdefaultspathfordomain(char *domain);
+/* Free the returned string when you no longer need it */
+char* wdefaultspathfordomain(const char *domain);
 
+/* Free the returned string when you no longer need it */
 char* wglobaldefaultspathfordomain(const char *domain);
 
 WMUserDefaults* WMGetStandardUserDefaults(void);
 
-WMUserDefaults* WMGetDefaultsFromPath(char *path);
+WMUserDefaults* WMGetDefaultsFromPath(const char *path);
 
 void WMSynchronizeUserDefaults(WMUserDefaults *database);
 
@@ -832,32 +860,33 @@ void WMEnableUDPeriodicSynchronization(WMUserDefaults *database, Bool enable);
  * Keys in array are just retained references to the original keys */
 WMPropList* WMGetUDKeys(WMUserDefaults *database);
 
-WMPropList* WMGetUDObjectForKey(WMUserDefaults *database, char *defaultName);
+WMPropList* WMGetUDObjectForKey(WMUserDefaults *database, const char *defaultName);
 
 void WMSetUDObjectForKey(WMUserDefaults *database, WMPropList *object,
-                         char *defaultName);
+                         const char *defaultName);
 
-void WMRemoveUDObjectForKey(WMUserDefaults *database, char *defaultName);
+void WMRemoveUDObjectForKey(WMUserDefaults *database, const char *defaultName);
 
-char* WMGetUDStringForKey(WMUserDefaults *database, char *defaultName);
+/* Returns a reference. Do not free it! */
+char* WMGetUDStringForKey(WMUserDefaults *database, const char *defaultName);
 
-int WMGetUDIntegerForKey(WMUserDefaults *database, char *defaultName);
+int WMGetUDIntegerForKey(WMUserDefaults *database, const char *defaultName);
 
-float WMGetUDFloatForKey(WMUserDefaults *database, char *defaultName);
+float WMGetUDFloatForKey(WMUserDefaults *database, const char *defaultName);
 
-Bool WMGetUDBoolForKey(WMUserDefaults *database, char *defaultName);
+Bool WMGetUDBoolForKey(WMUserDefaults *database, const char *defaultName);
 
-void WMSetUDStringForKey(WMUserDefaults *database, char *value,
-                         char *defaultName);
+void WMSetUDStringForKey(WMUserDefaults *database, const char *value,
+                         const char *defaultName);
 
 void WMSetUDIntegerForKey(WMUserDefaults *database, int value,
-                          char *defaultName);
+                          const char *defaultName);
 
 void WMSetUDFloatForKey(WMUserDefaults *database, float value,
-                        char *defaultName);
+                        const char *defaultName);
 
 void WMSetUDBoolForKey(WMUserDefaults *database, Bool value,
-                       char *defaultName);
+                       const char *defaultName);
 
 WMPropList* WMGetUDSearchList(WMUserDefaults *database);
 
