@@ -28,8 +28,6 @@
 #include <unistd.h>
 #include <math.h>
 
-/* double-click tester */
-#include "double.h"
 
 #define XSET	"xset"
 
@@ -95,7 +93,6 @@ static char *wheelActions[2];
 
 #define DELAY(i)		((i)*75+170)
 
-int ModifierFromKey(Display * dpy, char *key);
 
 static void setMouseAccel(WMScreen * scr, float accel, int threshold)
 {
@@ -161,7 +158,6 @@ static void doubleClick(WMWidget * w, void *data)
 {
 	_Panel *panel = (_Panel *) data;
 	int i;
-	extern _WINGsConfiguration WINGsConfiguration;
 	char buffer[32];
 
 	for (i = 0; i < 5; i++) {
@@ -174,7 +170,7 @@ static void doubleClick(WMWidget * w, void *data)
 	WMSetTextFieldText(panel->ddelaT, buffer);
 }
 
-static int getButtonAction(char *str)
+static int getButtonAction(const char *str)
 {
 	if (!str)
 		return -2;
@@ -192,7 +188,7 @@ static int getButtonAction(char *str)
 
 }
 
-static int getWheelAction(char *str)
+static int getWheelAction(const char *str)
 {
 	if (!str)
 		return -2;
@@ -319,10 +315,13 @@ static void showData(_Panel * panel)
 	}
 
 	if (a < 1) {
-		sscanf(WMGetPopUpButtonItem(panel->grabP, 0), "%s", buffer);
-		WMSetPopUpButtonSelectedItem(panel->grabP, 0);
+		char *previous;
+
+		previous = WMGetPopUpButtonItem(panel->grabP, 0);
+		if (previous != NULL)
+			WMSetPopUpButtonSelectedItem(panel->grabP, 0);
 		wwarning(_("modifier key %s for option ModifierKey was not recognized. Using %s as default"),
-			 str, buffer);
+					str, previous?previous:"(empty)");
 	}
 }
 
@@ -362,7 +361,10 @@ static void fillModifierPopUp(WMPopUpButton * pop)
 			if (mapping->modifiermap[idx] != 0) {
 				int l;
 				for (l = 0; l < 4; l++) {
-					ksym = XkbKeycodeToKeysym(dpy, mapping->modifiermap[idx], 0, l);
+					if (xext_xkb_supported)
+						ksym = XkbKeycodeToKeysym(dpy, mapping->modifiermap[idx], 0, l);
+					else
+						ksym = XKeycodeToKeysym(dpy, mapping->modifiermap[idx], 0);
 					if (ksym != NoSymbol)
 						break;
 				}
@@ -608,13 +610,13 @@ static void createPanel(Panel * p)
 	WMResizeWidget(panel->wheelP, 135, 20);
 	WMMoveWidget(panel->wheelP, 95, 129);
 
-	for (i = 0; i < sizeof(buttonActions) / sizeof(char *); i++) {
+	for (i = 0; i < sizeof(buttonActions) / sizeof(buttonActions[0]); i++) {
 		WMAddPopUpButtonItem(panel->button1P, buttonActions[i]);
 		WMAddPopUpButtonItem(panel->button2P, buttonActions[i]);
 		WMAddPopUpButtonItem(panel->button3P, buttonActions[i]);
 	}
 
-	for (i = 0; i < sizeof(wheelActions) / sizeof(char *); i++) {
+	for (i = 0; i < sizeof(wheelActions) / sizeof(wheelActions[0]); i++) {
 		WMAddPopUpButtonItem(panel->wheelP, wheelActions[i]);
 	}
 
@@ -644,7 +646,7 @@ static void createPanel(Panel * p)
 	showData(panel);
 }
 
-static void storeCommandInScript(char *cmd, char *line)
+static void storeCommandInScript(const char *cmd, const char *line)
 {
 	char *path;
 	FILE *f;
