@@ -14,10 +14,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- *  USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef TEXTUREPANEL_H_
@@ -36,7 +35,7 @@ void ShowTexturePanel(TexturePanel *panel);
 
 void HideTexturePanel(TexturePanel *panel);
 
-void SetTexturePanelTexture(TexturePanel *panel, char *name,
+void SetTexturePanelTexture(TexturePanel *panel, const char *name,
                             WMPropList *texture);
 
 
