@@ -87,16 +87,6 @@ typedef struct WObjDescriptor {
     void *parent;		       /* parent object (WWindow or WMenu) */
 } WObjDescriptor;
 
-
-/* shutdown modes */
-typedef enum {
-    WSExitMode,
-    WSLogoutMode,
-    WSKillMode,
-    WSRestartPreparationMode
-} WShutdownMode;
-
-
 /* internal buttons */
 #define WBUT_CLOSE              0
 #define WBUT_BROKENCLOSE        1
@@ -273,9 +263,9 @@ typedef enum {
 /* notifications */
 
 #ifdef MAINFILE
-#define NOTIFICATION(n) char *WN##n = #n
+#define NOTIFICATION(n) const char WN##n [] = #n
 #else
-#define NOTIFICATION(n) extern char *WN##n
+#define NOTIFICATION(n) extern const char WN##n []
 #endif
 
 NOTIFICATION(WindowAppearanceSettingsChanged);
@@ -307,7 +297,7 @@ typedef struct WCoord {
     int x, y;
 } WCoord;
 
-typedef struct WPreferences {
+extern struct WPreferences {
     char *pixmap_path;		       /* : separated list of paths to find pixmaps */
     char *icon_path;		       /* : separated list of paths to find icons */
     WMArray *fallbackWMs;              /* fallback window manager list */
@@ -428,6 +418,14 @@ typedef struct WPreferences {
     int history_lines;                  /* history of "Run..." dialog */
     char cycle_active_head_only;        /* Cycle only windows on the active head */
     char cycle_ignore_minimized;        /* Ignore minimized windows when cycling */
+    char strict_windoze_cycle;          /* don't close switchpanel when shift is released */
+    char panel_only_open;               /* Only open the switchpanel; don't switch */
+
+    /* All delays here are in ms. 0 means instant auto-action. */
+    int clip_auto_raise_delay;         /* Delay after which the clip will be raised when entered */
+    int clip_auto_lower_delay;         /* Delay after which the clip will be lowered when leaved */
+    int clip_auto_expand_delay;        /* Delay after which the clip will expand when entered */
+    int clip_auto_collapse_delay;      /* Delay after which the clip will collapse when leaved */
 
     RImage *swtileImage;
     RImage *swbackImage[9];
@@ -437,6 +435,9 @@ typedef struct WPreferences {
     struct {
         unsigned int nodock:1;	       /* don't display the dock */
         unsigned int noclip:1;         /* don't display the clip */
+        unsigned int clip_merged_in_dock:1; /* disable clip, dock gets its workspace switching functionality */
+        unsigned int nodrawer:1;       /* don't use drawers */
+        unsigned int wrap_appicons_in_dock:1; /* Whether to wrap appicons when Dock is moved up and down */
         unsigned int noupdates:1;      /* don't require ~/GNUstep (-static) */
         unsigned int noautolaunch:1;   /* don't autolaunch apps */
         unsigned int norestore:1;      /* don't restore session */
@@ -445,7 +446,7 @@ typedef struct WPreferences {
 #endif
         unsigned int restarting:2;
     } flags;			       /* internal flags */
-} WPreferences;
+} wPreferences;
 
 /****** Global Variables  ******/
 extern Display	*dpy;
@@ -454,18 +455,19 @@ extern char WProgramState;
 extern char WProgramSigState;
 
 /****** Notifications ******/
-extern const char *WMNManaged;
-extern const char *WMNUnmanaged;
-extern const char *WMNChangedWorkspace;
-extern const char *WMNChangedState;
-extern const char *WMNChangedFocus;
-extern const char *WMNChangedStacking;
-extern const char *WMNChangedName;
+extern const char WMNManaged[];
+extern const char WMNUnmanaged[];
+extern const char WMNChangedWorkspace[];
+extern const char WMNChangedState[];
+extern const char WMNChangedFocus[];
+extern const char WMNChangedStacking[];
+extern const char WMNChangedName[];
 
-extern const char *WMNWorkspaceCreated;
-extern const char *WMNWorkspaceDestroyed;
-extern const char *WMNWorkspaceChanged;
-extern const char *WMNWorkspaceNameChanged;
+extern const char WMNWorkspaceCreated[];
+extern const char WMNWorkspaceDestroyed[];
+extern const char WMNWorkspaceChanged[];
+extern const char WMNWorkspaceNameChanged[];
+
+extern const char WMNResetStacking[];
 
-extern const char *WMNResetStacking;
 #endif
