@@ -46,16 +46,16 @@
 #include "workspace.h"
 #include "xinerama.h"
 #include "usermenu.h"
+#include "placement.h"
+#include "misc.h"
+#include "event.h"
 
 /****** Global Variables ******/
 
 int ignore_wks_change = 0;
 extern Time LastTimestamp;
 extern Time LastFocusChange;
-extern WPreferences wPreferences;
 extern Atom _XA_WM_TAKE_FOCUS;
-extern void ProcessPendingEvents();
-extern int calcIntersectionLength(int p1, int l1, int p2, int l2);
 
 static void find_Maximus_geometry(WWindow *wwin, WArea usableArea, int *new_x, int *new_y,
 				  unsigned int *new_width, unsigned int *new_height);
@@ -314,8 +314,8 @@ static void remember_geometry(WWindow *wwin, int *x, int *y, int *w, int *h)
 	old_geom_rect = wmkrect(wwin->old_geometry.x, wwin->old_geometry.y, wwin->old_geometry.width, wwin->old_geometry.height);
 	old_head = wGetHeadForRect(wwin->screen_ptr, old_geom_rect);
 	same_head = (wGetHeadForWindow(wwin) == old_head);
-	*x = (wwin->old_geometry.x && same_head) ? wwin->old_geometry.x : wwin->frame_x;
-	*y = (wwin->old_geometry.y && same_head) ? wwin->old_geometry.y : wwin->frame_y;
+	*x = ((wwin->old_geometry.x || wwin->old_geometry.width) && same_head) ? wwin->old_geometry.x : wwin->frame_x;
+	*y = ((wwin->old_geometry.y || wwin->old_geometry.height) && same_head) ? wwin->old_geometry.y : wwin->frame_y;
 	*w = wwin->old_geometry.width ? wwin->old_geometry.width : wwin->client.width;
 	*h = wwin->old_geometry.height ? wwin->old_geometry.height : wwin->client.height;
 }
@@ -341,18 +341,17 @@ void update_saved_geometry(WWindow *wwin)
 		save_old_geometry(wwin, SAVE_GEOMETRY_X);
 }
 
-#define IS_MAX_HORIZONTALLY(directions) ((directions & MAX_HORIZONTAL) | (directions & MAX_LEFTHALF) | (directions & MAX_RIGHTHALF))
 void wMaximizeWindow(WWindow *wwin, int directions)
 {
-	int new_x, new_y;
-	unsigned int new_width, new_height, half_scr_width;
+	unsigned int new_width, new_height, half_scr_width, half_scr_height;
+	int new_x = 0;
+	int new_y = 0;
 	int maximus_x = 0;
 	int maximus_y = 0;
 	unsigned int maximus_width = 0;
 	unsigned int maximus_height = 0;
 	WArea usableArea, totalArea;
 	Bool has_border = 1;
-	int save_directions = 0;
 	int adj_size;
 
 	if (!IS_RESIZABLE(wwin))
@@ -362,29 +361,16 @@ void wMaximizeWindow(WWindow *wwin, int directions)
 		has_border = 0;
 
 	/* the size to adjust the geometry */
-	adj_size = FRAME_BORDER_WIDTH * 2 * has_border;
+	adj_size = wwin->screen_ptr->frame_border_width * 2 * has_border;
 
-	/* save old coordinates before we change the current values
-	 * always if the window is not currently maximized at all
-	 * but never if the window has been Maximusized */
+	/* save old coordinates before we change the current values */
 	if (!wwin->flags.maximized)
-		save_directions |= SAVE_GEOMETRY_ALL;
-	else if (!(wwin->flags.old_maximized & MAX_MAXIMUS)) {
-		if ((directions & MAX_VERTICAL) &&
-		    !(wwin->flags.maximized & MAX_VERTICAL))
-			save_directions |= SAVE_GEOMETRY_Y | SAVE_GEOMETRY_HEIGHT;
-		if (IS_MAX_HORIZONTALLY(directions) &&
-		    !IS_MAX_HORIZONTALLY(wwin->flags.maximized))
-			save_directions |= SAVE_GEOMETRY_X | SAVE_GEOMETRY_WIDTH;
-	}
-	if ((directions & MAX_MAXIMUS) && !wwin->flags.maximized)
-		save_directions |= SAVE_GEOMETRY_ALL;
-	save_old_geometry(wwin, save_directions);
+		save_old_geometry(wwin, SAVE_GEOMETRY_ALL);
 
-	totalArea.x1 = 0;
-	totalArea.y1 = 0;
 	totalArea.x2 = wwin->screen_ptr->scr_width;
 	totalArea.y2 = wwin->screen_ptr->scr_height;
+	totalArea.x1 = 0;
+	totalArea.y1 = 0;
 	usableArea = totalArea;
 
 	if (!(directions & MAX_IGNORE_XINERAMA)) {
@@ -399,89 +385,83 @@ void wMaximizeWindow(WWindow *wwin, int directions)
 		usableArea = wGetUsableAreaForHead(scr, head, &totalArea, True);
 	}
 
-	/* remember Maximus geometry if we'll need it later */
-	if ((wwin->flags.old_maximized & MAX_MAXIMUS) || (directions & MAX_MAXIMUS))
-		find_Maximus_geometry(wwin, usableArea, &maximus_x, &maximus_y, &maximus_width, &maximus_height);
-
 	/* Only save directions, not kbd or xinerama hints */
-	directions &= (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_MAXIMUS);
+	directions &= (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS);
 
 	if (WFLAGP(wwin, full_maximize)) {
 		usableArea = totalArea;
 	}
 	half_scr_width = (usableArea.x2 - usableArea.x1)/2;
+	half_scr_height = (usableArea.y2 - usableArea.y1)/2;
 
 	if (wwin->flags.shaded) {
 		wwin->flags.skip_next_animation = 1;
 		wUnshadeWindow(wwin);
 	}
 
-	if (directions & MAX_HORIZONTAL) {
-		new_width = usableArea.x2 - usableArea.x1 - adj_size;
-		new_x = usableArea.x1;
-	} else if (directions & MAX_LEFTHALF) {
-		new_width = half_scr_width - adj_size;
-		new_x = usableArea.x1;
-		wwin->flags.old_maximized |= MAX_LEFTHALF;
-		wwin->flags.old_maximized &= ~MAX_RIGHTHALF;
-	} else if (directions & MAX_RIGHTHALF) {
-		new_width = half_scr_width - adj_size;
-		new_x = usableArea.x1 + half_scr_width;
-		wwin->flags.old_maximized |= MAX_RIGHTHALF;
-		wwin->flags.old_maximized &= ~MAX_LEFTHALF;
-	} else if (wwin->flags.old_maximized & MAX_MAXIMUS) {
-		new_x = maximus_x;
-		new_width = maximus_width - adj_size;
-	} else if (IS_MAX_HORIZONTALLY(wwin->flags.maximized)) {
-		new_x = (wwin->old_geometry.x) ? wwin->old_geometry.x : wwin->frame_x;
-		new_width = (wwin->old_geometry.width) ? wwin->old_geometry.width : wwin->frame->core->width;
-	} else {
-		wwin->flags.old_maximized &= ~(MAX_LEFTHALF | MAX_RIGHTHALF);
-		new_x = wwin->frame_x;
-		new_width = wwin->frame->core->width;
-	}
-
-	if (directions & MAX_VERTICAL) {
-		new_height = usableArea.y2 - usableArea.y1 - adj_size;
-		new_y = usableArea.y1;
-		if (WFLAGP(wwin, full_maximize)) {
-			new_y -= wwin->frame->top_width;
-			new_height += wwin->frame->bottom_width - 1;
-		}
-	} else if (wwin->flags.old_maximized & MAX_MAXIMUS) {
-		new_y = maximus_y;
-		new_height = maximus_height - adj_size;
-		/* HACK: this will be subtracted again below */
-		new_height += wwin->frame->top_width + wwin->frame->bottom_width;
-	} else if (wwin->flags.maximized & MAX_VERTICAL) {
-		new_y = (wwin->old_geometry.y) ? wwin->old_geometry.y : wwin->frame_y;
-		new_height = (wwin->old_geometry.height) ? wwin->old_geometry.height : wwin->frame->core->height;
-		/* HACK: this will be subtracted again below */
-		new_height += wwin->frame->top_width + wwin->frame->bottom_width;
-		wwin->flags.old_maximized &= ~(MAX_LEFTHALF | MAX_RIGHTHALF);
-	} else {
-		new_y = wwin->frame_y;
-		new_height = wwin->frame->core->height;
-	}
-
-	if (!WFLAGP(wwin, full_maximize)) {
-		new_height -= wwin->frame->top_width + wwin->frame->bottom_width;
-	}
-
 	if (directions & MAX_MAXIMUS) {
-		new_x = maximus_x;
-		new_y = maximus_y;
+		find_Maximus_geometry(wwin, usableArea, &maximus_x, &maximus_y, &maximus_width, &maximus_height);
 		new_width = maximus_width - adj_size;
 		new_height = maximus_height - adj_size;
-		if (WFLAGP(wwin, full_maximize) && new_y == 0) {
+		new_x = maximus_x;
+		new_y = maximus_y;
+		if (WFLAGP(wwin, full_maximize) && (new_y == 0)) {
+			new_height += wwin->frame->bottom_width - 1;
 			new_y -= wwin->frame->top_width;
-			new_height += wwin->frame->top_width - 1;
 		}
+
 		wwin->maximus_x = new_x;
 		wwin->maximus_y = new_y;
 		wwin->flags.old_maximized |= MAX_MAXIMUS;
+	} else {
+		/* set default values if no option set then */
+		if (!(directions & (MAX_HORIZONTAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_MAXIMUS))) {
+			new_width = (wwin->old_geometry.width) ? wwin->old_geometry.width : wwin->frame->core->width;
+			new_x = (wwin->old_geometry.x) ? wwin->old_geometry.x : wwin->frame_x;
+		}
+		if (!(directions & (MAX_VERTICAL | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS))) {
+			new_height = (wwin->old_geometry.height) ? wwin->old_geometry.height : wwin->frame->core->height;
+			new_y = (wwin->old_geometry.y) ? wwin->old_geometry.y : wwin->frame_y;
+		}
+
+		/* left|right position */
+		if (directions & MAX_LEFTHALF) {
+			new_width = half_scr_width - adj_size;
+			new_x = usableArea.x1;
+		} else if (directions & MAX_RIGHTHALF) {
+			new_width = half_scr_width - adj_size;
+			new_x = usableArea.x1 + half_scr_width;
+		}
+		/* top|bottom position */
+		if (directions & MAX_TOPHALF) {
+			new_height = half_scr_height - adj_size;
+			new_y = usableArea.y1;
+		} else if (directions & MAX_BOTTOMHALF) {
+			new_height = half_scr_height - adj_size;
+			new_y = usableArea.y1 + half_scr_height;
+		}
+
+		/* vertical|horizontal position */
+		if (directions & MAX_HORIZONTAL) {
+			new_width = usableArea.x2 - usableArea.x1 - adj_size;
+			new_x = usableArea.x1;
+		}
+		if (directions & MAX_VERTICAL) {
+			new_height = usableArea.y2 - usableArea.y1 - adj_size;
+			new_y = usableArea.y1;
+			if (WFLAGP(wwin, full_maximize) && (new_y == 0))
+				new_y -= wwin->frame->top_width;
+		}
 	}
 
+	if (!WFLAGP(wwin, full_maximize) && !(directions == MAX_MAXIMUS || directions == MAX_HORIZONTAL))
+		new_height -= wwin->frame->top_width + wwin->frame->bottom_width;
+
+	/* set maximization state */
+	wwin->flags.maximized = directions;
+	if ((wwin->flags.old_maximized & MAX_MAXIMUS) && !wwin->flags.maximized)
+		wwin->flags.maximized = MAX_MAXIMUS;
+
 	wWindowConstrainSize(wwin, &new_width, &new_height);
 
 	wWindowCropSize(wwin, usableArea.x2 - usableArea.x1,
@@ -491,82 +471,72 @@ void wMaximizeWindow(WWindow *wwin, int directions)
 	wWindowSynthConfigureNotify(wwin);
 
 	WMPostNotificationName(WMNChangedState, wwin, "maximize");
-
-	/* set maximization state */
-	wwin->flags.maximized = directions;
-	if ((wwin->flags.old_maximized & MAX_MAXIMUS) && !wwin->flags.maximized)
-		wwin->flags.maximized = MAX_MAXIMUS;
 }
 
 /* generic (un)maximizer */
 void handleMaximize(WWindow *wwin, int directions)
 {
 	int current = wwin->flags.maximized;
-	int requested = directions & (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_MAXIMUS);
+	int requested = directions & (MAX_HORIZONTAL | MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS);
 	int effective = requested ^ current;
 	int flags = directions & ~requested;
 
 	if (!effective) {
 		/* allow wMaximizeWindow to restore the Maximusized size */
 		if ((wwin->flags.old_maximized & MAX_MAXIMUS) &&
-		    !(requested & MAX_MAXIMUS))
-			wMaximizeWindow(wwin, flags);
+				!(requested & MAX_MAXIMUS))
+			wMaximizeWindow(wwin, MAX_MAXIMUS | flags);
 		else
 			wUnmaximizeWindow(wwin);
-	}
+	/* these alone mean vertical|horizontal toggle */
+	} else if ((effective == MAX_LEFTHALF) ||
+			(effective == MAX_RIGHTHALF) ||
+			(effective == MAX_TOPHALF) ||
+			(effective == MAX_BOTTOMHALF))
+		wUnmaximizeWindow(wwin);
 	else {
-		/* MAX_MAXIMUS takes precedence */
-		effective &= ~MAX_MAXIMUS;
-		if (requested & MAX_MAXIMUS) {
-			/* window was previously Maximusized then maximized */
-			if ((wwin->flags.old_maximized & MAX_MAXIMUS) && !current) {
-				wUnmaximizeWindow(wwin);
-				return;
-			}
-			else
-				effective = MAX_MAXIMUS;
-		}
-		else if (requested == (MAX_HORIZONTAL | MAX_VERTICAL))
+		if ((requested == (MAX_HORIZONTAL | MAX_VERTICAL)) ||
+				(requested == MAX_MAXIMUS))
 			effective = requested;
 		else {
-			/* handle MAX_HORIZONTAL -> MAX_(LEFT|RIGHT)HALF */
-			if (IS_MAX_HORIZONTALLY(current)) {
-				if (IS_MAX_HORIZONTALLY(requested)) {
-					effective &= ~(MAX_HORIZONTAL | MAX_LEFTHALF | MAX_RIGHTHALF);
-					effective |= (requested & (MAX_HORIZONTAL | MAX_LEFTHALF | MAX_RIGHTHALF));
-					if (requested & MAX_HORIZONTAL) {
-						/* restore to half maximization */
-						if (wwin->flags.old_maximized & MAX_LEFTHALF)
-							effective |= MAX_LEFTHALF;
-						else if (wwin->flags.old_maximized & MAX_RIGHTHALF)
-							effective |= MAX_RIGHTHALF;
-					}
-					/* MAX_VERTICAL is implicit with MAX_(LEFT|RIGHT)HALF */
-					else
-						effective |= MAX_VERTICAL;
-				} else {
-					/* toggling MAX_VERTICAL */
-					if ((requested & MAX_VERTICAL) &&
-					    (current & MAX_VERTICAL)) {
-						effective &= ~(MAX_LEFTHALF | MAX_RIGHTHALF | MAX_VERTICAL);
-					}
-				}
-			}
-			/* handle MAX_VERTICAL -> MAX_(LEFT|RIGHT)HALF */
-			if (current & MAX_VERTICAL) {
-				if ((requested & MAX_LEFTHALF) ||
-				    (requested & MAX_RIGHTHALF)) {
+			if (requested & MAX_LEFTHALF) {
+				if (!(requested & (MAX_TOPHALF | MAX_BOTTOMHALF)))
 					effective |= MAX_VERTICAL;
-				}
+				else
+					effective |= requested & (MAX_TOPHALF | MAX_BOTTOMHALF);
+				effective |= MAX_LEFTHALF;
+				effective &= ~(MAX_HORIZONTAL | MAX_RIGHTHALF);
+			} else if (requested & MAX_RIGHTHALF) {
+				if (!(requested & (MAX_TOPHALF | MAX_BOTTOMHALF)))
+					effective |= MAX_VERTICAL;
+				else
+					effective |= requested & (MAX_TOPHALF | MAX_BOTTOMHALF);
+				effective |= MAX_RIGHTHALF;
+				effective &= ~(MAX_HORIZONTAL | MAX_LEFTHALF);
 			}
-			/* toggling MAX_HORIZONTAL */
-			if ((requested & MAX_HORIZONTAL) &&
-			    (current & MAX_HORIZONTAL))
-				effective &= ~MAX_HORIZONTAL;
+			if (requested & MAX_TOPHALF) {
+				if (!(requested & (MAX_LEFTHALF | MAX_RIGHTHALF)))
+					effective |= MAX_HORIZONTAL;
+				else
+					effective |= requested & (MAX_LEFTHALF | MAX_RIGHTHALF);
+				effective |= MAX_TOPHALF;
+				effective &= ~(MAX_VERTICAL | MAX_BOTTOMHALF);
+			} else if (requested & MAX_BOTTOMHALF) {
+				if (!(requested & (MAX_LEFTHALF | MAX_RIGHTHALF)))
+					effective |= MAX_HORIZONTAL;
+				else
+					effective |= requested & (MAX_LEFTHALF | MAX_RIGHTHALF);
+				effective |= MAX_BOTTOMHALF;
+				effective &= ~(MAX_VERTICAL | MAX_TOPHALF);
+			}
+			if (requested & MAX_HORIZONTAL)
+				effective &= ~(MAX_LEFTHALF | MAX_RIGHTHALF);
+			if (requested & MAX_VERTICAL)
+				effective &= ~(MAX_TOPHALF | MAX_BOTTOMHALF);
+			effective &= ~MAX_MAXIMUS;
 		}
 		wMaximizeWindow(wwin, effective | flags);
 	}
-	return;
 }
 
 /* the window boundary coordinates */
@@ -626,7 +596,7 @@ static void find_Maximus_geometry(WWindow *wwin, WArea usableArea, int *new_x, i
 	if (HAS_RESIZEBAR(wwin))
 		rbar_height_0 = RESIZEBAR_HEIGHT;
 	if (HAS_BORDER(wwin))
-		bd_width_0 = FRAME_BORDER_WIDTH;
+		bd_width_0 = wwin->screen_ptr->frame_border_width;
 
 	/* the length to be subtracted if the window has titlebar, etc */
 	adjust_height = tbar_height_0 + 2 * bd_width_0 + rbar_height_0;
@@ -1241,8 +1211,10 @@ void wDeiconifyWindow(WWindow *wwin)
 	if (!netwm_hidden)
 		wWindowChangeWorkspace(wwin, wwin->screen_ptr->current_workspace);
 
-	if (!wwin->flags.miniaturized)
+	if (!wwin->flags.miniaturized) {
+		ignore_wks_change = 0;
 		return;
+	}
 
 	if (wwin->transient_for != None && wwin->transient_for != wwin->screen_ptr->root_win) {
 		WWindow *owner = recursiveTransientFor(wwin);
@@ -1251,6 +1223,7 @@ void wDeiconifyWindow(WWindow *wwin)
 			wDeiconifyWindow(owner);
 			wSetFocusTo(wwin->screen_ptr, wwin);
 			wRaiseFrame(wwin->frame->core);
+			ignore_wks_change = 0;
 			return;
 		}
 	}
@@ -1338,8 +1311,10 @@ void wDeiconifyWindow(WWindow *wwin)
 			ProcessPendingEvents();
 
 			/* the window can disappear while ProcessPendingEvents() runs */
-			if (!wWindowFor(clientwin))
+			if (!wWindowFor(clientwin)) {
+				ignore_wks_change = 0;
 				return;
+			}
 		}
 #endif
 	}
@@ -1871,7 +1846,7 @@ void wSelectWindow(WWindow *wwin, Bool flag)
 			XSetWindowBorder(dpy, wwin->frame->core->window, scr->white_pixel);
 
 		if (!HAS_BORDER(wwin)) {
-			XSetWindowBorderWidth(dpy, wwin->frame->core->window, FRAME_BORDER_WIDTH);
+			XSetWindowBorderWidth(dpy, wwin->frame->core->window, wwin->screen_ptr->frame_border_width);
 		}
 
 		if (!scr->selected_windows)
