@@ -42,6 +42,7 @@
 #include "appicon.h"
 #include "appmenu.h"
 #include "wmspec.h"
+#include "misc.h"
 
 /****** Global Variables ******/
 
@@ -75,8 +76,8 @@ void wClientRestore(WWindow * wwin)
 
 	wClientGetGravityOffsets(wwin, &gx, &gy);
 	/* set the position of the frame on screen */
-	wwin->frame_x -= gx * FRAME_BORDER_WIDTH;
-	wwin->frame_y -= gy * FRAME_BORDER_WIDTH;
+	wwin->frame_x -= gx * wwin->screen_ptr->frame_border_width;
+	wwin->frame_y -= gy * wwin->screen_ptr->frame_border_width;
 	/* if gravity is to the south, account for the border sizes */
 	if (gy > 0)
 		wwin->frame_y += (wwin->frame->top_width + wwin->frame->bottom_width);
@@ -224,7 +225,7 @@ void wClientConfigure(WWindow * wwin, XConfigureRequestEvent * xcre)
 			nx = xcre->x;
 			/* Subtracting the border makes the window shift by 1 pixel -Dan */
 			/*if (HAS_BORDER(wwin)) {
-			   nx -= FRAME_BORDER_WIDTH;
+			   nx -= wwin->screen_ptr->frame_border_width;
 			   } */
 		} else {
 			nx = wwin->frame_x;
@@ -234,7 +235,7 @@ void wClientConfigure(WWindow * wwin, XConfigureRequestEvent * xcre)
 			ny = xcre->y - ((ofs_y < 0) ? 0 : wwin->frame->top_width);
 			/* Subtracting the border makes the window shift by 1 pixel -Dan */
 			/*if (HAS_BORDER(wwin)) {
-			   ny -= FRAME_BORDER_WIDTH;
+			   ny -= wwin->screen_ptr->frame_border_width;
 			   } */
 		} else {
 			ny = wwin->frame_y;
@@ -251,11 +252,11 @@ void wClientConfigure(WWindow * wwin, XConfigureRequestEvent * xcre)
 			nheight = wwin->frame->core->height - wwin->frame->top_width - wwin->frame->bottom_width;
 
 		if (nwidth != wwin->old_geometry.width)
-			wwin->flags.maximized &= ~(MAX_HORIZONTAL | MAX_MAXIMUS);
-                if (nheight != wwin->old_geometry.height)
+			wwin->flags.maximized &= ~(MAX_HORIZONTAL | MAX_TOPHALF | MAX_BOTTOMHALF | MAX_MAXIMUS);
+		if (nheight != wwin->old_geometry.height)
 			wwin->flags.maximized &= ~(MAX_VERTICAL | MAX_LEFTHALF | MAX_RIGHTHALF | MAX_MAXIMUS);
 
-                wWindowConfigure(wwin, nx, ny, nwidth, nheight);
+		wWindowConfigure(wwin, nx, ny, nwidth, nheight);
 		wwin->old_geometry.x = nx;
 		wwin->old_geometry.y = ny;
 		wwin->old_geometry.width = nwidth;
@@ -322,16 +323,10 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 		break;
 
 	case XA_WM_ICON_NAME:
-		if (!wwin->flags.net_has_icon_title) {
-			if (!wwin->icon)
-				break;
-			else {
-				char *new_title;
-
-				/* icon title was changed */
-				wGetIconName(dpy, wwin->client_win, &new_title);
-				wIconChangeTitle(wwin->icon, new_title);
-			}
+		/* Title has changed, update the icon title */
+		if (wwin->icon) {
+			wIconChangeTitle(wwin->icon, wwin);
+			wIconPaint(wwin->icon);
 		}
 		break;
 
@@ -472,12 +467,12 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 			    || (wwin->wm_hints->flags & IconWindowHint)) {
 				WApplication *wapp;
 
-				if (wwin->flags.miniaturized && wwin->icon) {
-					wIconUpdate(wwin->icon, NULL);
-				}
+				if (wwin->flags.miniaturized && wwin->icon)
+					wIconUpdate(wwin->icon);
+
 				wapp = wApplicationOf(wwin->main_window);
 				if (wapp && wapp->app_icon) {
-					wIconUpdate(wapp->app_icon->icon, NULL);
+					wIconUpdate(wapp->app_icon->icon);
 					wAppIconPaint(wapp->app_icon);
 				}
 			}
@@ -582,7 +577,6 @@ void wClientCheckProperty(WWindow * wwin, XPropertyEvent * event)
 					wAppMenuDestroy(wapp->menu);
 				}
 				if (wwin->fake_group) {
-					extern WPreferences wPreferences;
 					WScreen *scr = wwin->screen_ptr;
 					WWindow *foo = scr->focused_window;
 					WFakeGroupLeader *fPtr = wwin->fake_group;
