@@ -61,6 +61,9 @@
 #include "dock.h"
 #include "workspace.h"
 #include "properties.h"
+#include "misc.h"
+#include "event.h"
+#include "winmenu.h"
 
 #define MAX_SHORTCUT_LENGTH 32
 
@@ -73,86 +76,94 @@ extern WDDomain *WDWindowMaker;
 extern WDDomain *WDWindowAttributes;
 extern WDDomain *WDRootMenu;
 extern int wScreenCount;
-extern WPreferences wPreferences;
 extern WShortKey wKeyBindings[WKBD_LAST];
 
-typedef struct {
-	char *key;
-	char *default_value;
+typedef struct _WDefaultEntry  WDefaultEntry;
+typedef int (WDECallbackConvert) (WScreen *scr, WDefaultEntry *entry, WMPropList *plvalue, void *addr, void **tdata);
+typedef int (WDECallbackUpdate) (WScreen *scr, WDefaultEntry *entry, void *tdata, void *extra_data);
+
+struct _WDefaultEntry {
+	const char *key;
+	const char *default_value;
 	void *extra_data;
 	void *addr;
-	int (*convert) ();
-	int (*update) ();
+	WDECallbackConvert *convert;
+	WDECallbackUpdate *update;
 	WMPropList *plkey;
 	WMPropList *plvalue;	/* default value */
-} WDefaultEntry;
+};
 
 /* used to map strings to integers */
 typedef struct {
-	char *string;
+	const char *string;
 	short value;
 	char is_alias;
 } WOptionEnumeration;
 
 /* type converters */
-static int getBool();
-static int getInt();
-static int getCoord();
-static int getPathList();
-static int getEnum();
-static int getTexture();
-static int getWSBackground();
-static int getWSSpecificBackground();
-static int getFont();
-static int getColor();
-static int getKeybind();
-static int getModMask();
-static int getPropList();
+static WDECallbackConvert getBool;
+static WDECallbackConvert getInt;
+static WDECallbackConvert getCoord;
+static WDECallbackConvert getPathList;
+static WDECallbackConvert getEnum;
+static WDECallbackConvert getTexture;
+static WDECallbackConvert getWSBackground;
+static WDECallbackConvert getWSSpecificBackground;
+static WDECallbackConvert getFont;
+static WDECallbackConvert getColor;
+static WDECallbackConvert getKeybind;
+static WDECallbackConvert getModMask;
+static WDECallbackConvert getPropList;
 
 /* value setting functions */
-static int setJustify();
-static int setClearance();
-static int setIfDockPresent();
-static int setStickyIcons();
-static int setWidgetColor();
-static int setIconTile();
-static int setWinTitleFont();
-static int setMenuTitleFont();
-static int setMenuTextFont();
-static int setIconTitleFont();
-static int setIconTitleColor();
-static int setIconTitleBack();
-static int setLargeDisplayFont();
-static int setWTitleColor();
-static int setFTitleBack();
-static int setPTitleBack();
-static int setUTitleBack();
-static int setResizebarBack();
-static int setWorkspaceBack();
-static int setWorkspaceSpecificBack();
-static int setMenuTitleColor();
-static int setMenuTextColor();
-static int setMenuDisabledColor();
-static int setMenuTitleBack();
-static int setMenuTextBack();
-static int setHightlight();
-static int setHightlightText();
-static int setKeyGrab();
-static int setDoubleClick();
-static int setIconPosition();
+static WDECallbackUpdate setJustify;
+static WDECallbackUpdate setClearance;
+static WDECallbackUpdate setIfDockPresent;
+static WDECallbackUpdate setClipMergedInDock;
+static WDECallbackUpdate setWrapAppiconsInDock;
+static WDECallbackUpdate setStickyIcons;
+static WDECallbackUpdate setWidgetColor;
+static WDECallbackUpdate setIconTile;
+static WDECallbackUpdate setWinTitleFont;
+static WDECallbackUpdate setMenuTitleFont;
+static WDECallbackUpdate setMenuTextFont;
+static WDECallbackUpdate setIconTitleFont;
+static WDECallbackUpdate setIconTitleColor;
+static WDECallbackUpdate setIconTitleBack;
+static WDECallbackUpdate setFrameBorderWidth;
+static WDECallbackUpdate setFrameBorderColor;
+static WDECallbackUpdate setFrameSelectedBorderColor;
+static WDECallbackUpdate setLargeDisplayFont;
+static WDECallbackUpdate setWTitleColor;
+static WDECallbackUpdate setFTitleBack;
+static WDECallbackUpdate setPTitleBack;
+static WDECallbackUpdate setUTitleBack;
+static WDECallbackUpdate setResizebarBack;
+static WDECallbackUpdate setWorkspaceBack;
+static WDECallbackUpdate setWorkspaceSpecificBack;
+static WDECallbackUpdate setMenuTitleColor;
+static WDECallbackUpdate setMenuTextColor;
+static WDECallbackUpdate setMenuDisabledColor;
+static WDECallbackUpdate setMenuTitleBack;
+static WDECallbackUpdate setMenuTextBack;
+static WDECallbackUpdate setHightlight;
+static WDECallbackUpdate setHightlightText;
+static WDECallbackUpdate setKeyGrab;
+static WDECallbackUpdate setDoubleClick;
+static WDECallbackUpdate setIconPosition;
 
-static int setClipTitleFont();
-static int setClipTitleColor();
+static WDECallbackUpdate setClipTitleFont;
+static WDECallbackUpdate setClipTitleColor;
 
-static int setMenuStyle();
-static int setSwPOptions();
-static int updateUsableArea();
+static WDECallbackUpdate setMenuStyle;
+static WDECallbackUpdate setSwPOptions;
+static WDECallbackUpdate updateUsableArea;
 
-static int setModifierKeyLabels();
+static WDECallbackUpdate setModifierKeyLabels;
 
 extern Cursor wCursor[WCUR_LAST];
-static int getCursor();
-static int setCursor();
+static WDECallbackConvert getCursor;
+static WDECallbackUpdate setCursor;
 
 /*
  * Tables to convert strings to enumeration values.
@@ -181,6 +192,10 @@ static int setCursor();
 #define REFRESH_ICON_TITLE_COLOR (1<<13)
 #define REFRESH_ICON_TITLE_BACK (1<<14)
 
+#define REFRESH_WORKSPACE_MENU	(1<<15)
+
+#define REFRESH_FRAME_BORDER REFRESH_MENU_FONT|REFRESH_WINDOW_FONT
+
 static WOptionEnumeration seFocusModes[] = {
 	{"Manual", WKF_CLICK, 0}, {"ClickToFocus", WKF_CLICK, 1},
 	{"Sloppy", WKF_SLOPPY, 0}, {"SemiAuto", WKF_SLOPPY, 1}, {"Auto", WKF_SLOPPY, 1},
@@ -325,6 +340,10 @@ WDefaultEntry staticOptionList[] = {
 	    NULL, getBool, setIfDockPresent, NULL, NULL},
 	{"DisableClip", "NO", (void *)WM_CLIP,
 	    NULL, getBool, setIfDockPresent, NULL, NULL},
+	{"DisableDrawers", "NO", (void *)WM_DRAWER,
+	    NULL, getBool, setIfDockPresent, NULL, NULL},
+	{"ClipMergedInDock", "NO", NULL,
+	    NULL, getBool, setClipMergedInDock, NULL, NULL},
 	{"DisableMiniwindows", "NO", NULL,
 	    &wPreferences.disable_miniwindows, getBool, NULL, NULL, NULL}
 };
@@ -394,6 +413,16 @@ WDefaultEntry optionList[] = {
 	    &wPreferences.do_not_make_appicons_bounce, getBool, NULL, NULL, NULL},
 	{"DoubleClickTime", "250", (void *)&wPreferences.dblclick_time,
 	    &wPreferences.dblclick_time, getInt, setDoubleClick, NULL, NULL},
+	{"ClipAutoraiseDelay", "600", NULL,
+	     &wPreferences.clip_auto_raise_delay, getInt, NULL, NULL, NULL},
+	{"ClipAutolowerDelay", "1000", NULL,
+	    &wPreferences.clip_auto_lower_delay, getInt, NULL, NULL, NULL},
+	{"ClipAutoexpandDelay", "600", NULL,
+	    &wPreferences.clip_auto_expand_delay, getInt, NULL, NULL, NULL},
+	{"ClipAutocollapseDelay", "1000", NULL,
+	    &wPreferences.clip_auto_collapse_delay, getInt, NULL, NULL, NULL},
+	{"WrapAppiconsInDock", "YES", NULL,
+	    NULL, getBool, setWrapAppiconsInDock, NULL, NULL},
 	{"AlignSubmenus", "NO", NULL,
 	    &wPreferences.align_menus, getBool, NULL, NULL, NULL},
 	{"ViKeyMenus", "NO", NULL,
@@ -442,7 +471,7 @@ WDefaultEntry optionList[] = {
 	    &wPreferences.help_balloon, getBool, NULL, NULL, NULL},
 	{"EdgeResistance", "30", NULL,
 	    &wPreferences.edge_resistance, getInt, NULL, NULL, NULL},
-	{"ResizeIncrement", "32", NULL,
+	{"ResizeIncrement", "0", NULL,
 	    &wPreferences.resize_increment, getInt, NULL, NULL, NULL},
 	{"Attraction", "NO", NULL,
 	    &wPreferences.attract, getBool, NULL, NULL, NULL},
@@ -450,6 +479,10 @@ WDefaultEntry optionList[] = {
 	    &wPreferences.dont_blink, getBool, NULL, NULL, NULL},
 	{"SingleClickLaunch",	"NO",	NULL,
 	    &wPreferences.single_click, getBool, NULL, NULL, NULL},
+	{"StrictWindozeCycle",	"YES",	NULL,
+	    &wPreferences.strict_windoze_cycle, getBool, NULL, NULL, NULL},
+	{"SwitchPanelOnlyOpen",	"NO",	NULL,
+	    &wPreferences.panel_only_open, getBool, NULL, NULL, NULL},
 
 	/* style options */
 
@@ -538,6 +571,12 @@ WDefaultEntry optionList[] = {
 	    NULL, getPropList, setSwPOptions, NULL, NULL},
 	{"ModifierKeyLabels", "(\"Shift+\", \"Ctrl+\", \"Mod1+\", \"Mod2+\", \"Mod3+\", \"Mod4+\", \"Mod5+\")", &wPreferences,
 	    NULL, getPropList, setModifierKeyLabels, NULL, NULL},
+	{"FrameBorderWidth", "1", NULL,
+	    NULL, getInt, setFrameBorderWidth, NULL, NULL},
+	{"FrameBorderColor", "black", NULL,
+	    NULL, getColor, setFrameBorderColor, NULL, NULL},
+	{"FrameSelectedBorderColor", "white", NULL,
+	    NULL, getColor, setFrameSelectedBorderColor, NULL, NULL},
 
 	/* keybindings */
 
@@ -573,8 +612,20 @@ WDefaultEntry optionList[] = {
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"RHMaximizeKey", "None", (void*)WKBD_RHMAXIMIZE,
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"THMaximizeKey", "None", (void*)WKBD_THMAXIMIZE,
+		NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"BHMaximizeKey", "None", (void*)WKBD_BHMAXIMIZE,
+		NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"LTCMaximizeKey", "None", (void*)WKBD_LTCMAXIMIZE,
+		NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"RTCMaximizeKey", "None", (void*)WKBD_RTCMAXIMIZE,
+		NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"LBCMaximizeKey", "None", (void*)WKBD_LBCMAXIMIZE,
+		NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"RBCMaximizeKey", "None", (void*)WKBD_RBCMAXIMIZE,
+		NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"MaximusKey", "None", (void*)WKBD_MAXIMUS,
-	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+		NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"RaiseKey", "\"Meta+Up\"", (void *)WKBD_RAISE,
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"LowerKey", "\"Meta+Down\"", (void *)WKBD_LOWER,
@@ -597,6 +648,8 @@ WDefaultEntry optionList[] = {
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"PrevWorkspaceKey", "None", (void *)WKBD_PREVWORKSPACE,
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"LastWorkspaceKey", "None", (void *)WKBD_LASTWORKSPACE,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"NextWorkspaceLayerKey", "None", (void *)WKBD_NEXTWSLAYER,
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"PrevWorkspaceLayerKey", "None", (void *)WKBD_PREVWSLAYER,
@@ -621,6 +674,36 @@ WDefaultEntry optionList[] = {
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"Workspace10Key", "None", (void *)WKBD_WORKSPACE10,
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace1Key", "None", (void *)WKBD_MOVE_WORKSPACE1,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace2Key", "None", (void *)WKBD_MOVE_WORKSPACE2,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace3Key", "None", (void *)WKBD_MOVE_WORKSPACE3,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace4Key", "None", (void *)WKBD_MOVE_WORKSPACE4,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace5Key", "None", (void *)WKBD_MOVE_WORKSPACE5,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace6Key", "None", (void *)WKBD_MOVE_WORKSPACE6,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace7Key", "None", (void *)WKBD_MOVE_WORKSPACE7,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace8Key", "None", (void *)WKBD_MOVE_WORKSPACE8,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace9Key", "None", (void *)WKBD_MOVE_WORKSPACE9,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToWorkspace10Key", "None", (void *)WKBD_MOVE_WORKSPACE10,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToNextWorkspaceKey", "None", (void *)WKBD_MOVE_NEXTWORKSPACE,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToPrevWorkspaceKey", "None", (void *)WKBD_MOVE_PREVWORKSPACE,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToLastWorkspaceKey", "None", (void *)WKBD_MOVE_LASTWORKSPACE,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToNextWorkspaceLayerKey", "None", (void *)WKBD_MOVE_NEXTWSLAYER,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
+	{"MoveToPrevWorkspaceLayerKey", "None", (void *)WKBD_MOVE_PREVWSLAYER,
+	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"WindowShortcut1Key", "None", (void *)WKBD_WINDOW1,
 	    NULL, getKeybind, setKeyGrab, NULL, NULL},
 	{"WindowShortcut2Key", "None", (void *)WKBD_WINDOW2,
@@ -689,14 +772,14 @@ WDefaultEntry optionList[] = {
 	    &wPreferences.cycle_ignore_minimized, getBool, NULL, NULL, NULL}
 };
 
-static void initDefaults()
+static void initDefaults(void)
 {
 	unsigned int i;
 	WDefaultEntry *entry;
 
 	WMPLSetCaseSensitive(False);
 
-	for (i = 0; i < sizeof(optionList) / sizeof(WDefaultEntry); i++) {
+	for (i = 0; i < sizeof(optionList) / sizeof(optionList[0]); i++) {
 		entry = &optionList[i];
 
 		entry->plkey = WMCreatePLString(entry->key);
@@ -706,7 +789,7 @@ static void initDefaults()
 			entry->plvalue = NULL;
 	}
 
-	for (i = 0; i < sizeof(staticOptionList) / sizeof(WDefaultEntry); i++) {
+	for (i = 0; i < sizeof(staticOptionList) / sizeof(staticOptionList[0]); i++) {
 		entry = &staticOptionList[i];
 
 		entry->plkey = WMCreatePLString(entry->key);
@@ -717,7 +800,7 @@ static void initDefaults()
 	}
 }
 
-static WMPropList *readGlobalDomain(char *domainName, Bool requireDictionary)
+static WMPropList *readGlobalDomain(const char *domainName, Bool requireDictionary)
 {
 	WMPropList *globalDict = NULL;
 	char path[PATH_MAX];
@@ -803,12 +886,12 @@ void wDefaultsMergeGlobalMenus(WDDomain * menuDomain)
 	menuDomain->dictionary = menu;
 }
 
-WDDomain *wDefaultsInitDomain(char *domain, Bool requireDictionary)
+WDDomain *wDefaultsInitDomain(const char *domain, Bool requireDictionary)
 {
 	WDDomain *db;
 	struct stat stbuf;
 	static int inited = 0;
-	char *the_path;
+	const char *the_path;
 	WMPropList *shared_dict = NULL;
 
 	if (!inited) {
@@ -861,7 +944,7 @@ void wReadStaticDefaults(WMPropList * dict)
 	unsigned int i;
 	void *tdata;
 
-	for (i = 0; i < sizeof(staticOptionList) / sizeof(WDefaultEntry); i++) {
+	for (i = 0; i < sizeof(staticOptionList) / sizeof(staticOptionList[0]); i++) {
 		entry = &staticOptionList[i];
 
 		if (dict)
@@ -1015,7 +1098,7 @@ void wReadDefaults(WScreen * scr, WMPropList * new_dict)
 
 	needs_refresh = 0;
 
-	for (i = 0; i < sizeof(optionList) / sizeof(WDefaultEntry); i++) {
+	for (i = 0; i < sizeof(optionList) / sizeof(optionList[0]); i++) {
 		entry = &optionList[i];
 
 		if (new_dict)
@@ -1118,12 +1201,24 @@ void wReadDefaults(WScreen * scr, WMPropList * new_dict)
 		}
 		if (needs_refresh & REFRESH_ICON_TILE)
 			WMPostNotificationName(WNIconTileSettingsChanged, NULL, NULL);
+
+		if (needs_refresh & REFRESH_WORKSPACE_MENU) {
+			if (scr->workspace_menu)
+				wWorkspaceMenuUpdate(scr, scr->workspace_menu);
+			if (scr->clip_ws_menu)
+				wWorkspaceMenuUpdate(scr, scr->clip_ws_menu);
+			if (scr->workspace_submenu)
+				scr->workspace_submenu->flags.realized = 0;
+			if (scr->clip_submenu)
+				scr->clip_submenu->flags.realized = 0;
+		}
 	}
 }
 
 void wDefaultUpdateIcons(WScreen *scr)
 {
 	WAppIcon *aicon = scr->app_icon_list;
+	WDrawerChain *dc;
 	WWindow *wwin = scr->focused_window;
 
 	while (aicon) {
@@ -1133,9 +1228,12 @@ void wDefaultUpdateIcons(WScreen *scr)
 		aicon = aicon->next;
 	}
 
-	if (!wPreferences.flags.noclip)
+	if (!wPreferences.flags.noclip || wPreferences.flags.clip_merged_in_dock)
 		wClipIconPaint(scr->clip_icon);
 
+	for (dc = scr->drawers; dc != NULL; dc = dc->next)
+		wDrawerIconPaint(dc->adrawer->icon_array[0]);
+
 	while (wwin) {
 		if (wwin->icon && wwin->flags.miniaturized)
 			wIconChangeImageFile(wwin->icon, NULL);
@@ -1153,7 +1251,7 @@ void wDefaultUpdateIcons(WScreen *scr)
     } else var = WMGetFromPLString(value)\
 
 
-static int string2index(WMPropList * key, WMPropList * val, char *def, WOptionEnumeration * values)
+static int string2index(WMPropList *key, WMPropList *val, const char *def, WOptionEnumeration * values)
 {
 	char *str;
 	WOptionEnumeration *v;
@@ -1196,7 +1294,7 @@ static int string2index(WMPropList * key, WMPropList * val, char *def, WOptionEn
 static int getBool(WScreen * scr, WDefaultEntry * entry, WMPropList * value, void *addr, void **ret)
 {
 	static char data;
-	char *val;
+	const char *val;
 	int second_pass = 0;
 
 	GET_STRING_OR_DEFAULT("Boolean", val);
@@ -1239,7 +1337,7 @@ static int getBool(WScreen * scr, WDefaultEntry * entry, WMPropList * value, voi
 static int getInt(WScreen * scr, WDefaultEntry * entry, WMPropList * value, void *addr, void **ret)
 {
 	static int data;
-	char *val;
+	const char *val;
 
 	GET_STRING_OR_DEFAULT("Integer", val);
 
@@ -1910,7 +2008,7 @@ getWSSpecificBackground(WScreen * scr, WDefaultEntry * entry, WMPropList * value
 static int getFont(WScreen * scr, WDefaultEntry * entry, WMPropList * value, void *addr, void **ret)
 {
 	static WMFont *font;
-	char *val;
+	const char *val;
 
 	GET_STRING_OR_DEFAULT("Font", val);
 
@@ -1935,7 +2033,7 @@ static int getFont(WScreen * scr, WDefaultEntry * entry, WMPropList * value, voi
 static int getColor(WScreen * scr, WDefaultEntry * entry, WMPropList * value, void *addr, void **ret)
 {
 	static XColor color;
-	char *val;
+	const char *val;
 	int second_pass = 0;
 
 	GET_STRING_OR_DEFAULT("Color", val);
@@ -1968,7 +2066,7 @@ static int getKeybind(WScreen * scr, WDefaultEntry * entry, WMPropList * value,
 {
 	static WShortKey shortcut;
 	KeySym ksym;
-	char *val;
+	const char *val;
 	char *k;
 	char buf[MAX_SHORTCUT_LENGTH], *b;
 
@@ -2025,7 +2123,7 @@ static int getKeybind(WScreen * scr, WDefaultEntry * entry, WMPropList * value,
 static int getModMask(WScreen * scr, WDefaultEntry * entry, WMPropList * value, void *addr, void **ret)
 {
 	static int mask;
-	char *str;
+	const char *str;
 
 	GET_STRING_OR_DEFAULT("Modifier Key", str);
 
@@ -2050,13 +2148,13 @@ static int getModMask(WScreen * scr, WDefaultEntry * entry, WMPropList * value,
 
 # include <X11/cursorfont.h>
 typedef struct {
-	char *name;
+	const char *name;
 	int id;
 } WCursorLookup;
 
 #define CURSOR_ID_NONE	(XC_num_glyphs)
 
-static WCursorLookup cursor_table[] = {
+static const WCursorLookup cursor_table[] = {
 	{"X_cursor", XC_X_cursor},
 	{"arrow", XC_arrow},
 	{"based_arrow_down", XC_based_arrow_down},
@@ -2137,7 +2235,7 @@ static WCursorLookup cursor_table[] = {
 	{NULL, CURSOR_ID_NONE}
 };
 
-static void check_bitmap_status(int status, char *filename, Pixmap bitmap)
+static void check_bitmap_status(int status, const char *filename, Pixmap bitmap)
 {
 	switch (status) {
 	case BitmapOpenFailed:
@@ -2302,7 +2400,7 @@ static int getCursor(WScreen * scr, WDefaultEntry * entry, WMPropList * value, v
 #undef CURSOR_ID_NONE
 
 /* ---------------- value setting functions --------------- */
-static int setJustify(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setJustify(WScreen * scr, WDefaultEntry * entry, void *tdata, void *extra_data)
 {
 	return REFRESH_WINDOW_TITLE_COLOR;
 }
@@ -2312,21 +2410,46 @@ static int setClearance(WScreen * scr, WDefaultEntry * entry, void *bar, void *f
 	return REFRESH_WINDOW_FONT | REFRESH_BUTTON_IMAGES | REFRESH_MENU_TITLE_FONT | REFRESH_MENU_FONT;
 }
 
-static int setIfDockPresent(WScreen * scr, WDefaultEntry * entry, char *flag, long which)
+static int setIfDockPresent(WScreen * scr, WDefaultEntry * entry, void *tdata, void *extra_data)
 {
+	char *flag = tdata;
+	long which = (long) extra_data;
+
 	switch (which) {
 	case WM_DOCK:
 		wPreferences.flags.nodock = wPreferences.flags.nodock || *flag;
+		// Drawers require the dock
+		wPreferences.flags.nodrawer = wPreferences.flags.nodrawer || wPreferences.flags.nodock;
 		break;
 	case WM_CLIP:
 		wPreferences.flags.noclip = wPreferences.flags.noclip || *flag;
 		break;
+	case WM_DRAWER:
+		wPreferences.flags.nodrawer = wPreferences.flags.nodrawer || *flag;
+		break;
 	default:
 		break;
 	}
 	return 0;
 }
 
+static int setClipMergedInDock(WScreen *scr, WDefaultEntry *entry, void *tdata, void *foo)
+{
+	char *flag = tdata;
+
+	wPreferences.flags.clip_merged_in_dock = *flag;
+	wPreferences.flags.noclip = wPreferences.flags.noclip || *flag;
+	return 0;
+}
+
+static int setWrapAppiconsInDock(WScreen *scr, WDefaultEntry *entry, void *tdata, void *foo)
+{
+	char *flag = tdata;
+
+	wPreferences.flags.wrap_appicons_in_dock = *flag;
+	return 0;
+}
+
 static int setStickyIcons(WScreen * scr, WDefaultEntry * entry, void *bar, void *foo)
 {
 	if (scr->workspaces) {
@@ -2336,10 +2459,11 @@ static int setStickyIcons(WScreen * scr, WDefaultEntry * entry, void *bar, void
 	return 0;
 }
 
-static int setIconTile(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setIconTile(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
 	Pixmap pixmap;
 	RImage *img;
+	WTexture ** texture = tdata;
 	int reset = 0;
 
 	img = wTextureRenderImage(*texture, wPreferences.icon_size,
@@ -2364,13 +2488,20 @@ static int setIconTile(WScreen * scr, WDefaultEntry * entry, WTexture ** texture
 	/* put the icon in the noticeboard hint */
 	PropSetIconTileHint(scr, img);
 
-	if (!wPreferences.flags.noclip) {
+	if (!wPreferences.flags.noclip || wPreferences.flags.clip_merged_in_dock) {
 		if (scr->clip_tile) {
 			RReleaseImage(scr->clip_tile);
 		}
 		scr->clip_tile = wClipMakeTile(scr, img);
 	}
 
+	if (!wPreferences.flags.nodrawer) {
+		if (scr->drawer_tile) {
+			RReleaseImage(scr->drawer_tile);
+		}
+		scr->drawer_tile = wDrawerMakeTile(scr, img);
+	}
+
 	scr->icon_tile_pixmap = pixmap;
 
 	if (scr->def_icon_rimage) {
@@ -2383,20 +2514,17 @@ static int setIconTile(WScreen * scr, WDefaultEntry * entry, WTexture ** texture
 
 	scr->icon_back_texture = wTextureMakeSolid(scr, &((*texture)->any.color));
 
-	if (scr->clip_balloon)
-		XSetWindowBackground(dpy, scr->clip_balloon, (*texture)->any.color.pixel);
-
-	/*
-	 * Free the texture as nobody else will use it, nor refer to it.
-	 */
+	/* Free the texture as nobody else will use it, nor refer to it.  */
 	if (!entry->addr)
 		wTextureDestroy(scr, *texture);
 
 	return (reset ? REFRESH_ICON_TILE : 0);
 }
 
-static int setWinTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font, void *foo)
+static int setWinTitleFont(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMFont *font = tdata;
+
 	if (scr->title_font) {
 		WMReleaseFont(scr->title_font);
 	}
@@ -2405,8 +2533,10 @@ static int setWinTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font,
 	return REFRESH_WINDOW_FONT | REFRESH_BUTTON_IMAGES;
 }
 
-static int setMenuTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font, void *foo)
+static int setMenuTitleFont(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMFont *font = tdata;
+
 	if (scr->menu_title_font) {
 		WMReleaseFont(scr->menu_title_font);
 	}
@@ -2416,8 +2546,10 @@ static int setMenuTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font,
 	return REFRESH_MENU_TITLE_FONT;
 }
 
-static int setMenuTextFont(WScreen * scr, WDefaultEntry * entry, WMFont * font, void *foo)
+static int setMenuTextFont(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMFont *font = tdata;
+
 	if (scr->menu_entry_font) {
 		WMReleaseFont(scr->menu_entry_font);
 	}
@@ -2426,8 +2558,10 @@ static int setMenuTextFont(WScreen * scr, WDefaultEntry * entry, WMFont * font,
 	return REFRESH_MENU_FONT;
 }
 
-static int setIconTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font, void *foo)
+static int setIconTitleFont(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMFont *font = tdata;
+
 	if (scr->icon_title_font) {
 		WMReleaseFont(scr->icon_title_font);
 	}
@@ -2437,8 +2571,10 @@ static int setIconTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font,
 	return REFRESH_ICON_FONT;
 }
 
-static int setClipTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font, void *foo)
+static int setClipTitleFont(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMFont *font = tdata;
+
 	if (scr->clip_title_font) {
 		WMReleaseFont(scr->clip_title_font);
 	}
@@ -2448,8 +2584,10 @@ static int setClipTitleFont(WScreen * scr, WDefaultEntry * entry, WMFont * font,
 	return REFRESH_ICON_FONT;
 }
 
-static int setLargeDisplayFont(WScreen * scr, WDefaultEntry * entry, WMFont * font, void *foo)
+static int setLargeDisplayFont(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMFont *font = tdata;
+
 	if (scr->workspace_name_font) {
 		WMReleaseFont(scr->workspace_name_font);
 	}
@@ -2459,8 +2597,10 @@ static int setLargeDisplayFont(WScreen * scr, WDefaultEntry * entry, WMFont * fo
 	return 0;
 }
 
-static int setHightlight(WScreen * scr, WDefaultEntry * entry, XColor * color, void *foo)
+static int setHightlight(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	XColor *color = tdata;
+
 	if (scr->select_color)
 		WMReleaseColor(scr->select_color);
 
@@ -2471,8 +2611,10 @@ static int setHightlight(WScreen * scr, WDefaultEntry * entry, XColor * color, v
 	return REFRESH_MENU_COLOR;
 }
 
-static int setHightlightText(WScreen * scr, WDefaultEntry * entry, XColor * color, void *foo)
+static int setHightlightText(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	XColor *color = tdata;
+
 	if (scr->select_text_color)
 		WMReleaseColor(scr->select_text_color);
 
@@ -2483,8 +2625,11 @@ static int setHightlightText(WScreen * scr, WDefaultEntry * entry, XColor * colo
 	return REFRESH_MENU_COLOR;
 }
 
-static int setClipTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * color, long widx)
+static int setClipTitleColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *extra_data)
 {
+	XColor *color = tdata;
+	long widx = (long) extra_data;
+
 	if (scr->clip_title_color[widx])
 		WMReleaseColor(scr->clip_title_color[widx]);
 
@@ -2494,8 +2639,11 @@ static int setClipTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * colo
 	return REFRESH_ICON_TITLE_COLOR;
 }
 
-static int setWTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * color, long widx)
+static int setWTitleColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *extra_data)
 {
+	XColor *color = tdata;
+	long widx = (long) extra_data;
+
 	if (scr->window_title_color[widx])
 		WMReleaseColor(scr->window_title_color[widx]);
 
@@ -2507,8 +2655,10 @@ static int setWTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * color,
 	return REFRESH_WINDOW_TITLE_COLOR;
 }
 
-static int setMenuTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * color, long widx)
+static int setMenuTitleColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *extra_data)
 {
+	XColor *color = tdata;
+
 	if (scr->menu_title_color[0])
 		WMReleaseColor(scr->menu_title_color[0]);
 
@@ -2519,8 +2669,10 @@ static int setMenuTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * colo
 	return REFRESH_MENU_TITLE_COLOR;
 }
 
-static int setMenuTextColor(WScreen * scr, WDefaultEntry * entry, XColor * color, void *foo)
+static int setMenuTextColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	XColor *color = tdata;
+
 	if (scr->mtext_color)
 		WMReleaseColor(scr->mtext_color);
 
@@ -2537,8 +2689,10 @@ static int setMenuTextColor(WScreen * scr, WDefaultEntry * entry, XColor * color
 	return REFRESH_MENU_COLOR;
 }
 
-static int setMenuDisabledColor(WScreen * scr, WDefaultEntry * entry, XColor * color, void *foo)
+static int setMenuDisabledColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	XColor *color = tdata;
+
 	if (scr->dtext_color)
 		WMReleaseColor(scr->dtext_color);
 
@@ -2555,8 +2709,10 @@ static int setMenuDisabledColor(WScreen * scr, WDefaultEntry * entry, XColor * c
 	return REFRESH_MENU_COLOR;
 }
 
-static int setIconTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * color, void *foo)
+static int setIconTitleColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	XColor *color = tdata;
+
 	if (scr->icon_title_color)
 		WMReleaseColor(scr->icon_title_color);
 	scr->icon_title_color = WMCreateRGBColor(scr->wmscreen, color->red, color->green, color->blue, True);
@@ -2566,8 +2722,10 @@ static int setIconTitleColor(WScreen * scr, WDefaultEntry * entry, XColor * colo
 	return REFRESH_ICON_TITLE_COLOR;
 }
 
-static int setIconTitleBack(WScreen * scr, WDefaultEntry * entry, XColor * color, void *foo)
+static int setIconTitleBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	XColor *color = tdata;
+
 	if (scr->icon_title_texture) {
 		wTextureDestroy(scr, (WTexture *) scr->icon_title_texture);
 	}
@@ -2576,6 +2734,41 @@ static int setIconTitleBack(WScreen * scr, WDefaultEntry * entry, XColor * color
 	return REFRESH_ICON_TITLE_BACK;
 }
 
+static int setFrameBorderWidth(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
+{
+	int *value = tdata;
+
+	scr->frame_border_width = *value;
+
+	return REFRESH_FRAME_BORDER;
+}
+
+static int setFrameBorderColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
+{
+	XColor *color = tdata;
+
+	if (scr->frame_border_color)
+		WMReleaseColor(scr->frame_border_color);
+	scr->frame_border_color = WMCreateRGBColor(scr->wmscreen, color->red, color->green, color->blue, True);
+
+	wFreeColor(scr, color->pixel);
+
+	return REFRESH_FRAME_BORDER;
+}
+
+static int setFrameSelectedBorderColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
+{
+	XColor *color = tdata;
+
+	if (scr->frame_selected_border_color)
+		WMReleaseColor(scr->frame_selected_border_color);
+	scr->frame_selected_border_color = WMCreateRGBColor(scr->wmscreen, color->red, color->green, color->blue, True);
+
+	wFreeColor(scr, color->pixel);
+
+	return REFRESH_FRAME_BORDER;
+}
+
 static void trackDeadProcess(pid_t pid, unsigned char status, WScreen * scr)
 {
 	close(scr->helper_fd);
@@ -2584,8 +2777,9 @@ static void trackDeadProcess(pid_t pid, unsigned char status, WScreen * scr)
 	scr->flags.backimage_helper_launched = 0;
 }
 
-static int setWorkspaceSpecificBack(WScreen * scr, WDefaultEntry * entry, WMPropList * value, void *bar)
+static int setWorkspaceSpecificBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *bar)
 {
+	WMPropList *value = tdata;
 	WMPropList *val;
 	char *str;
 	int i;
@@ -2675,8 +2869,10 @@ static int setWorkspaceSpecificBack(WScreen * scr, WDefaultEntry * entry, WMProp
 	return 0;
 }
 
-static int setWorkspaceBack(WScreen * scr, WDefaultEntry * entry, WMPropList * value, void *bar)
+static int setWorkspaceBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *bar)
 {
+	WMPropList *value = tdata;
+
 	if (scr->flags.backimage_helper_launched) {
 		char *str;
 
@@ -2716,8 +2912,10 @@ static int setWorkspaceBack(WScreen * scr, WDefaultEntry * entry, WMPropList * v
 	return 0;
 }
 
-static int setWidgetColor(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setWidgetColor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WTexture **texture = tdata;
+
 	if (scr->widget_texture) {
 		wTextureDestroy(scr, (WTexture *) scr->widget_texture);
 	}
@@ -2726,8 +2924,10 @@ static int setWidgetColor(WScreen * scr, WDefaultEntry * entry, WTexture ** text
 	return 0;
 }
 
-static int setFTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setFTitleBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WTexture **texture = tdata;
+
 	if (scr->window_title_texture[WS_FOCUSED]) {
 		wTextureDestroy(scr, scr->window_title_texture[WS_FOCUSED]);
 	}
@@ -2736,8 +2936,10 @@ static int setFTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** textu
 	return REFRESH_WINDOW_TEXTURES;
 }
 
-static int setPTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setPTitleBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WTexture **texture = tdata;
+
 	if (scr->window_title_texture[WS_PFOCUSED]) {
 		wTextureDestroy(scr, scr->window_title_texture[WS_PFOCUSED]);
 	}
@@ -2746,8 +2948,10 @@ static int setPTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** textu
 	return REFRESH_WINDOW_TEXTURES;
 }
 
-static int setUTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setUTitleBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WTexture **texture = tdata;
+
 	if (scr->window_title_texture[WS_UNFOCUSED]) {
 		wTextureDestroy(scr, scr->window_title_texture[WS_UNFOCUSED]);
 	}
@@ -2756,8 +2960,10 @@ static int setUTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** textu
 	return REFRESH_WINDOW_TEXTURES;
 }
 
-static int setResizebarBack(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setResizebarBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WTexture **texture = tdata;
+
 	if (scr->resizebar_texture[0]) {
 		wTextureDestroy(scr, scr->resizebar_texture[0]);
 	}
@@ -2766,8 +2972,10 @@ static int setResizebarBack(WScreen * scr, WDefaultEntry * entry, WTexture ** te
 	return REFRESH_WINDOW_TEXTURES;
 }
 
-static int setMenuTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setMenuTitleBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WTexture **texture = tdata;
+
 	if (scr->menu_title_texture[0]) {
 		wTextureDestroy(scr, scr->menu_title_texture[0]);
 	}
@@ -2776,8 +2984,10 @@ static int setMenuTitleBack(WScreen * scr, WDefaultEntry * entry, WTexture ** te
 	return REFRESH_MENU_TITLE_TEXTURE;
 }
 
-static int setMenuTextBack(WScreen * scr, WDefaultEntry * entry, WTexture ** texture, void *foo)
+static int setMenuTextBack(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WTexture **texture = tdata;
+
 	if (scr->menu_item_texture) {
 		wTextureDestroy(scr, scr->menu_item_texture);
 		wTextureDestroy(scr, (WTexture *) scr->menu_item_auxtexture);
@@ -2789,9 +2999,12 @@ static int setMenuTextBack(WScreen * scr, WDefaultEntry * entry, WTexture ** tex
 	return REFRESH_MENU_TEXTURE;
 }
 
-static int setKeyGrab(WScreen * scr, WDefaultEntry * entry, WShortKey * shortcut, long widx)
+static int setKeyGrab(WScreen * scr, WDefaultEntry * entry, void *tdata, void *extra_data)
 {
+	WShortKey *shortcut = tdata;
 	WWindow *wwin;
+	long widx = (long) extra_data;
+
 	wKeyBindings[widx] = *shortcut;
 
 	wwin = scr->focused_window;
@@ -2805,6 +3018,14 @@ static int setKeyGrab(WScreen * scr, WDefaultEntry * entry, WShortKey * shortcut
 		wwin = wwin->prev;
 	}
 
+	/* do we need to update window menus? */
+	if (widx >= WKBD_WORKSPACE1 && widx <= WKBD_WORKSPACE10)
+		return REFRESH_WORKSPACE_MENU;
+	if (widx == WKBD_LASTWORKSPACE)
+		return REFRESH_WORKSPACE_MENU;
+	if (widx >= WKBD_MOVE_WORKSPACE1 && widx <= WKBD_MOVE_WORKSPACE10)
+		return REFRESH_WORKSPACE_MENU;
+
 	return 0;
 }
 
@@ -2823,7 +3044,7 @@ static int updateUsableArea(WScreen * scr, WDefaultEntry * entry, void *bar, voi
 	return 0;
 }
 
-static int setMenuStyle(WScreen * scr, WDefaultEntry * entry, int *value, void *foo)
+static int setMenuStyle(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
 	return REFRESH_MENU_TEXTURE;
 }
@@ -2837,12 +3058,13 @@ static RImage *chopOffImage(RImage * image, int x, int y, int w, int h)
 	return img;
 }
 
-static int setSwPOptions(WScreen * scr, WDefaultEntry * entry, WMPropList * array, void *foo)
+static int setSwPOptions(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMPropList *array = tdata;
 	char *path;
 	RImage *bgimage;
 	int cwidth, cheight;
-	WPreferences *prefs = (WPreferences *) foo;
+	struct WPreferences *prefs = foo;
 
 	if (!WMIsPLArray(array) || WMGetPropListItemCount(array) == 0) {
 		if (prefs->swtileImage)
@@ -2955,10 +3177,11 @@ static int setSwPOptions(WScreen * scr, WDefaultEntry * entry, WMPropList * arra
 	return 0;
 }
 
-static int setModifierKeyLabels(WScreen * scr, WDefaultEntry * entry, WMPropList * array, void *foo)
+static int setModifierKeyLabels(WScreen * scr, WDefaultEntry * entry, void *tdata, void *foo)
 {
+	WMPropList *array = tdata;
 	int i;
-	WPreferences *prefs = (WPreferences *) foo;
+	struct WPreferences *prefs = foo;
 
 	if (!WMIsPLArray(array) || WMGetPropListItemCount(array) != 7) {
 		wwarning(_("Value for option \"%s\" must be an array of 7 strings"), entry->key);
@@ -2985,28 +3208,23 @@ static int setModifierKeyLabels(WScreen * scr, WDefaultEntry * entry, WMPropList
 	return 0;
 }
 
-/*
- * Very ugly kluge.
- * Need access to the double click variables, so that all widgets in
- * wmaker panels will have the same dbl-click values.
- * TODO: figure a better way of dealing with it.
- */
-#include <WINGs/WINGsP.h>
-
-static int setDoubleClick(WScreen * scr, WDefaultEntry * entry, int *value, void *foo)
+static int setDoubleClick(WScreen *scr, WDefaultEntry *entry, void *tdata, void *foo)
 {
-	extern _WINGsConfiguration WINGsConfiguration;
+	int *value = tdata;
 
 	if (*value <= 0)
 		*(int *)foo = 1;
 
-	WINGsConfiguration.doubleClickDelay = *value;
+	W_setconf_doubleClickDelay(*value);
 
 	return 0;
 }
 
-static int setCursor(WScreen * scr, WDefaultEntry * entry, Cursor * cursor, long widx)
+static int setCursor(WScreen * scr, WDefaultEntry * entry, void *tdata, void *extra_data)
 {
+	Cursor *cursor = tdata;
+	long widx = (long) extra_data;
+
 	if (wCursor[widx] != None) {
 		XFreeCursor(dpy, wCursor[widx]);
 	}
