@@ -51,22 +51,22 @@
 #include "WindowMaker.h"
 #include "GNUstep.h"
 #include "screen.h"
+#include "window.h"
 #include "dialog.h"
-#include "funcs.h"
+#include "misc.h"
 #include "stacking.h"
 #include "framewin.h"
 #include "window.h"
 #include "actions.h"
 #include "xinerama.h"
 
-extern WPreferences wPreferences;
 
 static WMPoint getCenter(WScreen * scr, int width, int height)
 {
 	return wGetPointToCenterRectInHead(scr, wGetHeadForPointerLocation(scr), width, height);
 }
 
-int wMessageDialog(WScreen * scr, char *title, char *message, char *defBtn, char *altBtn, char *othBtn)
+int wMessageDialog(WScreen *scr, const char *title, const char *message, const char *defBtn, const char *altBtn, const char *othBtn)
 {
 	WMAlertPanel *panel;
 	Window parent;
@@ -108,7 +108,7 @@ static void toggleSaveSession(WMWidget *w, void *data)
 	wPreferences.save_session_on_exit = WMGetButtonSelected((WMButton *) w);
 }
 
-int wExitDialog(WScreen * scr, char *title, char *message, char *defBtn, char *altBtn, char *othBtn)
+int wExitDialog(WScreen *scr, const char *title, const char *message, const char *defBtn, const char *altBtn, const char *othBtn)
 {
 	WMAlertPanel *panel;
 	WMButton *saveSessionBtn;
@@ -167,7 +167,7 @@ typedef struct _WMInputPanelWithHistory {
 	int varpos;
 } WMInputPanelWithHistory;
 
-static char *HistoryFileName(char *name)
+static char *HistoryFileName(const char *name)
 {
 	char *filename = NULL;
 
@@ -185,35 +185,50 @@ static int strmatch(const void *str1, const void *str2)
 	return !strcmp((const char *)str1, (const char *)str2);
 }
 
-static WMArray *LoadHistory(char *filename, int max)
+static WMArray *LoadHistory(const char *filename, int max)
 {
 	WMPropList *plhistory;
 	WMPropList *plitem;
 	WMArray *history;
 	int i, num;
+	char *str;
 
 	history = WMCreateArrayWithDestructor(1, wfree);
 	WMAddToArray(history, wstrdup(""));
 
-	plhistory = WMReadPropListFromFile((char *)filename);
+	plhistory = WMReadPropListFromFile(filename);
 
-	if (plhistory && WMIsPLArray(plhistory)) {
-		num = WMGetPropListItemCount(plhistory);
-		if (num > max)
-			num = max;
+	if (plhistory) {
+		if (WMIsPLArray(plhistory)) {
+			num = WMGetPropListItemCount(plhistory);
 
-		for (i = 0; i < num; ++i) {
-			plitem = WMGetFromPLArray(plhistory, i);
-			if (WMIsPLString(plitem) && WMFindInArray(history, strmatch,
-								  WMGetFromPLString(plitem)) == WANotFound)
-				WMAddToArray(history, WMGetFromPLString(plitem));
+			for (i = 0; i < num; ++i) {
+				plitem = WMGetFromPLArray(plhistory, i);
+				if (WMIsPLString(plitem)) {
+					str = WMGetFromPLString(plitem);
+					if (WMFindInArray(history, strmatch, str) == WANotFound) {
+						/*
+						 * The string here is duplicated because it will be freed
+						 * automatically when the array is deleted. This is not really
+						 * great because it is already an allocated string,
+						 * unfortunately we cannot re-use it because it will be freed
+						 * when we discard the PL (and we don't want to waste the PL's
+						 * memory either)
+						 */
+						WMAddToArray(history, wstrdup(str));
+						if (--max <= 0)
+							break;
+					}
+				}
+			}
 		}
+		WMReleasePropList(plhistory);
 	}
 
 	return history;
 }
 
-static void SaveHistory(WMArray * history, char *filename)
+static void SaveHistory(WMArray * history, const char *filename)
 {
 	int i;
 	WMPropList *plhistory;
@@ -223,7 +238,7 @@ static void SaveHistory(WMArray * history, char *filename)
 	for (i = 0; i < WMGetArrayItemCount(history); ++i)
 		WMAddToPLArray(plhistory, WMCreatePLString(WMGetFromArray(history, i)));
 
-	WMWritePropListToFile(plhistory, (char *)filename);
+	WMWritePropListToFile(plhistory, filename);
 	WMReleasePropList(plhistory);
 }
 
@@ -391,7 +406,7 @@ static void handleHistoryKeyPress(XEvent * event, void *clientData)
 	}
 }
 
-int wAdvancedInputDialog(WScreen * scr, char *title, char *message, char *name, char **text)
+int wAdvancedInputDialog(WScreen *scr, const char *title, const char *message, const char *name, char **text)
 {
 	WWindow *wwin;
 	Window parent;
@@ -455,7 +470,7 @@ int wAdvancedInputDialog(WScreen * scr, char *title, char *message, char *name,
 	}
 }
 
-int wInputDialog(WScreen * scr, char *title, char *message, char **text)
+int wInputDialog(WScreen *scr, const char *title, const char *message, char **text)
 {
 	WWindow *wwin;
 	Window parent;
@@ -539,7 +554,7 @@ typedef struct IconPanel {
 	short preview;
 } IconPanel;
 
-static void listPixmaps(WScreen * scr, WMList * lPtr, char *path)
+static void listPixmaps(WScreen *scr, WMList *lPtr, const char *path)
 {
 	struct dirent *dentry;
 	DIR *dir;
@@ -592,7 +607,7 @@ static void listPixmaps(WScreen * scr, WMList * lPtr, char *path)
 	panel->preview = True;
 }
 
-static void setViewedImage(IconPanel * panel, char *file)
+static void setViewedImage(IconPanel *panel, const char *file)
 {
 	WMPixmap *pixmap;
 	RColor color;
@@ -875,7 +890,7 @@ static void keyPressHandler(XEvent * event, void *data)
 	}
 }
 
-Bool wIconChooserDialog(WScreen * scr, char **file, char *instance, char *class)
+Bool wIconChooserDialog(WScreen *scr, char **file, const char *instance, const char *class)
 {
 	WWindow *wwin;
 	Window parent;
@@ -1103,8 +1118,9 @@ void wShowInfoPanel(WScreen * scr)
 	WMPixmap *logo;
 	WMFont *font;
 	char *strbuf = NULL;
+	const char *separator;
 	char buffer[256];
-	char *name;
+	const char *name;
 	Window parent;
 	WWindow *wwin;
 	char **strl;
@@ -1228,8 +1244,22 @@ void wShowInfoPanel(WScreen * scr)
 	{
 		struct mallinfo ma = mallinfo();
 		snprintf(buffer, sizeof(buffer),
-			 _("Total memory allocated: %i kB (in use: %i kB).\n"),
-			 (ma.arena + ma.hblkhd) / 1024, (ma.uordblks + ma.hblkhd) / 1024);
+#ifdef DEBUG
+					_("Total memory allocated: %i kB (in use: %i kB, %d free chunks).\n"),
+#else
+					_("Total memory allocated: %i kB (in use: %i kB).\n"),
+#endif
+					(ma.arena + ma.hblkhd) / 1024,
+					(ma.uordblks + ma.hblkhd) / 1024
+#ifdef DEBUG
+					/*
+					 * This information is representative of the memory
+					 * fragmentation. In ideal case it should be 1, but
+					 * that is never possible
+					 */
+					, ma.ordblks
+#endif
+					);
 
 		strbuf = wstrappend(strbuf, buffer);
 	}
@@ -1237,14 +1267,27 @@ void wShowInfoPanel(WScreen * scr)
 
 	strbuf = wstrappend(strbuf, _("Supported image formats: "));
 	strl = RSupportedFileFormats();
+	separator = NULL;
 	for (i = 0; strl[i] != NULL; i++) {
+		if (separator != NULL)
+			strbuf = wstrappend(strbuf, separator);
 		strbuf = wstrappend(strbuf, strl[i]);
-		strbuf = wstrappend(strbuf, " ");
+		separator = ", ";
 	}
 
-	strbuf = wstrappend(strbuf, _("\nAdditional support for: WMSPEC"));
+	strbuf = wstrappend(strbuf, _("\nAdditional support for: "));
+	strbuf = wstrappend(strbuf, "WMSPEC");
+
+#ifdef HAVE_XRANDR
+	strbuf = wstrappend(strbuf, ", XRandR ");
+	if (has_randr)
+		strbuf = wstrappend(strbuf, _("(Supported)"));
+	else
+		strbuf = wstrappend(strbuf, _("(Unsupported)"));
+#endif
+
 #ifdef MWM_HINTS
-	strbuf = wstrappend(strbuf, " and MWM");
+	strbuf = wstrappend(strbuf, ", MWM");
 #endif
 
 #ifdef XINERAMA
@@ -1255,7 +1298,7 @@ void wShowInfoPanel(WScreen * scr)
 	strbuf = wstrappend(strbuf, _("Xinerama: "));
 	{
 		char tmp[128];
-		snprintf(tmp, sizeof(tmp) - 1, "%d heads found.", scr->xine_info.count);
+		snprintf(tmp, sizeof(tmp) - 1, _("%d heads found."), scr->xine_info.count);
 		strbuf = wstrappend(strbuf, tmp);
 	}
 #endif
