@@ -23,10 +23,7 @@
 #ifndef WMDOCK_H_
 #define WMDOCK_H_
 
-
 #include "appicon.h"
-#include "funcs.h"
-
 
 typedef struct WDock {
     WScreen *screen_ptr;
@@ -39,8 +36,7 @@ typedef struct WDock {
 
 #define WM_DOCK        0
 #define WM_CLIP        1
-#define NORMAL_DOCK    WM_DOCK
-#define EXTENDED_DOCK  WM_CLIP
+#define WM_DRAWER      2
     int type;
 
     WMagicNumber auto_expand_magic;
@@ -68,7 +64,7 @@ typedef struct WDock {
 
 
 
-WDock *wDockCreate(WScreen *scr, int type);
+WDock *wDockCreate(WScreen *scr, int type, const char *name);
 WDock *wDockRestoreState(WScreen *scr, WMPropList *dock_state, int type);
 
 void wDockDestroy(WDock *dock);
@@ -84,6 +80,11 @@ Bool wDockSnapIcon(WDock *dock, WAppIcon *icon, int req_x, int req_y,
                    int *ret_x, int *ret_y, int redocking);
 Bool wDockFindFreeSlot(WDock *dock, int *req_x, int *req_y);
 void wDockDetach(WDock *dock, WAppIcon *icon);
+Bool wDockMoveIconBetweenDocks(WDock *src, WDock *dest, WAppIcon *icon, int x, int y);
+void wDockReattachIcon(WDock *dock, WAppIcon *icon, int x, int y);
+
+void wSlideAppicons(WAppIcon **appicons, int n, int to_the_left);
+void wDrawerFillTheGap(WDock *drawer, WAppIcon *aicon, Bool redocking);
 
 void wDockFinishLaunch(WDock *dock, WAppIcon *icon);
 void wDockTrackWindowLaunch(WDock *dock, Window window);
@@ -100,14 +101,27 @@ void wClipSaveState(WScreen *scr);
 WMPropList *wClipSaveWorkspaceState(WScreen *scr, int workspace);
 WAppIcon *wClipRestoreState(WScreen *scr, WMPropList *clip_state);
 
+void wDrawerIconPaint(WAppIcon *dicon);
+void wDrawersSaveState(WScreen *scr);
+void wDrawersRestoreState(WScreen *scr);
+int wIsADrawer(WScreen *scr, WAppIcon *aicon);
+
 void wClipUpdateForWorkspaceChange(WScreen *scr, int workspace);
 
 RImage *wClipMakeTile(WScreen *scr, RImage *normalTile);
+RImage* wDrawerMakeTile(WScreen *scr, RImage *normalTile);
 
 #define WO_FAILED          0
 #define WO_NOT_APPLICABLE  1
 #define WO_SUCCESS         2
 
+typedef enum
+{
+	P_NORMAL = 0,
+	P_AUTO_RAISE_LOWER,
+	P_KEEP_ON_TOP,
+} dockPosition;
+
 int wClipMakeIconOmnipresent(WAppIcon *aicon, int omnipresent);
 
 #endif
