@@ -55,6 +55,7 @@
 #include "actions.h"
 #include "client.h"
 #include "main.h"
+#include "cycling.h"
 #include "keybind.h"
 #include "application.h"
 #include "stacking.h"
@@ -68,6 +69,12 @@
 #include "wmspec.h"
 #include "rootmenu.h"
 #include "colormap.h"
+#include "screen.h"
+#include "shutdown.h"
+#include "misc.h"
+#include "event.h"
+#include "winmenu.h"
+#include "switchmenu.h"
 
 /******** Global Variables **********/
 extern XContext wWinContext;
@@ -80,8 +87,6 @@ extern int wScreenCount;
 extern Time LastTimestamp;
 extern Time LastFocusChange;
 
-extern WPreferences wPreferences;
-
 #define MOD_MASK wPreferences.modifier_mask
 
 extern Atom _XA_WM_COLORMAP_NOTIFY;
@@ -104,14 +109,6 @@ extern int wShapeEventBase;
 extern int wXkbEventBase;
 #endif
 
-#ifdef HAVE_XRANDR
-extern Bool has_randr;
-extern int randr_event_base;
-#endif
-
-/* special flags */
-/*extern char WDelayedActionSet;*/
-
 /************ Local stuff ***********/
 
 static void saveTimestamp(XEvent *event);
@@ -144,9 +141,6 @@ static void handleShapeNotify(XEvent *event);
 static void handleXkbIndicatorStateNotify(XEvent *event);
 #endif
 
-/* called from the signal handler */
-void NotifyDeadProcess(pid_t pid, unsigned char status);
-
 /* real dead process handler */
 static void handleDeadProcess(void *foo);
 
@@ -301,11 +295,10 @@ void DispatchEvent(XEvent * event)
 		break;
 
 	case ConfigureNotify:
-		if (event->xconfigure.window == DefaultRootWindow(dpy)) {
 #ifdef HAVE_XRANDR
-		XRRUpdateConfiguration(event);
+		if (event->xconfigure.window == DefaultRootWindow(dpy))
+			XRRUpdateConfiguration(event);
 #endif
-		}
 		break;
 
 	default:
@@ -332,7 +325,6 @@ void DispatchEvent(XEvent * event)
 #define BUFF_SIZE ((sizeof(struct inotify_event) + 64)*5)
 static void handle_inotify_events(int fd, int wd)
 {
-	extern void wDefaultsCheckDomains(void *);
 	ssize_t eventQLength, i = 0;
 	char buff[BUFF_SIZE] = { 0 };
 	/* Check config only once per read of the event queue */
@@ -390,7 +382,7 @@ static void handle_inotify_events(int fd, int wd)
  *      Calls inotifyGetEvents if defaults database changes.
  *----------------------------------------------------------------------
  */
-void EventLoop(void)
+noreturn void EventLoop(void)
 {
 	XEvent event;
 #ifdef HAVE_INOTIFY
@@ -951,7 +943,7 @@ static void handleClientMessage(XEvent * event)
 		if (!wwin->flags.miniaturized)
 			wIconifyWindow(wwin);
 	} else if (event->xclient.message_type == _XA_WM_COLORMAP_NOTIFY && event->xclient.format == 32) {
-		WScreen *scr = wScreenSearchForRootWindow(event->xclient.window);
+		WScreen *scr = wScreenForRootWindow(event->xclient.window);
 
 		if (!scr)
 			return;
@@ -1041,7 +1033,7 @@ static void handleClientMessage(XEvent * event)
 			break;
 		}
 	} else if (event->xclient.message_type == _XA_WM_IGNORE_FOCUS_EVENTS) {
-		WScreen *scr = wScreenSearchForRootWindow(event->xclient.window);
+		WScreen *scr = wScreenForRootWindow(event->xclient.window);
 		if (!scr)
 			return;
 		scr->flags.ignore_focus_events = event->xclient.data.l[0] ? 1 : 0;
@@ -1476,6 +1468,48 @@ static void handleKeyPress(XEvent * event)
 			handleMaximize(wwin, MAX_VERTICAL | MAX_RIGHTHALF | MAX_KEYBOARD);
 		}
 		break;
+	case WKBD_THMAXIMIZE:
+		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && IS_RESIZABLE(wwin)) {
+			CloseWindowMenu(scr);
+
+			handleMaximize(wwin, MAX_HORIZONTAL | MAX_TOPHALF | MAX_KEYBOARD);
+		}
+		break;
+	case WKBD_BHMAXIMIZE:
+		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && IS_RESIZABLE(wwin)) {
+			CloseWindowMenu(scr);
+
+			handleMaximize(wwin, MAX_HORIZONTAL | MAX_BOTTOMHALF | MAX_KEYBOARD);
+		}
+		break;
+	case WKBD_LTCMAXIMIZE:
+		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && IS_RESIZABLE(wwin)) {
+			CloseWindowMenu(scr);
+
+			handleMaximize(wwin, MAX_LEFTHALF | MAX_TOPHALF | MAX_KEYBOARD);
+		}
+		break;
+	case WKBD_RTCMAXIMIZE:
+		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && IS_RESIZABLE(wwin)) {
+			CloseWindowMenu(scr);
+
+			handleMaximize(wwin, MAX_RIGHTHALF | MAX_TOPHALF | MAX_KEYBOARD);
+		}
+		break;
+	case WKBD_LBCMAXIMIZE:
+		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && IS_RESIZABLE(wwin)) {
+			CloseWindowMenu(scr);
+
+			handleMaximize(wwin, MAX_LEFTHALF | MAX_BOTTOMHALF | MAX_KEYBOARD);
+		}
+		 break;
+	case WKBD_RBCMAXIMIZE:
+		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && IS_RESIZABLE(wwin)) {
+			CloseWindowMenu(scr);
+
+			handleMaximize(wwin, MAX_RIGHTHALF | MAX_BOTTOMHALF | MAX_KEYBOARD);
+		}
+		break;
 	case WKBD_MAXIMUS:
 		if (ISMAPPED(wwin) && ISFOCUSED(wwin) && IS_RESIZABLE(wwin)) {
 			CloseWindowMenu(scr);
@@ -1561,6 +1595,49 @@ static void handleKeyPress(XEvent * event)
 	case WKBD_PREVWORKSPACE:
 		wWorkspaceRelativeChange(scr, -1);
 		break;
+	case WKBD_LASTWORKSPACE:
+		wWorkspaceChange(scr, scr->last_workspace);
+		break;
+
+	case WKBD_MOVE_WORKSPACE1 ... WKBD_MOVE_WORKSPACE10:
+		widx = command - WKBD_MOVE_WORKSPACE1;
+		i = (scr->current_workspace / 10) * 10 + widx;
+		if (wwin && (wPreferences.ws_advance || i < scr->workspace_count))
+			wWindowChangeWorkspace(wwin, i);
+		break;
+
+	case WKBD_MOVE_NEXTWORKSPACE:
+		if (wwin)
+			wWindowChangeWorkspaceRelative(wwin, 1);
+		break;
+	case WKBD_MOVE_PREVWORKSPACE:
+		if (wwin)
+			wWindowChangeWorkspaceRelative(wwin, -1);
+		break;
+	case WKBD_MOVE_LASTWORKSPACE:
+		if (wwin)
+			wWindowChangeWorkspace(wwin, scr->last_workspace);
+		break;
+
+	case WKBD_MOVE_NEXTWSLAYER:
+	case WKBD_MOVE_PREVWSLAYER:
+		{
+			if (wwin) {
+				int row, column;
+
+				row = scr->current_workspace / 10;
+				column = scr->current_workspace % 10;
+
+				if (command == WKBD_MOVE_NEXTWSLAYER) {
+					if ((row + 1) * 10 < scr->workspace_count)
+						wWindowChangeWorkspace(wwin, column + (row + 1) * 10);
+				} else {
+					if (row > 0)
+						wWindowChangeWorkspace(wwin, column + (row - 1) * 10);
+				}
+			}
+		}
+		break;
 
 	case WKBD_WINDOW1:
 	case WKBD_WINDOW2:
