@@ -38,15 +38,17 @@
 #include "window.h"
 #include "icon.h"
 #include "actions.h"
-#include "funcs.h"
 #include "stacking.h"
 #include "application.h"
 #include "defaults.h"
 #include "appicon.h"
 #include "wmspec.h"
+#include "misc.h"
+#include "startup.h"
+#include "event.h"
+#include "winmenu.h"
 
 /**** Global varianebles ****/
-extern WPreferences wPreferences;
 
 #define MOD_MASK wPreferences.modifier_mask
 #define CACHE_ICON_PATH "/Library/WindowMaker/CachedPixmaps"
@@ -93,7 +95,7 @@ static void tileObserver(void *self, WMNotification *notif)
 {
 	WIcon *icon = (WIcon *) self;
 
-	wIconUpdate(icon, NULL);
+	update_icon_pixmap(icon);
 
 	XClearArea(dpy, icon->core->window, 0, 0, 1, 1, True);
 }
@@ -113,7 +115,6 @@ WIcon *icon_create_for_wwindow(WWindow *wwin)
 {
 	WScreen *scr = wwin->screen_ptr;
 	WIcon *icon;
-	char *file;
 
 	icon = icon_create_core(scr, wwin->icon_x, wwin->icon_y);
 
@@ -135,23 +136,12 @@ WIcon *icon_create_for_wwindow(WWindow *wwin)
 	icon->show_title = 1;
 #endif
 
-	icon->icon_name = wNETWMGetIconName(wwin->client_win);
-	if (icon->icon_name)
-		wwin->flags.net_has_icon_title = 1;
-	else
-		wGetIconName(dpy, wwin->client_win, &icon->icon_name);
-
-	/* Get the application icon, default included */
-	file = get_icon_filename(scr, wwin->wm_instance, wwin->wm_class, NULL, True);
-	if (file) {
-		icon->file = wstrdup(file);
-		icon->file_image = get_rimage_from_file(scr, icon->file, wPreferences.icon_size);
-		wfree(file);
-	}
-
+	wIconChangeTitle(icon, wwin);
 	icon->tile_type = TILE_NORMAL;
 
-	wIconUpdate(icon, NULL);
+	set_icon_image_from_database(icon, wwin->wm_instance, wwin->wm_class, NULL);
+	/* Update the icon, because icon could be NULL */
+	wIconUpdate(icon);
 
 	WMAddNotificationObserver(appearanceObserver, icon, WNIconAppearanceSettingsChanged, icon);
 	WMAddNotificationObserver(tileObserver, icon, WNIconTileSettingsChanged, icon);
@@ -159,24 +149,16 @@ WIcon *icon_create_for_wwindow(WWindow *wwin)
 	return icon;
 }
 
-WIcon *icon_create_for_dock(WScreen *scr, char *command, char *wm_instance, char *wm_class, int tile)
+WIcon *icon_create_for_dock(WScreen *scr, const char *command, const char *wm_instance, const char *wm_class, int tile)
 {
 	WIcon *icon;
-	char *file = NULL;
 
 	icon = icon_create_core(scr, 0, 0);
-
-	/* Search the icon using instance and class, without default icon */
-	file = get_icon_filename(scr, wm_instance, wm_class, command, False);
-	if (file) {
-		icon->file = wstrdup(file);
-		icon->file_image = get_rimage_from_file(scr, icon->file, wPreferences.icon_size);
-		wfree(file);
-	}
-
 	icon->tile_type = tile;
 
-	wIconUpdate(icon, NULL);
+	set_icon_image_from_database(icon, wm_instance, wm_class, command);
+	/* Update the icon, because icon could be NULL */
+	wIconUpdate(icon);
 
 	WMAddNotificationObserver(appearanceObserver, icon, WNIconAppearanceSettingsChanged, icon);
 	WMAddNotificationObserver(tileObserver, icon, WNIconTileSettingsChanged, icon);
@@ -187,8 +169,6 @@ WIcon *icon_create_for_dock(WScreen *scr, char *command, char *wm_instance, char
 static WIcon *icon_create_core(WScreen *scr, int coord_x, int coord_y)
 {
 	WIcon *icon;
-	unsigned long vmask = 0;
-	XSetWindowAttributes attribs;
 
 	icon = wmalloc(sizeof(WIcon));
 	icon->core = wCoreCreateTopLevel(scr,
@@ -196,18 +176,8 @@ static WIcon *icon_create_core(WScreen *scr, int coord_x, int coord_y)
 					 coord_y,
 					 wPreferences.icon_size,
 					 wPreferences.icon_size,
-					 0, scr->w_depth, scr->w_visual, scr->w_colormap);
-
-	if (wPreferences.use_saveunders) {
-		vmask = CWSaveUnder;
-		attribs.save_under = True;
-	}
-
-	/* a white border for selecting it */
-	vmask |= CWBorderPixel;
-	attribs.border_pixel = scr->white_pixel;
-
-	XChangeWindowAttributes(dpy, icon->core->window, vmask, &attribs);
+					 0, scr->w_depth, scr->w_visual, scr->w_colormap,
+					 scr->white_pixel);
 
 	/* will be overriden if this is a application icon */
 	icon->core->descriptor.handle_mousedown = miniwindowMouseDown;
@@ -260,7 +230,7 @@ void wIconDestroy(WIcon * icon)
 	wfree(icon);
 }
 
-static void drawIconTitle(WScreen * scr, Pixmap pixmap, int height)
+static void drawIconTitleBackground(WScreen *scr, Pixmap pixmap, int height)
 {
 	XFillRectangle(dpy, pixmap, scr->icon_title_texture->normal_gc, 0, 0, wPreferences.icon_size, height + 1);
 	XDrawLine(dpy, pixmap, scr->icon_title_texture->light_gc, 0, 0, wPreferences.icon_size, 0);
@@ -277,13 +247,25 @@ static void icon_update_pixmap(WIcon *icon, RImage *image)
 	unsigned w, h;
 	int theight = 0;
 	WScreen *scr = icon->core->screen_ptr;
-	int titled = icon->show_title;
 
-	if (icon->tile_type == TILE_NORMAL) {
+	switch (icon->tile_type) {
+	case TILE_NORMAL:
 		tile = RCloneImage(scr->icon_tile);
-	} else {
-		assert(scr->clip_tile);
+		break;
+	case TILE_CLIP:
 		tile = RCloneImage(scr->clip_tile);
+		break;
+	case TILE_DRAWER:
+		tile = RCloneImage(scr->drawer_tile);
+		break;
+	default:
+		/*
+		 * The icon has always rigth value, this case is
+		 * only to avoid a compiler warning with "tile"
+		 * "may be used uninitialized"
+		 */
+		wwarning("Unknown tile type: %d.\n", icon->tile_type);
+		tile = RCloneImage(scr->icon_tile);
 	}
 
 	if (image) {
@@ -292,7 +274,7 @@ static void icon_update_pixmap(WIcon *icon, RImage *image)
 		x = (wPreferences.icon_size - w) / 2;
 		sx = (image->width - w) / 2;
 
-		if (titled)
+		if (icon->show_title)
 			theight = WMFontHeight(scr->icon_title_font);
 
 		h = (image->height + theight > wPreferences.icon_size
@@ -326,28 +308,26 @@ static void icon_update_pixmap(WIcon *icon, RImage *image)
 
 	RReleaseImage(tile);
 
-	if (titled)
-		drawIconTitle(scr, pixmap, theight);
+	/* Draw the icon's title background (without text) */
+	if (icon->show_title)
+		drawIconTitleBackground(scr, pixmap, theight);
 
 	icon->pixmap = pixmap;
 }
 
-void wIconChangeTitle(WIcon *icon, char *new_title)
+void wIconChangeTitle(WIcon *icon, WWindow *wwin)
 {
-	int changed;
-
-	changed = (new_title == NULL && icon->icon_name != NULL) ||
-		  (new_title != NULL && icon->icon_name == NULL);
+	if (!icon || !wwin)
+		return;
 
+	/* Remove the previous icon title */
 	if (icon->icon_name != NULL)
 		XFree(icon->icon_name);
 
-	icon->icon_name = new_title;
-
-	if (changed)
-		wIconUpdate(icon, NULL);
-	else
-		wIconPaint(icon);
+	/* Set the new one, using two methods */
+	icon->icon_name = wNETWMGetIconName(wwin->client_win);
+	if (!icon->icon_name)
+		wGetIconName(dpy, wwin->client_win, &icon->icon_name);
 }
 
 RImage *wIconValidateIconSize(RImage *icon, int max_size)
@@ -373,7 +353,7 @@ RImage *wIconValidateIconSize(RImage *icon, int max_size)
 	return icon;
 }
 
-Bool wIconChangeImageFile(WIcon *icon, char *file)
+Bool wIconChangeImageFile(WIcon *icon, const char *file)
 {
 	WScreen *scr = icon->core->screen_ptr;
 	char *path;
@@ -393,12 +373,10 @@ Bool wIconChangeImageFile(WIcon *icon, char *file)
 
 	/* New image! */
 	if (!error && image) {
-		/* Remove the old one */
-		unset_icon_image(icon);
-
 		/* Set the new image */
+		set_icon_image_from_image(icon, image);
 		icon->file = wstrdup(path);
-		wIconUpdate(icon, image);
+		update_icon_pixmap(icon);
 	} else {
 		error = 1;
 	}
@@ -414,7 +392,7 @@ static char *get_name_for_wwin(WWindow *wwin)
 	return get_name_for_instance_class(wwin->wm_instance, wwin->wm_class);
 }
 
-char *get_name_for_instance_class(char *wm_instance, char *wm_class)
+char *get_name_for_instance_class(const char *wm_instance, const char *wm_class)
 {
 	char *suffix;
 	int len;
@@ -440,7 +418,8 @@ char *get_name_for_instance_class(char *wm_instance, char *wm_class)
 
 static char *get_icon_cache_path(void)
 {
-	char *prefix, *path;
+	const char *prefix;
+	char *path;
 	int len, ret;
 
 	prefix = wusergnusteppath();
@@ -595,35 +574,42 @@ static void unset_icon_image(WIcon *icon)
 	}
 }
 
-void wIconUpdate(WIcon *icon, RImage *image)
+void set_icon_image_from_image(WIcon *icon, RImage *image)
+{
+	if (!icon)
+		return;
+
+	unset_icon_image(icon);
+
+	icon->file_image = NULL;
+	icon->file_image = image;
+}
+
+void wIconUpdate(WIcon *icon)
 {
 	WWindow *wwin = NULL;
 
-	if (image) {
-		icon->file_image = image;
-	} else {
-		if (icon && icon->owner)
-			wwin = icon->owner;
+	if (icon && icon->owner)
+		wwin = icon->owner;
 
-		if (wwin && WFLAGP(wwin, always_user_icon)) {
-			/* Forced use user_icon */
+	if (wwin && WFLAGP(wwin, always_user_icon)) {
+		/* Forced use user_icon */
+		get_rimage_icon_from_user_icon(icon);
+	} else if (icon->icon_win != None) {
+		/* Get the Pixmap from the WIcon */
+		get_rimage_icon_from_icon_win(icon);
+	} else if (wwin && wwin->net_icon_image) {
+		/* Use _NET_WM_ICON icon */
+		get_rimage_icon_from_x11(icon);
+	} else if (wwin && wwin->wm_hints && (wwin->wm_hints->flags & IconPixmapHint)) {
+		/* Get the Pixmap from the wm_hints, else, from the user */
+		unset_icon_image(icon);
+		icon->file_image = get_rimage_icon_from_wm_hints(icon);
+		if (!icon->file_image)
 			get_rimage_icon_from_user_icon(icon);
-		} else if (icon->icon_win != None) {
-			/* Get the Pixmap from the WIcon */
-			get_rimage_icon_from_icon_win(icon);
-		} else if (wwin && wwin->net_icon_image) {
-			/* Use _NET_WM_ICON icon */
-			get_rimage_icon_from_x11(icon);
-		} else if (wwin && wwin->wm_hints && (wwin->wm_hints->flags & IconPixmapHint)) {
-			/* Get the Pixmap from the wm_hints, else, from the user */
-			unset_icon_image(icon);
-			icon->file_image = get_rimage_icon_from_wm_hints(icon);
-			if (!icon->file_image)
-				get_rimage_icon_from_user_icon(icon);
-		} else {
-			/* Get the Pixmap from the user */
-			get_rimage_icon_from_user_icon(icon);
-		}
+	} else {
+		/* Get the Pixmap from the user */
+		get_rimage_icon_from_user_icon(icon);
 	}
 
 	update_icon_pixmap(icon);
@@ -652,7 +638,6 @@ void update_icon_pixmap(WIcon *icon)
 		XSetWindowBackgroundPixmap(dpy, icon->core->window, icon->pixmap);
 
 	/* Paint it */
-	XClearWindow(dpy, icon->core->window);
 	wIconPaint(icon);
 }
 
@@ -708,9 +693,7 @@ static void set_dockapp_in_icon(WIcon *icon)
 {
 	XWindowAttributes attr;
 	WScreen *scr = icon->core->screen_ptr;
-	int title_height = WMFontHeight(scr->icon_title_font);
 	unsigned int w, h, d;
-	int theight = 0;
 
 	/* Reparent the dock application to the icon */
 
@@ -718,13 +701,8 @@ static void set_dockapp_in_icon(WIcon *icon)
 	 * and show in the correct position */
 	getSize(icon->icon_win, &w, &h, &d);
 
-	/* Set extra space for title */
-	if (icon->show_title && (h + title_height < wPreferences.icon_size)) {
-		theight = title_height;
-		drawIconTitle(scr, icon->pixmap, theight);
-	} else {
-                XSetWindowBackgroundPixmap(dpy, icon->core->window, scr->icon_tile_pixmap);
-        }
+	/* Set the background pixmap */
+	XSetWindowBackgroundPixmap(dpy, icon->core->window, scr->icon_tile_pixmap);
 
 	/* Set the icon border */
 	XSetWindowBorderWidth(dpy, icon->icon_win, 0);
@@ -732,7 +710,7 @@ static void set_dockapp_in_icon(WIcon *icon)
 	/* Put the dock application in the icon */
 	XReparentWindow(dpy, icon->icon_win, icon->core->window,
 			(wPreferences.icon_size - w) / 2,
-			theight + (wPreferences.icon_size - h - theight) / 2);
+			(wPreferences.icon_size - h) / 2);
 
 	/* Show it and save */
 	XMapWindow(dpy, icon->icon_win);
@@ -768,14 +746,13 @@ RImage *get_rimage_icon_from_wm_hints(WIcon *icon)
 	return image;
 }
 
-void wIconPaint(WIcon *icon)
+/* This function updates in the screen the icon title */
+static void update_icon_title(WIcon *icon)
 {
 	WScreen *scr = icon->core->screen_ptr;
 	int x, l, w;
 	char *tmp;
 
-	XClearWindow(dpy, icon->core->window);
-
 	/* draw the icon title */
 	if (icon->show_title && icon->icon_name != NULL) {
 		tmp = ShrinkString(scr->icon_title_font, icon->icon_name, wPreferences.icon_size - 4);
@@ -790,6 +767,19 @@ void wIconPaint(WIcon *icon)
 			     scr->icon_title_font, x, 1, tmp, l);
 		wfree(tmp);
 	}
+}
+
+
+void wIconPaint(WIcon *icon)
+{
+	if (!icon || !icon->core || !icon->core->screen_ptr)
+		return;
+
+	WScreen *scr = icon->core->screen_ptr;
+
+	XClearWindow(dpy, icon->core->window);
+
+	update_icon_title(icon);
 
 	if (icon->selected)
 		XDrawRectangle(dpy, icon->core->window, scr->icon_select_gc, 0, 0,
@@ -908,3 +898,15 @@ static void miniwindowMouseDown(WObjDescriptor * desc, XEvent * event)
 		}
 	}
 }
+
+void set_icon_image_from_database(WIcon *icon, const char *wm_instance, const char *wm_class, const char *command)
+{
+	char *file = NULL;
+
+	file = get_icon_filename(icon->core->screen_ptr, wm_instance, wm_class, command, False);
+	if (file) {
+		icon->file = wstrdup(file);
+		icon->file_image = get_rimage_from_file(icon->core->screen_ptr, icon->file, wPreferences.icon_size);
+		wfree(file);
+	}
+}
