@@ -45,12 +45,17 @@
 
 #include "WindowMaker.h"
 #include "window.h"
-#include "funcs.h"
+#include "defaults.h"
+#include "event.h"
+#include "startup.h"
 #include "menu.h"
 #include "keybind.h"
 #include "xmodifier.h"
 #include "session.h"
+#include "shutdown.h"
 #include "dialog.h"
+#include "main.h"
+#include "monitor.h"
 
 #include <WINGs/WUtil.h>
 
@@ -77,7 +82,7 @@ char *Locale = NULL;
 
 int wScreenCount = 0;
 
-WPreferences wPreferences;
+struct WPreferences wPreferences;
 
 WShortKey wKeyBindings[WKBD_LAST];
 
@@ -139,31 +144,26 @@ Bool wXkbSupported;
 int wXkbEventBase;
 #endif
 
-#ifdef HAVE_XRANDR
-Bool has_randr;
-int randr_event_base;
-#endif
-
 /* special flags */
 char WProgramSigState = 0;
 char WProgramState = WSTATE_NORMAL;
 char WDelayedActionSet = 0;
 
 /* notifications */
-const char *WMNManaged = "WMNManaged";
-const char *WMNUnmanaged = "WMNUnmanaged";
-const char *WMNChangedWorkspace = "WMNChangedWorkspace";
-const char *WMNChangedState = "WMNChangedState";
-const char *WMNChangedFocus = "WMNChangedFocus";
-const char *WMNChangedStacking = "WMNChangedStacking";
-const char *WMNChangedName = "WMNChangedName";
+const char WMNManaged[] = "WMNManaged";
+const char WMNUnmanaged[] = "WMNUnmanaged";
+const char WMNChangedWorkspace[] = "WMNChangedWorkspace";
+const char WMNChangedState[] = "WMNChangedState";
+const char WMNChangedFocus[] = "WMNChangedFocus";
+const char WMNChangedStacking[] = "WMNChangedStacking";
+const char WMNChangedName[] = "WMNChangedName";
 
-const char *WMNWorkspaceCreated = "WMNWorkspaceCreated";
-const char *WMNWorkspaceDestroyed = "WMNWorkspaceDestroyed";
-const char *WMNWorkspaceChanged = "WMNWorkspaceChanged";
-const char *WMNWorkspaceNameChanged = "WMNWorkspaceNameChanged";
+const char WMNWorkspaceCreated[] = "WMNWorkspaceCreated";
+const char WMNWorkspaceDestroyed[] = "WMNWorkspaceDestroyed";
+const char WMNWorkspaceChanged[] = "WMNWorkspaceChanged";
+const char WMNWorkspaceNameChanged[] = "WMNWorkspaceNameChanged";
 
-const char *WMNResetStacking = "WMNResetStacking";
+const char WMNResetStacking[] = "WMNResetStacking";
 
 /******** End Global Variables *****/
 
@@ -173,10 +173,6 @@ static char **Arguments;
 
 static int ArgCount;
 
-extern void EventLoop(void);
-extern void StartUp(Bool defaultScreenOnly);
-extern int MonitorLoop(int argc, char **argv);
-
 static Bool multiHead = True;
 
 static int *wVisualID = NULL;
@@ -273,7 +269,7 @@ static int initWVisualID(const char *user_str)
 	return 0;
 }
 
-void Exit(int status)
+noreturn void Exit(int status)
 {
 	if (dpy)
 		XCloseDisplay(dpy);
@@ -357,7 +353,7 @@ static void shellCommandHandler(pid_t pid, unsigned char status, _tuple * data)
 	wfree(data);
 }
 
-void ExecuteShellCommand(WScreen * scr, char *command)
+void ExecuteShellCommand(WScreen *scr, const char *command)
 {
 	static char *shell = NULL;
 	pid_t pid;
@@ -468,7 +464,7 @@ Bool RelaunchWindow(WWindow *wwin)
  *
  *----------------------------------------------------------------------
  */
-void wAbort(Bool dumpCore)
+noreturn void wAbort(Bool dumpCore)
 {
 	int i;
 	WScreen *scr;
@@ -603,7 +599,7 @@ int main(int argc, char **argv)
 	/* setup common stuff for the monitor and wmaker itself */
 	WMInitializeApplication("WindowMaker", &argc, argv);
 
-	memset(&wPreferences, 0, sizeof(WPreferences));
+	memset(&wPreferences, 0, sizeof(wPreferences));
 
 	wPreferences.fallbackWMs = WMCreateArray(8);
 	alt = getenv("WINDOWMAKER_ALT_WM");
@@ -690,8 +686,11 @@ static int real_main(int argc, char **argv)
 				wPreferences.flags.norestore = 1;
 			} else if (strcmp(argv[i], "-nodock") == 0 || strcmp(argv[i], "--no-dock") == 0) {
 				wPreferences.flags.nodock = 1;
+				wPreferences.flags.nodrawer = 1;
 			} else if (strcmp(argv[i], "-noclip") == 0 || strcmp(argv[i], "--no-clip") == 0) {
 				wPreferences.flags.noclip = 1;
+			} else if (strcmp(argv[i], "-nodrawer") == 0 || strcmp(argv[i], "--no-drawer") == 0) {
+				wPreferences.flags.nodrawer = 1;
 			} else if (strcmp(argv[i], "-version") == 0 || strcmp(argv[i], "--version") == 0) {
 				printf("Window Maker %s\n", VERSION);
 				exit(0);
