@@ -37,12 +37,13 @@
 #include "framewin.h"
 #include "menu.h"
 #include "actions.h"
-#include "funcs.h"
+#include "winmenu.h"
 #include "stacking.h"
 #include "xinerama.h"
 #include "workspace.h"
 #include "dialog.h"
 #include "rootmenu.h"
+#include "switchmenu.h"
 
 /****** Global Variables ******/
 
@@ -50,15 +51,13 @@ extern Cursor wCursor[WCUR_LAST];
 
 extern XContext wWinContext;
 
-extern WPreferences wPreferences;
-
 #define MOD_MASK wPreferences.modifier_mask
 
 #define MENU_SCROLL_STEP  menuScrollParameters[(int)wPreferences.menu_scroll_speed].steps
 #define MENU_SCROLL_DELAY menuScrollParameters[(int)wPreferences.menu_scroll_speed].delay
 
-#define MENUW(m)	((m)->frame->core->width+2*FRAME_BORDER_WIDTH)
-#define MENUH(m)	((m)->frame->core->height+2*FRAME_BORDER_WIDTH)
+#define MENUW(m)	((m)->frame->core->width+2*(m)->frame->screen_ptr->frame_border_width)
+#define MENUH(m)	((m)->frame->core->height+2*(m)->frame->screen_ptr->frame_border_width)
 
 /***** Local Stuff ******/
 
@@ -84,7 +83,7 @@ static void menuTitleMouseDown(WCoreWindow * sender, void *data, XEvent * event)
 static void menuCloseClick(WCoreWindow * sender, void *data, XEvent * event);
 static void updateTexture(WMenu * menu);
 static int saveMenuRecurs(WMPropList * menus, WScreen * scr, WMenu * menu);
-static int restoreMenuRecurs(WScreen * scr, WMPropList * menus, WMenu * menu, char *path);
+static int restoreMenuRecurs(WScreen *scr, WMPropList *menus, WMenu *menu, const char *path);
 static void selectEntry(WMenu * menu, int entry_no);
 static void closeCascade(WMenu * menu);
 
@@ -139,7 +138,7 @@ static void appearanceObserver(void *self, WMNotification * notif)
  * 	The created menu.
  *----------------------------------------------------------------------
  */
-WMenu *wMenuCreate(WScreen * screen, char *title, int main_menu)
+WMenu *wMenuCreate(WScreen *screen, const char *title, int main_menu)
 {
 	WMenu *menu;
 	static int brother = 0;
@@ -229,7 +228,7 @@ WMenu *wMenuCreate(WScreen * screen, char *title, int main_menu)
 	return menu;
 }
 
-WMenu *wMenuCreateForApp(WScreen * screen, char *title, int main_menu)
+WMenu *wMenuCreateForApp(WScreen *screen, const char *title, int main_menu)
 {
 	WMenu *menu;
 
@@ -253,7 +252,7 @@ static void insertEntry(WMenu * menu, WMenuEntry * entry, int index)
 	menu->entries[index] = entry;
 }
 
-WMenuEntry *wMenuInsertCallback(WMenu * menu, int index, char *text,
+WMenuEntry *wMenuInsertCallback(WMenu *menu, int index, const char *text,
 				void (*callback) (WMenu * menu, WMenuEntry * entry), void *clientdata)
 {
 	WMenuEntry *entry;
@@ -1062,15 +1061,16 @@ static int keyboardMenu(WMenu * menu)
 
 void wMenuMapAt(WMenu * menu, int x, int y, int keyboard)
 {
-	WMRect rect = wGetRectForHead(menu->frame->screen_ptr,
-				      wGetHeadForPointerLocation(menu->frame->screen_ptr));
-
 	if (!menu->flags.realized) {
 		menu->flags.realized = 1;
 		wMenuRealize(menu);
 	}
+
 	if (!menu->flags.mapped) {
 		if (wPreferences.wrap_menus) {
+			WScreen *scr = menu->frame->screen_ptr;
+			WMRect rect = wGetRectForHead(scr, wGetHeadForPointerLocation(scr));
+
 			if (x < rect.pos.x)
 				x = rect.pos.x;
 			if (y < rect.pos.y)
@@ -1764,7 +1764,7 @@ static void menuMouseDown(WObjDescriptor * desc, XEvent * event)
 		if (!close_on_exit && (bev->state & ControlMask) && smenu && entry->flags.editable) {
 			char buffer[128];
 			char *name;
-			int number = entry_no - 2; /* Entries "New" and "Destroy Last" appear before workspaces */
+			int number = entry_no - 3; /* Entries "New", "Destroy Last" and "Last Used" appear before workspaces */
 
 			name = wstrdup(scr->workspaces[number]->name);
 			snprintf(buffer, sizeof(buffer), _("Type the name for workspace %i:"), number + 1);
@@ -2414,7 +2414,7 @@ static int restoreMenu(WScreen * scr, WMPropList * menu, int which)
 	return False;
 }
 
-static int restoreMenuRecurs(WScreen * scr, WMPropList * menus, WMenu * menu, char *path)
+static int restoreMenuRecurs(WScreen *scr, WMPropList *menus, WMenu *menu, const char *path)
 {
 	WMPropList *key, *entry;
 	char buffer[512];
