@@ -13,10 +13,9 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- *  USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 
@@ -34,7 +33,6 @@ int PropGetNormalHints(Window window, XSizeHints *size_hints, int *pre_iccm);
 void PropGetProtocols(Window window, WProtocols *prots);
 int PropGetWMClass(Window window, char **wm_class, char **wm_instance);
 int PropGetGNUstepWMAttr(Window window, GNUstepWMAttributes **attr);
-void PropWriteGNUstepWMAttr(Window window, GNUstepWMAttributes *attr);
 
 void PropSetWMakerProtocols(Window root);
 void PropCleanUp(Window root);
@@ -42,9 +40,4 @@ void PropSetIconTileHint(WScreen *scr, RImage *image);
 
 Window PropGetClientLeader(Window window);
 
-#ifdef XSMP_ENABLED
-char *PropGetClientID(Window window);
-#endif
-
-
 #endif
