@@ -41,7 +41,7 @@
 #include "WindowMaker.h"
 #include "actions.h"
 #include "menu.h"
-#include "funcs.h"
+#include "misc.h"
 #include "main.h"
 #include "dialog.h"
 #include "keybind.h"
@@ -50,7 +50,11 @@
 #include "defaults.h"
 #include "framewin.h"
 #include "session.h"
+#include "shutdown.h"
 #include "xmodifier.h"
+#include "rootmenu.h"
+#include "startup.h"
+#include "switchmenu.h"
 
 #include <WINGs/WUtil.h>
 
@@ -59,11 +63,11 @@
 extern char *Locale;
 extern WDDomain *WDRootMenu;
 extern Cursor wCursor[WCUR_LAST];
-extern WPreferences wPreferences;
 
 static WMenu *readMenuPipe(WScreen * scr, char **file_name);
-static WMenu *readMenuFile(WScreen * scr, char *file_name);
-static WMenu *readMenuDirectory(WScreen * scr, char *title, char **file_name, char *command);
+static WMenu *readPLMenuPipe(WScreen * scr, char **file_name);
+static WMenu *readMenuFile(WScreen *scr, const char *file_name);
+static WMenu *readMenuDirectory(WScreen *scr, const char *title, char **file_name, const char *command);
 static WMenu *configureMenu(WScreen * scr, WMPropList * definition, Bool includeGlobals);
 static void menu_parser_register_macros(WMenuParser parser);
 
@@ -118,6 +122,12 @@ static Shortcut *shortcutList = NULL;
  *                command must be a valid menu description.
  *                The space between '|' and command is optional.
  *                || will do the same, but will not cache the contents.
+ * OPEN_PLMENU | command
+ *		- opens command and uses its stdout which must be in proplist
+ *		  fromat to construct and insert the resulting menu in current
+ *		  position.
+ *		  The space between '|' and command is optional.
+ *		  || will do the same, but will not cache the contents.
  * SAVE_SESSION - saves the current state of the desktop, which include
  *		  all running applications, all their hints (geometry,
  *		  position on screen, workspace they live on, the dock
@@ -294,7 +304,7 @@ static void legalPanelCommand(WMenu * menu, WMenuEntry * entry)
 
 /********************************************************************/
 
-static char * getLocalizedMenuFile(char *menu)
+static char *getLocalizedMenuFile(const char *menu)
 {
 	char *buffer, *ptr, *locale;
 	int len;
@@ -411,7 +421,7 @@ static void removeShortcutsForMenu(WMenu * menu)
 	menu->menu->screen_ptr->flags.root_menu_changed_shortcuts = 1;
 }
 
-static Bool addShortcut(const char *file, char *shortcutDefinition, WMenu * menu, WMenuEntry * entry)
+static Bool addShortcut(const char *file, const char *shortcutDefinition, WMenu *menu, WMenuEntry *entry)
 {
 	Shortcut *ptr;
 	KeySym ksym;
@@ -553,7 +563,7 @@ static void separateCommand(char *line, char ***file, char **command)
 	WMFreeArray(array);
 }
 
-static WMenu *constructPLMenu(WScreen *screen, char *path)
+static WMenu *constructPLMenu(WScreen *screen, const char *path)
 {
 	WMPropList *pl = NULL;
 	WMenu *menu = NULL;
@@ -566,6 +576,9 @@ static WMenu *constructPLMenu(WScreen *screen, char *path)
 		return NULL;
 
 	menu = configureMenu(screen, pl, False);
+
+	WMReleasePropList(pl);
+
 	if (!menu)
 		return NULL;
 
@@ -573,6 +586,8 @@ static WMenu *constructPLMenu(WScreen *screen, char *path)
 	return menu;
 }
 
+
+
 static void constructMenu(WMenu * menu, WMenuEntry * entry)
 {
 	WMenu *submenu;
@@ -695,13 +710,60 @@ static void constructMenu(WMenu * menu, WMenuEntry * entry)
 		wfree(cmd);
 }
 
+static void constructPLMenuFromPipe(WMenu * menu, WMenuEntry * entry)
+{
+	WMenu *submenu = NULL;
+	char **path;
+	char *cmd;
+	int i;
+
+	separateCommand((char *)entry->clientdata, &path, &cmd);
+	if (path == NULL || *path == NULL || **path == 0) {
+		wwarning(_("invalid OPEN_PLMENU specification: %s"),
+		    (char *)entry->clientdata);
+		if (cmd)
+			wfree(cmd);
+		return;
+	}
+
+	if (path[0][0] == '|') {
+		/* pipe menu */
+
+		if (!menu->cascades[entry->cascade]
+		|| menu->cascades[entry->cascade]->timestamp == 0) {
+			/* parse pipe */
+			submenu = readPLMenuPipe(menu->frame->screen_ptr, path);
+
+			if (submenu != NULL) {
+				if (path[0][1] == '|')
+					submenu->timestamp = 0;
+				else
+					submenu->timestamp = 1;	/* there's no automatic reloading */
+			}
+		}
+	}
+
+	if (submenu) {
+		wMenuEntryRemoveCascade(menu, entry);
+		wMenuEntrySetCascade(menu, entry, submenu);
+	}
+
+	i = 0;
+	while (path[i] != NULL)
+		wfree(path[i++]);
+
+	wfree(path);
+	if (cmd)
+		wfree(cmd);
+
+}
 static void cleanupWorkspaceMenu(WMenu * menu)
 {
 	if (menu->frame->screen_ptr->workspace_menu == menu)
 		menu->frame->screen_ptr->workspace_menu = NULL;
 }
 
-static WMenuEntry *addWorkspaceMenu(WScreen * scr, WMenu * menu, char *title)
+static WMenuEntry *addWorkspaceMenu(WScreen *scr, WMenu *menu, const char *title)
 {
 	WMenu *wsmenu;
 	WMenuEntry *entry;
@@ -731,7 +793,7 @@ static void cleanupWindowsMenu(WMenu * menu)
 		menu->frame->screen_ptr->switch_menu = NULL;
 }
 
-static WMenuEntry *addWindowsMenu(WScreen * scr, WMenu * menu, char *title)
+static WMenuEntry *addWindowsMenu(WScreen *scr, WMenu *menu, const char *title)
 {
 	WMenu *wwmenu;
 	WWindow *wwin;
@@ -759,8 +821,8 @@ static WMenuEntry *addWindowsMenu(WScreen * scr, WMenu * menu, char *title)
 	return entry;
 }
 
-static WMenuEntry *addMenuEntry(WMenu * menu, char *title, char *shortcut, char *command,
-				char *params, const char *file_name)
+static WMenuEntry *addMenuEntry(WMenu *menu, const char *title, const char *shortcut, const char *command,
+				const char *params, const char *file_name)
 {
 	WScreen *scr;
 	WMenuEntry *entry = NULL;
@@ -786,6 +848,23 @@ static WMenuEntry *addMenuEntry(WMenu * menu, char *title, char *shortcut, char
 			entry->free_cdata = wfree;
 			wMenuEntrySetCascade(menu, entry, dummy);
 		}
+	} else if (strcmp(command, "OPEN_PLMENU") == 0) {
+		if (!params) {
+			wwarning(_("%s:missing parameter for menu command \"%s\""), file_name, command);
+		} else {
+			WMenu *dummy;
+			char *path;
+
+			path = wfindfile(DEF_CONFIG_PATHS, params);
+			if (!path)
+				path = wstrdup(params);
+
+			dummy = wMenuCreate(scr, title, False);
+			dummy->on_destroy = removeShortcutsForMenu;
+			entry = wMenuAddCallback(menu, title, constructPLMenuFromPipe, path);
+			entry->free_cdata = wfree;
+			wMenuEntrySetCascade(menu, entry, dummy);
+		}
 	} else if (strcmp(command, "EXEC") == 0) {
 		if (!params)
 			wwarning(_("%s:missing parameter for menu command \"%s\""), file_name, command);
@@ -933,7 +1012,7 @@ static WMenu *parseCascade(WScreen * scr, WMenu * menu, WMenuParser parser)
 	return NULL;
 }
 
-static WMenu *readMenuFile(WScreen * scr, char *file_name)
+static WMenu *readMenuFile(WScreen *scr, const char *file_name)
 {
 	WMenu *menu = NULL;
 	FILE *file = NULL;
@@ -979,6 +1058,37 @@ static WMenu *readMenuFile(WScreen * scr, char *file_name)
 }
 
 /************    Menu Configuration From Pipe      *************/
+static WMenu *readPLMenuPipe(WScreen * scr, char **file_name)
+{
+	WMPropList *plist = NULL;
+	WMenu *menu = NULL;
+	char *filename;
+	char flat_file[MAXLINE];
+	int i;
+
+	flat_file[0] = '\0';
+
+	for (i = 0; file_name[i] != NULL; i++) {
+		strcat(flat_file, file_name[i]);
+		strcat(flat_file, " ");
+	}
+	filename = flat_file + (flat_file[1] == '|' ? 2 : 1);
+
+	plist = WMReadPropListFromPipe(filename);
+
+	if (!plist)
+		return NULL;
+
+	menu = configureMenu(scr, plist, False);
+
+	WMReleasePropList(plist);
+
+	if (!menu)
+		return NULL;
+
+	menu->on_destroy = removeShortcutsForMenu;
+	return menu;
+}
 
 static WMenu *readMenuPipe(WScreen * scr, char **file_name)
 {
@@ -1080,7 +1190,7 @@ static void menu_parser_register_macros(WMenuParser parser)
 
 /************  Menu Configuration From Directory   *************/
 
-static Bool isFilePackage(char *file)
+static Bool isFilePackage(const char *file)
 {
 	int l;
 
@@ -1096,7 +1206,7 @@ static Bool isFilePackage(char *file)
 	}
 }
 
-static WMenu *readMenuDirectory(WScreen * scr, char *title, char **path, char *command)
+static WMenu *readMenuDirectory(WScreen *scr, const char *title, char **path, const char *command)
 {
 	DIR *dir;
 	struct dirent *dentry;
