@@ -45,7 +45,7 @@
 #include "texture.h"
 #include "pixmap.h"
 #include "menu.h"
-#include "funcs.h"
+#include "window.h"
 #include "main.h"
 #include "actions.h"
 #include "properties.h"
@@ -71,7 +71,6 @@
 
 /**** Global variables ****/
 extern Cursor wCursor[WCUR_LAST];
-extern WPreferences wPreferences;
 extern Atom _XA_WINDOWMAKER_STATE;
 extern Atom _XA_WINDOWMAKER_NOTICEBOARD;
 
@@ -80,8 +79,10 @@ extern int wScreenCount;
 #ifdef KEEP_XKB_LOCK_STATUS
 extern int wXkbSupported;
 #endif
+
 #ifdef HAVE_XRANDR
-extern int has_randr;
+Bool has_randr;
+int randr_event_base;
 #endif
 
 extern WDDomain *WDWindowMaker;
@@ -97,6 +98,7 @@ static WMPropList *dApplications = NULL;
 static WMPropList *dWorkspace;
 static WMPropList *dDock;
 static WMPropList *dClip;
+static WMPropList *dDrawers = NULL;
 
 static void make_keys(void)
 {
@@ -107,6 +109,7 @@ static void make_keys(void)
 	dWorkspace = WMCreatePLString("Workspace");
 	dDock = WMCreatePLString("Dock");
 	dClip = WMCreatePLString("Clip");
+	dDrawers = WMCreatePLString("Drawers");
 }
 
 /*
@@ -468,17 +471,6 @@ static void createInternalWindows(WScreen * scr)
 	    XCreateWindow(dpy, scr->root_win, 0, 0, wPreferences.icon_size,
 			  wPreferences.icon_size, 0, scr->w_depth, CopyFromParent, scr->w_visual, vmask, &attribs);
 
-	/* workspace name balloon for clip */
-	vmask = CWBackPixel | CWSaveUnder | CWOverrideRedirect | CWColormap | CWBorderPixel;
-	attribs.save_under = True;
-	attribs.override_redirect = True;
-	attribs.colormap = scr->w_colormap;
-	attribs.background_pixel = scr->icon_back_texture->normal.pixel;
-	attribs.border_pixel = 0;	/* do not care */
-	scr->clip_balloon =
-	    XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth,
-			  CopyFromParent, scr->w_visual, vmask, &attribs);
-
 	/* workspace name */
 	vmask = CWBackPixel | CWSaveUnder | CWOverrideRedirect | CWColormap | CWBorderPixel;
 	attribs.save_under = True;
@@ -489,12 +481,6 @@ static void createInternalWindows(WScreen * scr)
 	scr->workspace_name =
 	    XCreateWindow(dpy, scr->root_win, 0, 0, 10, 10, 0, scr->w_depth,
 			  CopyFromParent, scr->w_visual, vmask, &attribs);
-
-	/*
-	 * If the window is clicked without having ButtonPress selected, the
-	 * resulting event will have event.xbutton.window == root.
-	 */
-	XSelectInput(dpy, scr->clip_balloon, ButtonPressMask);
 }
 
 /*
@@ -656,13 +642,6 @@ WScreen *wScreenInit(int screen_number)
 	scr->light_pixel = WMColorPixel(scr->gray);
 	scr->dark_pixel = WMColorPixel(scr->darkGray);
 
-	{
-		XColor xcol;
-		/* frame boder color */
-		wGetColor(scr, FRAME_BORDER_COLOR, &xcol);
-		scr->frame_border_pixel = xcol.pixel;
-	}
-
 	/* create GCs with default values */
 	allocGCs(scr);
 
@@ -674,6 +653,15 @@ WScreen *wScreenInit(int screen_number)
 	/* read defaults for this screen */
 	wReadDefaults(scr, WDWindowMaker->dictionary);
 
+	{
+		XColor xcol;
+		/* frame boder color */
+		wGetColor(scr, WMGetColorRGBDescription(scr->frame_border_color), &xcol);
+		scr->frame_border_pixel = xcol.pixel;
+		wGetColor(scr, WMGetColorRGBDescription(scr->frame_selected_border_color), &xcol);
+		scr->frame_selected_border_pixel = xcol.pixel;
+	}
+
 	createInternalWindows(scr);
 
 	wNETWMInitStuff(scr);
@@ -860,12 +848,18 @@ void wScreenRestoreState(WScreen * scr)
 	if (!wPreferences.flags.nodock) {
 		state = WMGetFromPLDictionary(scr->session_state, dDock);
 		scr->dock = wDockRestoreState(scr, state, WM_DOCK);
+		/* If clip_merged_in_dock, setting scr->clip_icon is done by
+		 * wDockRestoreState()->wDockCreate()->mainIconCreate() */
 	}
 
 	if (!wPreferences.flags.noclip) {
 		state = WMGetFromPLDictionary(scr->session_state, dClip);
 		scr->clip_icon = wClipRestoreState(scr, state);
 	}
+	
+	if (!wPreferences.flags.nodrawer) {
+		wDrawersRestoreState(scr);
+	}
 
 	wWorkspaceRestoreState(scr);
 
@@ -913,6 +907,15 @@ void wScreenSaveState(WScreen * scr)
 
 	wWorkspaceSaveState(scr, old_state);
 
+	if (!wPreferences.flags.nodrawer) {
+		wDrawersSaveState(scr);
+	} else {
+		if ((foo = WMGetFromPLDictionary(old_state, dDrawers)) != NULL) {
+			WMPutInPLDictionary(scr->session_state, dDrawers, foo);
+		}
+	}
+
+
 	if (wPreferences.save_session_on_exit) {
 		wSessionSaveState(scr);
 	} else {
