@@ -35,6 +35,10 @@
 #define WTB_PFOCUSED	4
 #define WTB_MENU 6
 
+#ifdef HAVE_XRANDR
+extern Bool has_randr;
+extern int randr_event_base;
+#endif
 
 typedef struct {
     WMRect *screens;
@@ -58,6 +62,12 @@ typedef struct WAppIconChain {
 } WAppIconChain;
 
 
+/* Drawers, which are docks, really */
+typedef struct WDrawerChain {
+    struct WDock *adrawer;
+    struct WDrawerChain *next;
+} WDrawerChain;
+
 /*
  * each WScreen is saved into a context associated with it's root window
  */
@@ -116,6 +126,7 @@ typedef struct _WScreen {
     struct WWorkspace **workspaces;    /* workspace array */
 
     int current_workspace;	       /* current workspace number */
+    int last_workspace;		       /* last used workspace number */
 
 
     WReservedArea *reservedAreas;      /* used to build totalUsableArea */
@@ -163,8 +174,13 @@ typedef struct _WScreen {
     WMColor *mtext_color;	       /* menu item text */
     WMColor *dtext_color;	       /* disabled menu item text */
 
+    int frame_border_width;
+    WMColor *frame_border_color;
+    WMColor *frame_selected_border_color;
+
     WMPixel line_pixel;
     WMPixel frame_border_pixel;	       /* frame border */
+    WMPixel frame_selected_border_pixel;/* frame border */
 
 
     union WTexture *menu_title_texture[3];/* menu titlebar texture (tex, -, -) */
@@ -209,18 +225,22 @@ typedef struct _WScreen {
     struct WMenu *workspace_submenu;   /* workspace list for window_menu */
 
     struct WDock *dock;		       /* the application dock */
+    struct WMenu *dock_pos_menu;       /* Dock position menu */
     struct WPixmap *dock_dots;	       /* 3 dots for the Dock */
     Window dock_shadow;		       /* shadow for dock buttons */
-    struct WAppIcon *clip_icon;        /* The clip main icon */
+    struct WAppIcon *clip_icon;        /* The clip main icon, or the dock's, if they are merged */
     struct WMenu *clip_menu;           /* Menu for clips */
     struct WMenu *clip_submenu;        /* Workspace list for clips */
     struct WMenu *clip_options;	       /* Options for Clip */
     struct WMenu *clip_ws_menu;	       /* workspace menu for clip */
+    struct WMenu *drawer_menu;         /* Menu for drawers */
     struct WDock *last_dock;
     WAppIconChain *global_icons;       /* for omnipresent icons chain in clip */
     int global_icon_count;	       /* How many global icons do we have */
-
-    Window clip_balloon;	       /* window for workspace name */
+    WDrawerChain *drawers;             /* Chain of drawers */
+    /* Cache the following two informations, as they are used quite a lot */
+    int drawer_count;                  /* Nb of drawers that */
+    struct WDock *attracting_drawer;   /* The drawer that auto-attracts icons, or NULL */
 
     int keymove_tick;
     struct RContext *rcontext;	       /* wrlib context */
@@ -228,8 +248,8 @@ typedef struct _WScreen {
     WMScreen *wmscreen;		       /* for widget library */
 
     struct RImage *icon_tile;
-    struct RImage *clip_tile;
-
+    struct RImage *clip_tile;          /* tile with arrows to change workspace */
+    struct RImage *drawer_tile;        /* tile for a drawer (tile + arrow) */
     Pixmap icon_tile_pixmap;		/* For app supplied icons */
 
     struct RImage *def_icon_rimage;	/* Default RImage icon */
@@ -305,7 +325,6 @@ int wScreenKeepInside(WScreen *scr, int *x, int *y, int width, int height);
 /* in startup.c */
 WScreen *wScreenWithNumber(int i);
 WScreen *wScreenForRootWindow(Window window);   /* window must be valid */
-WScreen *wScreenSearchForRootWindow(Window window);
 WScreen *wScreenForWindow(Window window);   /* slower than above functions */
 
 void wScreenFinish(WScreen *scr);
