@@ -56,6 +56,7 @@
 #include "actions.h"
 #include "client.h"
 #include "main.h"
+#include "startup.h"
 #include "dock.h"
 #include "workspace.h"
 #include "keybind.h"
@@ -65,6 +66,8 @@
 #include "properties.h"
 #include "dialog.h"
 #include "wmspec.h"
+#include "event.h"
+#include "switchmenu.h"
 #ifdef XDND
 #include "xdnd.h"
 #endif
@@ -82,7 +85,6 @@
 #endif
 
 /****** Global Variables ******/
-extern WPreferences wPreferences;
 extern WDDomain *WDWindowMaker;
 extern WDDomain *WDRootMenu;
 extern WDDomain *WDWindowAttributes;
@@ -99,11 +101,6 @@ extern Bool wXkbSupported;
 extern int wXkbEventBase;
 #endif
 
-#ifdef HAVE_XRANDR
-extern Bool has_randr;
-extern int randr_event_base;
-#endif
-
 /* contexts */
 extern XContext wWinContext;
 extern XContext wAppWinContext;
@@ -141,8 +138,6 @@ extern Cursor wCursor[WCUR_LAST];
 extern char WDelayedActionSet;
 #endif
 
-extern void NotifyDeadProcess(pid_t pid, unsigned char status);
-
 /***** Local *****/
 static WScreen **wScreen = NULL;
 static unsigned int _NumLockMask = 0;
@@ -420,37 +415,12 @@ WScreen *wScreenForRootWindow(Window window)
 	if (wScreenCount == 1)
 		return wScreen[0];
 
-	/*
-	 * Since the number of heads will probably be small (normally 2),
+	/* Since the number of heads will probably be small (normally 2),
 	 * it should be faster to use this than a hash table, because
-	 * of the overhead.
-	 */
-	for (i = 0; i < wScreenCount; i++) {
-		if (wScreen[i]->root_win == window) {
+	 * of the overhead. */
+	for (i = 0; i < wScreenCount; i++)
+		if (wScreen[i]->root_win == window)
 			return wScreen[i];
-		}
-	}
-
-	return wScreenForWindow(window);
-}
-
-WScreen *wScreenSearchForRootWindow(Window window)
-{
-	int i;
-
-	if (wScreenCount == 1)
-		return wScreen[0];
-
-	/*
-	 * Since the number of heads will probably be small (normally 2),
-	 * it should be faster to use this than a hash table, because
-	 * of the overhead.
-	 */
-	for (i = 0; i < wScreenCount; i++) {
-		if (wScreen[i]->root_win == window) {
-			return wScreen[i];
-		}
-	}
 
 	return wScreenForWindow(window);
 }
@@ -462,9 +432,9 @@ WScreen *wScreenForWindow(Window window)
 	if (wScreenCount == 1)
 		return wScreen[0];
 
-	if (XGetWindowAttributes(dpy, window, &attr)) {
+	if (XGetWindowAttributes(dpy, window, &attr))
 		return wScreenForRootWindow(attr.root);
-	}
+
 	return NULL;
 }
 
@@ -512,7 +482,7 @@ void StartUp(Bool defaultScreenOnly)
 #ifdef HAVE_XRANDR
 	int dummy;
 #endif
-	Atom atom[sizeof(atomNames) / sizeof(char *)];
+	Atom atom[sizeof(atomNames) / sizeof(atomNames[0])];
 
 	/*
 	 * Ignore CapsLock in modifiers
@@ -535,12 +505,12 @@ void StartUp(Bool defaultScreenOnly)
 	/*    _XA_VERSION = XInternAtom(dpy, "VERSION", False); */
 
 #ifdef HAVE_XINTERNATOMS
-	XInternAtoms(dpy, atomNames, sizeof(atomNames) / sizeof(char *), False, atom);
+	XInternAtoms(dpy, atomNames, sizeof(atomNames) / sizeof(atomNames[0]), False, atom);
 #else
 
 	{
 		int i;
-		for (i = 0; i < sizeof(atomNames) / sizeof(char *); i++)
+		for (i = 0; i < sizeof(atomNames) / sizeof(atomNames[0]); i++)
 			atom[i] = XInternAtom(dpy, atomNames[i], False);
 	}
 #endif
@@ -777,6 +747,14 @@ void StartUp(Bool defaultScreenOnly)
 					}
 				}
 			}
+			/* auto-launch apps in drawers */
+			if (!wPreferences.flags.nodrawer) {
+				WDrawerChain *dc;
+				for (dc = wScreen[j]->drawers; dc; dc = dc->next) {
+					wScreen[j]->last_dock = dc->adrawer;
+					wDockDoAutoLaunch(dc->adrawer, 0);
+				}
+			}
 		}
 
 		/* go to workspace where we were before restart */
@@ -881,7 +859,7 @@ static void manageAllWindows(WScreen * scr, int crashRecovery)
 			if (crashRecovery) {
 				int border;
 
-				border = (!HAS_BORDER(wwin) ? 0 : FRAME_BORDER_WIDTH);
+				border = (!HAS_BORDER(wwin) ? 0 : scr->frame_border_width);
 
 				wWindowMove(wwin, wwin->frame_x - border,
 					    wwin->frame_y - border -
@@ -916,6 +894,7 @@ static void manageAllWindows(WScreen * scr, int crashRecovery)
 		WMNextEvent(dpy, &ev);
 		WMHandleEvent(&ev);
 	}
+	scr->last_workspace = 0;
 	wWorkspaceForceChange(scr, 0);
 	if (!wPreferences.flags.noclip)
 		wDockShowIcons(scr->workspaces[scr->current_workspace]->clip);
