@@ -72,15 +72,13 @@
 #include "WindowMaker.h"
 #include "menu.h"
 #include "actions.h"
-#include "funcs.h"
 #include "keybind.h"
+#include "xmodifier.h"
 
 #include "framewin.h"
 
 #define MAX_SHORTCUT_LENGTH 32
 
-/*** var ***/
-extern WPreferences wPreferences;
 
 typedef struct {
 	WScreen *screen;
@@ -318,7 +316,7 @@ void wUserMenuRefreshInstances(WMenu * menu, WWindow * wwin)
 		wMenuPaint(menu);
 }
 
-static WMenu *readUserMenuFile(WScreen * scr, char *file_name)
+static WMenu *readUserMenuFile(WScreen *scr, const char *file_name)
 {
 	WMenu *menu = NULL;
 	WMPropList *plum;
