@@ -36,7 +36,7 @@
 #include <math.h>
 
 /* For getting mouse wheel mappings from WINGs */
-#include <WINGs/WINGsP.h>
+#include <WINGs/WINGs.h>
 
 #include "WindowMaker.h"
 #include "GNUstep.h"
@@ -49,7 +49,6 @@
 #include "properties.h"
 #include "actions.h"
 #include "client.h"
-#include "funcs.h"
 #include "colormap.h"
 #include "keybind.h"
 #include "stacking.h"
@@ -60,6 +59,11 @@
 #include "appicon.h"
 #include "superfluous.h"
 #include "rootmenu.h"
+#include "placement.h"
+#include "misc.h"
+#include "startup.h"
+#include "winmenu.h"
+#include "osdep.h"
 
 #ifdef MWM_HINTS
 # include "motif.h"
@@ -82,7 +86,6 @@ extern XContext wWinContext;
 extern Atom _XA_WM_DELETE_WINDOW;
 extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
 extern Atom _XA_WINDOWMAKER_STATE;
-extern WPreferences wPreferences;
 extern Time LastTimestamp;
 
 /***** Local Stuff *****/
@@ -1320,7 +1323,7 @@ WWindow *wManageWindow(WScreen *scr, Window window)
 }
 
 WWindow *wManageInternalWindow(WScreen *scr, Window window, Window owner,
-			       char *title, int x, int y, int width, int height)
+			       const char *title, int x, int y, int width, int height)
 {
 	WWindow *wwin;
 	int foo;
@@ -1721,9 +1724,9 @@ void wWindowUnfocus(WWindow *wwin)
 	WMPostNotificationName(WMNChangedFocus, wwin, (void *)False);
 }
 
-void wWindowUpdateName(WWindow *wwin, char *newTitle)
+void wWindowUpdateName(WWindow *wwin, const char *newTitle)
 {
-	char *title;
+	const char *title;
 
 	if (!wwin->frame)
 		return;
@@ -1917,6 +1920,30 @@ void wWindowChangeWorkspace(WWindow *wwin, int workspace)
 		wWindowUnmap(wwin);
 }
 
+void wWindowChangeWorkspaceRelative(WWindow *wwin, int amount)
+{
+	WScreen *scr = wwin->screen_ptr;
+	int w = scr->current_workspace + amount;
+
+	if (amount < 0) {
+		if (w >= 0) {
+			wWindowChangeWorkspace(wwin, w);
+		} else if (wPreferences.ws_cycle) {
+			wWindowChangeWorkspace(wwin, scr->workspace_count + w);
+		}
+	} else if (amount > 0) {
+		if (w < scr->workspace_count) {
+			wWindowChangeWorkspace(wwin, w);
+		} else if (wPreferences.ws_advance) {
+			int workspace = WMIN(w, MAX_WORKSPACES - 1);
+			wWorkspaceMake(scr, workspace);
+			wWindowChangeWorkspace(wwin, workspace);
+		} else if (wPreferences.ws_cycle) {
+			wWindowChangeWorkspace(wwin, w % scr->workspace_count);
+		}
+	}
+}
+
 void wWindowSynthConfigureNotify(WWindow *wwin)
 {
 	XEvent sevent;
@@ -2021,8 +2048,8 @@ void wWindowConfigure(WWindow *wwin, int req_x, int req_y, int req_width, int re
 	wwin->frame_x = req_x;
 	wwin->frame_y = req_y;
 	if (HAS_BORDER(wwin)) {
-		wwin->client.x += FRAME_BORDER_WIDTH;
-		wwin->client.y += FRAME_BORDER_WIDTH;
+		wwin->client.x += wwin->screen_ptr->frame_border_width;
+		wwin->client.y += wwin->screen_ptr->frame_border_width;
 	}
 #ifdef SHAPE
 	if (wShapeSupported && wwin->flags.shaped && resize)
@@ -2058,8 +2085,8 @@ void wWindowMove(WWindow *wwin, int req_x, int req_y)
 	wwin->client.x = req_x;
 	wwin->client.y = req_y + wwin->frame->top_width;
 	if (HAS_BORDER(wwin)) {
-		wwin->client.x += FRAME_BORDER_WIDTH;
-		wwin->client.y += FRAME_BORDER_WIDTH;
+		wwin->client.x += wwin->screen_ptr->frame_border_width;
+		wwin->client.y += wwin->screen_ptr->frame_border_width;
 	}
 
 	XMoveWindow(dpy, wwin->frame->core->window, req_x, req_y);
@@ -2198,6 +2225,8 @@ void wWindowConfigureBorders(WWindow *wwin)
 			flags |= WFF_BORDER;
 		if (wwin->flags.shaded)
 			flags |= WFF_IS_SHADED;
+		if (wwin->flags.selected)
+			flags |= WFF_SELECTED;
 
 		oldh = wwin->frame->top_width;
 		wFrameWindowUpdateBorders(wwin->frame, flags);
@@ -2518,7 +2547,8 @@ void wWindowUpdateGNUstepAttr(WWindow * wwin, GNUstepWMAttributes * attr)
 	}
 }
 
-WMagicNumber wWindowAddSavedState(char *instance, char *class, char *command, pid_t pid, WSavedState * state)
+WMagicNumber wWindowAddSavedState(const char *instance, const char *class,
+											 const char *command, pid_t pid, WSavedState * state)
 {
 	WWindowState *wstate;
 
@@ -2735,9 +2765,9 @@ static void titlebarDblClick(WCoreWindow *sender, void *data, XEvent *event)
 			wHideOtherApplications(wwin);
 	} else if (event->xbutton.button == Button2) {
 		wSelectWindow(wwin, !wwin->flags.selected);
-	} else if (event->xbutton.button == WINGsConfiguration.mouseWheelUp) {
+	} else if (event->xbutton.button == W_getconf_mouseWheelUp()) {
 		wShadeWindow(wwin);
-	} else if (event->xbutton.button == WINGsConfiguration.mouseWheelDown) {
+	} else if (event->xbutton.button == W_getconf_mouseWheelDown()) {
 		wUnshadeWindow(wwin);
 	}
 }
