@@ -249,8 +249,8 @@ typedef struct WWindow {
 		unsigned int miniaturized:1;
 		unsigned int hidden:1;
 		unsigned int shaded:1;
-		unsigned int maximized:5;
-		unsigned int old_maximized:5;
+		unsigned int maximized:7;
+		unsigned int old_maximized:7;
 		unsigned int fullscreen:1;
 		unsigned int omnipresent:1;
 		unsigned int semi_focused:1;
@@ -282,7 +282,6 @@ typedef struct WWindow {
 		unsigned int net_handle_icon:1;
 		unsigned int net_show_desktop:1;
 		unsigned int net_has_title:1;	/* use netwm version of WM_NAME */
-		unsigned int net_has_icon_title:1;
 	} flags;				/* state of the window */
 
 	struct WIcon *icon;			/* Window icon when miminized
@@ -343,7 +342,7 @@ void wUnmanageWindow(WWindow *wwin, Bool restore, Bool destroyed);
 void wWindowFocus(WWindow *wwin, WWindow *owin);
 void wWindowUnfocus(WWindow *wwin);
 
-void wWindowUpdateName(WWindow *wwin, char *newTitle);
+void wWindowUpdateName(WWindow *wwin, const char *newTitle);
 void wWindowConstrainSize(WWindow *wwin, unsigned int *nwidth, unsigned int *nheight);
 void wWindowCropSize(WWindow *wwin, unsigned int maxw, unsigned int maxh,
                      unsigned int *nwidth, unsigned int *nheight);
@@ -364,13 +363,14 @@ void wWindowUpdateButtonImages(WWindow *wwin);
 void wWindowSaveState(WWindow *wwin);
 
 void wWindowChangeWorkspace(WWindow *wwin, int workspace);
+void wWindowChangeWorkspaceRelative(WWindow *wwin, int amount);
 
 void wWindowSetKeyGrabs(WWindow *wwin);
 
 void wWindowResetMouseGrabs(WWindow *wwin);
 
 WWindow *wManageInternalWindow(WScreen *scr, Window window, Window owner,
-                               char *title, int x, int y,
+                               const char *title, int x, int y,
                                int width, int height);
 
 void wWindowSetupInitialAttributes(WWindow *wwin, int *level, int *workspace);
@@ -383,7 +383,7 @@ void wWindowUnmap(WWindow *wwin);
 
 void wWindowDeleteSavedStatesForPID(pid_t pid);
 
-WMagicNumber wWindowAddSavedState(char *instance, char *class, char *command,
+WMagicNumber wWindowAddSavedState(const char *instance, const char *class, const char *command,
                                   pid_t pid, WSavedState *state);
 
 WMagicNumber wWindowGetSavedState(Window win);
