@@ -38,6 +38,7 @@
 #include "client.h"
 #include "application.h"
 #include "keybind.h"
+#include "misc.h"
 #include "framewin.h"
 #include "workspace.h"
 #include "winspector.h"
@@ -45,6 +46,7 @@
 #include "stacking.h"
 #include "icon.h"
 #include "xinerama.h"
+#include "winmenu.h"
 
 #define MC_MAXIMIZE	0
 #define MC_MINIATURIZE	1
@@ -73,7 +75,6 @@ extern Atom _XA_GNUSTEP_WM_MINIATURIZE_WINDOW;
 
 extern WShortKey wKeyBindings[WKBD_LAST];
 
-extern WPreferences wPreferences;
 
 static void updateOptionsMenu(WMenu * menu, WWindow * wwin);
 
@@ -219,50 +220,42 @@ static void updateWorkspaceMenu(WMenu * menu)
 {
 	WScreen *scr = menu->frame->screen_ptr;
 	char title[MAX_WORKSPACENAME_WIDTH + 1];
+	WMenuEntry *entry;
 	int i;
 
 	for (i = 0; i < scr->workspace_count; i++) {
 		if (i < menu->entry_no) {
-			if (strcmp(menu->entries[i]->text, scr->workspaces[i]->name) != 0) {
-				wfree(menu->entries[i]->text);
+
+			entry = menu->entries[i];
+			if (strcmp(entry->text, scr->workspaces[i]->name) != 0) {
+				wfree(entry->text);
 				strncpy(title, scr->workspaces[i]->name, MAX_WORKSPACENAME_WIDTH);
 				title[MAX_WORKSPACENAME_WIDTH] = 0;
 				menu->entries[i]->text = wstrdup(title);
+				menu->entries[i]->rtext = GetShortcutKey(wKeyBindings[WKBD_MOVE_WORKSPACE1 + i]);
 				menu->flags.realized = 0;
 			}
 		} else {
 			strncpy(title, scr->workspaces[i]->name, MAX_WORKSPACENAME_WIDTH);
 			title[MAX_WORKSPACENAME_WIDTH] = 0;
 
-			wMenuAddCallback(menu, title, switchWSCommand, NULL);
+			entry = wMenuAddCallback(menu, title, switchWSCommand, NULL);
+			entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MOVE_WORKSPACE1 + i]);
 
 			menu->flags.realized = 0;
 		}
+
+		/* workspace shortcut labels */
+		if (i / 10 == scr->current_workspace / 10)
+			entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MOVE_WORKSPACE1 + (i % 10)]);
+		else
+			entry->rtext = NULL;
 	}
 
 	if (!menu->flags.realized)
 		wMenuRealize(menu);
 }
 
-static char *getShortcutString(WShortKey key)
-{
-	char *tmp = NULL;
-	char *k = XKeysymToString(XkbKeycodeToKeysym(dpy, key.keycode, 0, 0));
-	if (!k) return NULL;
-
-	char **m = wPreferences.modifier_labels;
-	if (key.modifier & ControlMask) tmp = wstrappend(tmp, m[1] ? m[1] : "Ctrl+");
-	if (key.modifier & ShiftMask)   tmp = wstrappend(tmp, m[0] ? m[0] : "Shift+");
-	if (key.modifier & Mod1Mask)    tmp = wstrappend(tmp, m[2] ? m[2] : "Mod1+");
-	if (key.modifier & Mod2Mask)    tmp = wstrappend(tmp, m[3] ? m[3] : "Mod2+");
-	if (key.modifier & Mod3Mask)    tmp = wstrappend(tmp, m[4] ? m[4] : "Mod3+");
-	if (key.modifier & Mod4Mask)    tmp = wstrappend(tmp, m[5] ? m[5] : "Mod4+");
-	if (key.modifier & Mod5Mask)    tmp = wstrappend(tmp, m[6] ? m[6] : "Mod5+");
-	tmp = wstrappend(tmp, k);
-
-	return tmp;
-}
-
 static void updateMakeShortcutMenu(WMenu * menu, WWindow * wwin)
 {
 	WMenu *smenu = menu->cascades[menu->entries[MC_SHORTCUT]->cascade];
@@ -304,7 +297,7 @@ static void updateMakeShortcutMenu(WMenu * menu, WWindow * wwin)
 		kcode = wKeyBindings[WKBD_WINDOW1 + shortcutNo].keycode;
 
 		if (kcode) {
-			if ((tmp = getShortcutString(wKeyBindings[WKBD_WINDOW1 + shortcutNo]))
+			if ((tmp = GetShortcutKey(wKeyBindings[WKBD_WINDOW1 + shortcutNo]))
 			    && (!entry->rtext || strcmp(tmp, entry->rtext) != 0)) {
 				if (entry->rtext)
 					wfree(entry->rtext);
@@ -428,22 +421,22 @@ static WMenu *createWindowMenu(WScreen * scr)
 	 * this file.
 	 */
 	entry = wMenuAddCallback(menu, _("Maximize"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_MAXIMIZE]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MAXIMIZE]);
 
 	entry = wMenuAddCallback(menu, _("Miniaturize"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_MINIATURIZE]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MINIATURIZE]);
 
 	entry = wMenuAddCallback(menu, _("Shade"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_SHADE]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_SHADE]);
 
 	entry = wMenuAddCallback(menu, _("Hide"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_HIDE]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_HIDE]);
 
 	entry = wMenuAddCallback(menu, _("Resize/Move"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_MOVERESIZE]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_MOVERESIZE]);
 
 	entry = wMenuAddCallback(menu, _("Select"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_SELECT]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_SELECT]);
 
 	entry = wMenuAddCallback(menu, _("Move To"), NULL, NULL);
 	scr->workspace_submenu = makeWorkspaceMenu(scr);
@@ -461,10 +454,10 @@ static WMenu *createWindowMenu(WScreen * scr)
 	 */
 
 	entry = wMenuAddCallback(menu, _("Launch"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_RELAUNCH]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_RELAUNCH]);
 
 	entry = wMenuAddCallback(menu, _("Close"), execMenuCommand, NULL);
-	entry->rtext = getShortcutString(wKeyBindings[WKBD_CLOSE]);
+	entry->rtext = GetShortcutKey(wKeyBindings[WKBD_CLOSE]);
 
 	entry = wMenuAddCallback(menu, _("Kill"), execMenuCommand, NULL);
 
@@ -551,6 +544,20 @@ static void updateMenuForWindow(WMenu * menu, WWindow * wwin)
 	wMenuSetEnabled(menu, MC_SHADE, !WFLAGP(wwin, no_shadeable)
 			&& !wwin->flags.miniaturized);
 
+	if (wwin->flags.selected) {
+		static char *text = NULL;
+		if (!text)
+			text = _("Deselect");
+
+		menu->entries[MC_SELECT]->text = text;
+	} else {
+		static char *text = NULL;
+		if (!text)
+			text = _("Select");
+
+		menu->entries[MC_SELECT]->text = text;
+	}
+
 	wMenuSetEnabled(menu, MC_DUMMY_MOVETO, !IS_OMNIPRESENT(wwin));
 
 	if (!wwin->flags.inspector_open) {
@@ -591,6 +598,7 @@ static WMenu *open_window_menu_core(WWindow *wwin, int x, int y)
 		wfree(scr->window_menu->entries[MC_MINIATURIZE]->text);
 		wfree(scr->window_menu->entries[MC_MAXIMIZE]->text);
 		wfree(scr->window_menu->entries[MC_SHADE]->text);
+		wfree(scr->window_menu->entries[MC_SELECT]->text);
 	} else {
 		updateWorkspaceMenu(scr->workspace_submenu);
 	}
@@ -685,6 +693,7 @@ void DestroyWindowMenu(WScreen *scr)
 		scr->window_menu->entries[MC_MINIATURIZE]->text = NULL;
 		scr->window_menu->entries[MC_MAXIMIZE]->text = NULL;
 		scr->window_menu->entries[MC_SHADE]->text = NULL;
+		scr->window_menu->entries[MC_SELECT]->text = NULL;
 		wMenuDestroy(scr->window_menu, True);
 		scr->window_menu = NULL;
 	}
