@@ -12,16 +12,15 @@
  *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  *  GNU General Public License for more details.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
- *  USA.
+ *  You should have received a copy of the GNU General Public License along
+ *  with this program; if not, write to the Free Software Foundation, Inc.,
+ *  51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
  */
 
 #ifndef _XMODIFIER_H_INCLUDED
 #define _XMODIFIER_H_INCLUDED
 
 void wXModifierInitialize(void);
-int  wXModifierFromKey(char *key);
+int  wXModifierFromKey(const char *key);
 
 #endif /* _XMODIFIER_H_INCLUDED */
