@@ -15,7 +15,8 @@
  *
  *  You should have received a copy of the GNU General Public License
  *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston,
+ *  MA 02110-1301, USA.
  */
 
 #include <X11/Xlib.h>
@@ -167,7 +168,7 @@ void FormatXError(Display * dpy, XErrorEvent * error, char *buffer, int size)
 	if (i > size - 100)
 		return;
 	buffer += i;
-	if (error->request_code >= sizeof(requestCodes) / sizeof(char *)) {
+	if (error->request_code >= sizeof(requestCodes) / sizeof(requestCodes[0])) {
 		sprintf(buffer, "\n	Request code: %i\n", error->request_code);
 	} else {
 		sprintf(buffer, "\n	Request code: %i %s\n", error->request_code,
