@@ -34,7 +34,7 @@
 
 extern char *__progname;
 
-void print_help(int print_usage, int exitval)
+static void print_help(int print_usage, int exitval)
 {
 	printf("Usage: %s [-h] [-v] [file]\n", __progname);
 	if (print_usage) {
